package smail;

import smail.gui.* ;
import smail.localStore.* ;
import smail.utils.* ;
import javax.mail.* ;
import javax.mail.internet.* ;
import java.util.* ;
import java.io.* ;
import smail.server.* ;

/**
This class initializes the whole application. Furthermore it is responsible for the communication
between the application components (except GUI and Fassade).
Error code -55 indicates that the store-provider definition was not found.
*/
public class Control
{
	private Fassade fassade ;
	private Smail gui ;
	private LocalStore store ;
	private LocalFolder outBox ;
	private LocalFolder inBox ;
	private boolean expunge = true ;	
	private boolean d = false ;
	private PropertiesManager propMan ;
    private Account account;
	private AccountData accData ;
	private smail.filter.FilterManager filterManager ;
	private static smail.gui.Intro intro;
	private static Thread t;
	/**
	 *start-up routine that now handles commandline parameters.
	 *<ul>
	 *<li>-d : activates the debug mode for the Control-class</li>
	 *</ul>
	 */
	public static void main(String[] args)
	{
		boolean debug = false ; 
		//lets make a real start up routine ... :-)
		for (int i=0;i<args.length ;i++ )
		{
			if (0 == args[i].compareTo("-d"))
			{
				debug = true ; 
				System.out.println("activated debug mode for control") ;
			}
		}
		intro = new smail.gui.Intro() ;
		t = new Thread(intro);
        	t.start();
//
		Control ctrl = new Control(debug) ;
	}

	/**
	 *applies all manual filter currently stored in filtermanager on ALL messages 
	 *in all folder existing in localstore.
	 *@see smail.localStore.LocalStore
	 *@see smail.filter.FilterManager
	 *@see smail.filter.Filter
	 */
	public void applyManualFilter() {
		synchronized(filterManager) {
			try {
			
				smail.filter.Filter[] allFilter = filterManager.getAllFilter() ;
				Folder rootFolder = store.getDefaultFolder() ;
				Folder[] allFolder = rootFolder.list("*");
//				LocalFolder[] allLocalFolder = new LocalFolder[allFolder.length];
//				for (int i = 0 ; i<allFolder.length ; i++) {
//					allLocalFolder[i] = (LocalFolder)allFolder[i];				
//				}
				for (int i = 0 ; i<allFilter.length ; i++ ) {
					if (allFilter[i].getType() == smail.filter.Filter.MANUALLY) {
						for (int k = 0 ; k < allFolder.length ; k++) {
							try {
								filterManager.applyFilter( (LocalFolder)allFolder[k],	allFilter[i] );
							} catch (smail.filter.FilterNotActiveException exc) {
								// dont care , apply next filter 	
							}
						}
					}
				}
			} catch (MessagingException exc) {
				String[] msg =  {
					"There is a problem with the internal FileSystem.",
					"Sorry your mails can not be filtered !"
				};
				smail.Popup p = new smail.Popup(msg,true);
			} catch (NullPointerException exc) {
				String[] msg =  {
					"There is a problem with the internal FileSystem.",
					"Sorry your mails can not be filtered !"
				};
				smail.Popup p = new smail.Popup(msg,true);				
			} catch (ClassCastException ccexc) {
				System.out.print( "ClassCastException caught !!!" ) ;
				System.out.print( "message is :"+ccexc.getMessage() ) ;
			}
		}
	}
	 

	/**
	 *returns the FilterManager-reference.This should only be used by the "FilterGuy"
	 *to manage filter.
	 *@see smail.gui.filterGui.FilterGuy
	 *@see smail.filter.FilterManager
	 */
	smail.filter.FilterManager getFilterManager()
	{
		return this.filterManager ;
	}
	
	/**
	 *returns the programs localstore instance-reference.
	 *@see smail.gui.filterGui.FilterGuy
	 *@see smail.localStore.LocalStore
	 */
	Store getStore()
	{
		return this.store ;
	}

	/**
	 *getMail() returns the "gui fomatted" representation of a message 
	 *referenced by the given message-number and the "full-named" folder
	 *@param msgnum is the number of the message to open related to the given folder 
	 *@param folderName is the folder to get the message from
	 *@throws Exception when the given number does not exists in this folder
	 *@returns the referenced message in "gui format"
	 */
	MessageContent getMail(int msgnum, String folderName) throws Exception
	{
		javax.mail.internet.MimeMessage msg = null;
		try
		{	
			LocalFolder folder ;
			if (folderName.equals("OUTBOX"))
				folder = this.outBox ;
			else
				folder = (LocalFolder)store.getFolder(folderName) ;
			if (folder.exists() && folder.isOpen())	
			{				
				System.out.println("Folder geffnet");
/*DEBUG*/				System.out.println( "(Control.java:148) :: msg number ist :"+msgnum+" add one to fit mailAPI !!!" ) ;
				msg = (javax.mail.internet.MimeMessage)folder.getMessage(msgnum) ;
				folder.close(true) ;
			}
			else if (folder.exists() && !folder.isOpen())
			{
				folder.open(Folder.READ_ONLY) ;
				System.out.println("Folder geffnet");
				msg = (javax.mail.internet.MimeMessage)folder.getMessage(msgnum) ;
				folder.close(true) ;
			}	
		}
		catch(MessagingException mExc)
		{	
			if (d)
				System.out.println("got stuck here ") ;
			throw new Exception("No such Mail/Folder to open Mail from !::Control.getMail(int,String)") ;
		}
		
		if (d)
		{
			try
			{
				System.out.println("storeNewMail(msg,String)") ;
				msg.writeTo(System.out) ;
			}
			catch (IOException e)
			{
				System.out.println("IOException writing Message to System.out") ;
				System.out.println( e.getMessage()) ; 
			}			
		}

		//static converter or not ??
		
		return Converter.reConvert(msg) ;
	}
	
	/**
	 *Gets a "pseudo-message" from GUI and calls the converter to create 
	 *a legal message. This method stores the converted message in the OutBox
	 *by calling the method : storeNewMail(PseudoMessage pMsg, "OutBox") !
	 *@see #storeNewMail(MessageContent pMsg, String folderName)
	 */
	void storeNewMail(MessageContent pMsg) throws NonExistentFolderException
	{
		this.storeNewMail(pMsg,"OUTBOX") ;
	}
	
	/**
	 *This method calls the converter to convert the PseudoMessage and stores the converted message
	 *in the spezified Folder defined by folderName
	 *@param pMsg A message in "gui format" that has to be converted in javax.mail.Message format
	 *@see smail.gui.MessageContent
	 *@see javax.mail.Message javax.mail.internet.MimeMessage
	 */
	void storeNewMail(MessageContent pMsg, String folderName) 
							throws NonExistentFolderException
	{
		LocalMessage localMsg = null ; 
		
		try
		{
	         //convert a MessageContent-object
	         
	         String from = pMsg.getFrom() ;
	         Account[] allAcc = accData.getAllAccounts() ;
			 Account account = null ;
			 for (int i = 0 ; i<allAcc.length ; i++ ) 
			 {
			 	if (from.equals(allAcc[i].username) || from.equals(allAcc[i].accountName) || from.equals(allAcc[i].address) )
				{
					account = allAcc[i] ;
			 	}
			 }
/*DEBUG*/	 if (account == null)
			 {
			 	System.out.println( "Accounts werden noch nicht im editor ausgewqaehlt" ) ;
			 	account = allAcc[0];
			 }			
	         localMsg = (smail.localStore.LocalMessage)smail.utils.Converter.convert(account,pMsg) ;
		
			if (d)
			{
				try
				{
					localMsg.writeTo(System.out) ;
				}
				catch (IOException e)
				{
					System.out.println("Control.storeNewMail() :: IOException ") ;
					System.out.println(e.getMessage()) ;
					e.printStackTrace() ;
				}
			}
	         LocalFolder folder ;
			if (folderName.equals("OUTBOX"))
			{
				folder = outBox ;
			}
			else if (folderName.equals("INBOX"))
			{
				folder = inBox ;
			}
			else
			{
				folder = (LocalFolder)store.getFolder(folderName) ;
				if (!folder.exists())
					throw new NonExistentFolderException("the folder "+folderName+" told he does not exists :)") ;
			}	
				
			//is this folder open ?
			if (!folder.isOpen())
			{
				folder.open(Folder.READ_WRITE) ;
			}
			//if it is open , is it in readWrite-mode ?
			if (Folder.READ_WRITE != folder.getMode())
			{
				folder.close(expunge) ;
				folder.open(Folder.READ_WRITE) ;
			}

			//ok store this mail
			folder.appendMessages(new Message[] { localMsg } ) ;
			folder.close(true) ;
			
			//this is a test of store and get Mail
			
		}
		catch(MessagingException e)
		{
                    System.out.println(e.getMessage());
                    e.printStackTrace();
                    System.out.println("MessagingException in Control bei :: Converter.convert() !!!") ;
		}
		
	}
	
	FolderContent getFolderContent(String path)
	{
	
		FolderContent fc;
		try
		{
			smail.localStore.LocalFolder folder = (LocalFolder)store.getFolder(path) ;
			fc = new FolderContent(folder) ;
		}
		catch (MessagingException ex)
		{
			System.out.println("MessagingExc in Control.getFolderContent(path) ") ;
			ex.printStackTrace();
			return new FolderContent(new LocalFolder(store,path, null)) ;
		}
		
		return fc ;	
	}
	
//	void send(JFrame upload)
	void send()
	{
//		System.out.println("COntrol.senmd() :: send message()");
 		LocalFolder folder ;
		Message[] msgS ;
		int i = 0 ;		/* this counts the send messages */
		try
		{
			folder = outBox ;
			if (!folder.isOpen())
				folder.open(Folder.READ_ONLY) ;
			msgS = folder.getMessages() ;
	  		// senden testen
//	  		((UploadFrame)upload).setAmount(msgS.length);
	  		for (;i<msgS.length ;i++ )
	  		{
				Account account = null;
		  		SendMail.sendMail(account, (MimeMessage)msgS[i]);
///*DEBUG*/		  		((UploadFrame)upload).progress(i+1);				
	  		}
		}
		catch(Exception e)
		{
			System.out.println( "stuck at message number "+i ) ;
			System.out.println("Exception in Control.send() :: "+e.getMessage()) ;
		}
		
	}
	
//	void receive(JFrame download)
	void receive()
	{
		
		try
		{

			final Account[] accounts = accData.getAllAccounts();
			for (int i = 0 ; i<accounts.length; i++)
                        {
                          ///*DEBUG*/		accounts[i].password = "xyzzy" ;
                          //				final String password ;
                          final smail.gui.PassWordEnter pwe = new smail.gui.PassWordEnter(accounts[i]);
                          
                          if ( accounts[i].password != null && !
                          accounts[i].password.equals("")) 					{
                            Message[] msgArray =
                            smail.server.FetchMail.getMails(accounts[i], false);
                            if ( msgArray != null && msgArray.length > 0 )
                            {
                              inBox.appendMessages(msgArray) ;
                            }
                            accounts[i].password = ""; 					}
                          // repeat with next accountw
                        }
                }
		catch (MessagingException e)
		{
			System.out.println( "could not store received messages " ) ; 
			System.out.println( e.getMessage() ) ;
			e.printStackTrace() ;
		}
		
	}

	
     
	/**
	 *returns an Array of all Accounts available from AccountData. If no Account is defined 
	 *an empty Account array is returned.
	 *@returns smail.server.Account
	 *@see AccountData
	 */
	Account[] getAllAccounts()
	{	
		return accData.getAllAccounts() ;
	}
	   
	/**
	 *adds a new Account to the Account list stored by AccountData.
	 *@see smail.server.AccountData
	 *@see smail.server.Account
	 */
	void addAccount(String accountName, String address ,String username,  String smtpHost, String pop3Host ) throws InvalidAccountName
	{	
		accData.addAccount(accountName, address, username, smtpHost, pop3Host) ;
	}
	
	/**
	 *removes the Account identified with the accountName field in Account-class.
	 *@param name is the name of the Account to remove
	 *@see smail.server.Account
	 */
	void removeAccount(String name)
	{
		accData.remove(name) ;
	}
	
	/**
	 *will change the account identified by <CODE>accountName</CODE> and sets ALL fields
	 *in Account with the parametes passed here.
	 *@see smail.server.Account
	 *@param accountName the idetifier of the account that has to be changed
	 *@param address the email address of the account
	 *@param username is the name needed for authorization on the mail server
	 *@param smtpHost is the host address where to send mails to
	 *@param pop3Host is the host address where to receive mails from
	 */
	void changeAccount(String accountName, String address ,String username,  String smtpHost, String pop3Host )
	{
		accData.changeAccount( accountName, address , username, smtpHost, pop3Host ) ;
	}
	
	/**
	 *removes the folder that is addressed by <code>fullPath</code> AND recursively 
	 *all contained folders and mails !. Only the last part (separated by the 
	 *default separator) is deleted.
	 *@param fullPath defines the the folder (by full path name) to be deleted.
	 */
	void removeFolder(String fullPath)
	{
	
		try
		{
//			System.out.println( "Control.removeFolder()  :: FOlder to delete is :"+fullPath) ;
			LocalFolder delFolder = (LocalFolder)store.getFolder(fullPath) ;
			if (delFolder.isOpen())
			{
				delFolder.close(true) ;
			}
			if (delFolder.exists())
			{
				delFolder.delete(true) ;
			}
		}
		catch (Exception e)
		{
			System.out.println( "Control.removeFolder() :: Exc on deleting a folder" ) ;
			System.out.println( e.getMessage() ) ;
			e.printStackTrace() ;
		}
		
	}
	
	/**
	 *adds a new Folder named <code>newFolder</code> to the folder 
	 *<code>parentFolder</code>. 
	 *@param parentFolder is the folder to make a new folder in.
	 *@param newFolderName is the name of the new folder.
	 */
	void addNewFolder(String parentFolder , String newFolderName) 
									throws FolderExistsException
	{
	
		try
		{
			LocalFolder parent = (LocalFolder)store.getFolder(parentFolder) ;
			String separator = new String(new char[] { parent.getSeparator() }) ;
			LocalFolder newFolder = (LocalFolder)parent.getFolder(parent+separator+newFolderName) ;
			if (newFolder.exists())
			{
				throw new FolderExistsException() ;
			}
			newFolder.create() ;
		}
		catch (Exception e)
		{
			System.out.println( "Control.addFolder() :: Exc on adding folder" ) ;
			System.out.println( e.getMessage() ) ;
			e.printStackTrace() ;
		}
		
	}
			
	/**
	 *this copy a folder and its contents to another folder. The folder
	 *<code>sourceFolderName</code> and its content is copied recursively to
	 *<code>destFolderName</code> folder so that <code>srcFolderName</code> 
	 *will become a subfolder of <code>destFolderName</code> .
	 *@param srcFolderName is the folder to copy .
	 *@param destFolderName is the destination folder to copy <code>srcFolderName</code> and its content
	 *@throws FolderExistsException if the source-folder already exists at the 
	 *									destination.
	 */
	void copyFolder(String srcFolderName, String destFolderName) 
										throws FolderExistsException
	{
	
		try
		{
			LocalFolder srcFolder = (LocalFolder)store.getFolder(srcFolderName) ;
			LocalFolder destFolder = (LocalFolder)store.getFolder(destFolderName) ;
			if (destFolder.exists())
			{
				throw new FolderExistsException() ;
			}
			destFolder.create() ;
			
			// difficult part : copy all subFolder and messages
			LocalFolder[] subfolders ; 
			Message[] msgS ;
			while ((subfolders = (LocalFolder[])srcFolder.list()).length > 0)
			{
				if (!srcFolder.isOpen())
				{
					srcFolder.open(javax.mail.Folder.READ_ONLY) ;
				}
				msgS = srcFolder.getMessages() ;
				if (msgS.length > 0)
				{
					if (!destFolder.isOpen())
					{
						destFolder.open(javax.mail.Folder.READ_WRITE) ;
					}
					else if (destFolder.getMode() != javax.mail.Folder.READ_WRITE)
					{
						destFolder.close(false) ;
						destFolder.open(javax.mail.Folder.READ_WRITE) ;
					}
					destFolder.appendMessages(msgS) ;
					destFolder.close(true) ;
				}				
				for (int i = 0;i<subfolders.length ; i++ )
				{
					copyFolder(subfolders[i].getFullName(), destFolder.getFullName()) ;
				}
				srcFolder.close(true) ;
			}			
		}
//		catch (javax.mail.InvalidStateException e)
//		{
//			System.out.println( "Control.copyFolder() :: InvalidstateExc" ) ;
//			System.out.println( e.getMessage() ) ;
//			e.printStackTrace() ;
//		}
		catch (MessagingException e)
		{
			System.out.println( "Control.copyFolder() :: MessagingExc" ) ;
		}
		
	}

	/**
	 *will copy one message from one folder to another. The Message with message-number
	 *<code>msgNumber</code> will be copied from the folder <code>srcFolderName</code>
	 *to the folder <code>destFolderName</code>. 
	 *@param srcFolderName is the full-path-name of the sourceFOlder.
	 *@param destFolderName is the full-paht-name of the destination folder.
	 *@param msgNumber is the number of the message in the source folder.
	 *@throws IndexOutOfBoundsException if the msgNumber is "out of bounds" :)
	 *@throws NonExistentFolderException if one of the folder does not exists.
	 */
	void copyMessage(String srcFolderName, int msgNumber, String destFolderName) 
				throws NonExistentFolderException , IndexOutOfBoundsException
	{
	
		try
		{
			LocalFolder srcFolder = (LocalFolder)store.getFolder(srcFolderName) ;
			if (!srcFolder.exists())
			{
				throw new NonExistentFolderException() ;
			}
			if (!srcFolder.isOpen())
			{
				srcFolder.open(javax.mail.Folder.READ_ONLY) ;
			}
			Message msg = srcFolder.getMessage(msgNumber) ;
			LocalFolder destFolder = (LocalFolder)store.getFolder(destFolderName) ;
			if (!srcFolder.exists())
			{
				throw new NonExistentFolderException() ;
			}
			if (!destFolder.isOpen())
			{
				destFolder.open(javax.mail.Folder.READ_WRITE) ;
			}
			else if (destFolder.getMode() != javax.mail.Folder.READ_WRITE)
			{
				destFolder.close(true) ;
				destFolder.open(javax.mail.Folder.READ_WRITE) ;
			}
			destFolder.appendMessages(new Message[] { msg }) ;
			srcFolder.close(false) ;
			destFolder.close(true) ;
		}
		catch (MessagingException e)
		{
			System.out.println( "Control.copyMessage() :: MessagingExc") ;
			System.out.println( e.getMessage() ) ;
			e.printStackTrace() ;
		}
		
	}		
			
	/**
	 *deletes the message with <code>msgNumber</code> in the folder 
	 *<code>folderName</code> X expunge does not work X.
	 *@param folderName is the full-path-name of the folder containing the message.
	 *@param msgNumber is the number of the message in the folder.
	 *@throws NonExistentFolderException if the folder does not exists.
	 *@throws IndexOutOfBoundsException if the message-number is out of range(invalid).
	 */
	void deleteMessage(String folderName, int msgNumber) 
					throws NonExistentFolderException , IndexOutOfBoundsException
	{
	
		try
		{
			LocalFolder folder = (LocalFolder)store.getFolder(folderName) ;
			if (!folder.isOpen())
			{
				folder.open(javax.mail.Folder.READ_WRITE) ;
			}
			else if (folder.getMode() != javax.mail.Folder.READ_WRITE)
			{
				folder.close(true) ;
				folder.open(javax.mail.Folder.READ_WRITE) ;
			}
		        // get Message
                        Message lMsg = folder.getMessage(msgNumber) ;
                        
                        // delete Message
                        lMsg.setFlag(javax.mail.Flags.Flag.DELETED, true);
                        
			folder.close(true) ;
		}
		catch (MessagingException e)
		{
			System.out.println( "Control.deleteMessage() :: MessagingException" ) ;
			System.out.println( e.getMessage() ) ;
			e.printStackTrace() ;
		}
		
	}
	
    /**
	 *This method stops the programm and exits this JavaVirtualMachine.
	 *In further releases there will also be a handling of unsaved configurations
	 *and settings and messages.
	 */
	void shutdown ()
	{
		//garbage collection (not by JRE)
		if (d)
			System.out.println( "is propMan gleich null ? :"+(propMan == null) ) ;
		propMan.exiting() ;			/* try to save our properties */
		System.exit(0) ;
	}
	
	/**
	 *Control-constructor which initializes and holds references to GUI,
	 *Fassade and LocalStore
	 */
	private Control(boolean d) 
	{
		
		this.d = d ;
		//	Konfiguration ???	
		propMan = PropertiesManager.getInstance() ;
		
		/* setting the language stored in properties */
		String location = PropertiesManager.get("smail.language") ;
		if (location.equals("GE") )
		{
			//
			Locale.setDefault(new Locale("de","DE"));
		}
		else if (location.equals("ENG") ) {
			//
			Locale.setDefault(new Locale("en","GB"));
		}
		else if (location.equals("FR") ) {
			//
			Locale.setDefault(new Locale("fr","FR"));
			// there is no coresponding property file
		}
		else if (location.equals("SW")) {
			Locale.setDefault(new Locale("sv","SE"));
		}
		else if (location.equals("ES")) {
			Locale.setDefault(new Locale("es","ES"));
			//
		}
		//	LocalStore initialisieren -> LocalStore
		String mboxPath = PropertiesManager.get("smail.home") ;
		System.out.println( "the smail home is : "+mboxPath ) ;
		Properties props = System.getProperties() ;
		Session session = Session.getDefaultInstance(props,null) ;
		try
		{
			URLName storeUrl = new URLName("localstore:"+mboxPath) ;
			javax.mail.Store store = session.getStore(storeUrl) ;
			this.store = (LocalStore)store ;
/*DEBUG*/			System.out.println( "Control.Control() :: get OUTBOX :" ) ;
			this.outBox = (LocalFolder)this.store.getFolder("OUTBOX") ;
/*DEBUG*/			System.out.println( "Control.Control() :: get INBOX :" ) ;
			this.inBox = (LocalFolder)this.store.getFolder("INBOX") ;
			Folder[] folder = inBox.list("*") ;
			for (int i = 0 ; i<folder.length ; i++ ) {
				System.out.println( folder[i] ) ;
			}
		}
		catch(NoSuchProviderException nspexc)
		{
			System.out.println("Provider not found :: Control.Control() :: session.getStore(localStore+smail.home)") ;
			System.out.println(nspexc.getMessage()) ;
			System.out.println("LocalStore could not be initialized") ;
			System.out.println( "Please check the META-INF directory" ) ;
			System.exit(-55) ;
		}
		catch (MessagingException e)
		{
			System.out.println("Control.Control() :: Can not get outbox/inbox") ;
		}

		accData = new AccountData() ;
				
      // creates an Account
      // for testing here are used the following Account 
//     	this.account = new smail.server.Account("swp06", "xyzzy", "pop3", "smtp", "swp.inf.fu-berlin.de", "swp.inf.fu-berlin.de");
		
		this.filterManager = new smail.filter.FilterManager(this.store) ;
				
		//	Fassade				
		this.fassade = new smail.Fassade(this, this.store) ;

		//	GUI initialisieren	-> smail
		this.gui = new Smail(fassade) ;
		
		try {
/*DEBUG*/ System.out.println( "Control.COnntrol() :: OUTBOX subfolder length :"+outBox.list().length) ;
			System.out.println( "COntrol.Control() :: getFolderContent(OUTBOX)") ;
			FolderContent fc = getFolderContent("OUTBOX");
			System.out.println( fc.getMessageHeader().length ) ;
		} catch (MessagingException e) {
			System.out.println( "messsaging exception :: caught Excetpion on outbox" ) ;
		}
	}
}
