package smail;
import smail.gui.* ;
import smail.Control ;
import smail.utils.Converter ;
import smail.localStore.* ;
import javax.mail.* ;
import javax.mail.internet.* ;


/**
 *This is not for testing purpose anymore. The Fassade handles communication 
 *from GUI to other program components. The intention is to simply interfacing
 *the the program for better abstraction from the underlying logic so that 
 *the GUI can be developt without having unwire(?) it from the rest of the 
 *program.
 *<p>
 *All methods that are not commented are usually not implemented and 
 *when called will simply popup a small window with the name of the called method.
 */
 
public class Fassade
{

	private Control ctrl ;
	private LocalStore store;

	/**
	 *applies all manual filter currently stored in filtermanager on ALL messages 
	 *in all folder existing in localstore.
	 *@see smail.localStore.LocalStore
	 *@see smail.filter.FilterManager
	 *@see smail.filter.Filter
	 */
	public void applyManualFilter() {
		ctrl.applyManualFilter();
	}

	/**
	 *returns the FilterManager-reference.This should only be used by the "FilterGuy"
	 *to manage filter.
	 *@see smail.gui.filterGui.FilterGuy
	 *@see smail.filter.FilterManager
	 */
	public smail.filter.FilterManager getFilterManager()
	{
		return ctrl.getFilterManager() ;
	}
	
	/**
	 *returns the programs localstore instance-reference.
	 *@see smail.gui.filterGui.FilterGuy
	 *@see smail.localStore.LocalStore
	 */
	public Store getStore()
	{
		return ctrl.getStore();
	}

	/**
	 *removes the folder that is addressed by <code>fullPath</code> AND recursively 
	 *all contained folders and mails !. Only the last part (separated by the 
	 *default separator) is deleted.
	 *@param fullPath defines the the folder (by full path name) to be deleted.
	 */
	public void removeFolder(String fullPath)
	{
		ctrl.removeFolder(fullPath) ;
	}

	/**
	 *adds a new Folder named <code>newFolderName</code> to the folder 
	 *<code>parentFolderName</code>. 
	 *@param parentFolderName is the folder to make a new folder in.
	 *@param newFolderName is the name of the new folder.
	 */
	public void addNewFolder(String parentFolderName , String newFolderName) 
									throws FolderExistsException
	{
		ctrl.addNewFolder(parentFolderName,newFolderName) ;
	}
	
	/**
	 *this copy a folder and its contents to another folder. The folder
	 *<code>sourceFolderName</code> and its content is copied recursively to
	 *<code>destFolderName</code> folder so that <code>srcFolderName</code> 
	 *will become a subfolder of <code>destFolderName</code> .
	 *@param srcFolderName is the folder to copy .
	 *@param destFolderName is the destination folder to copy <code>srcFolderName</code> and its content
	 *@throws FolderExistsException if the source-folder already exists at the 
	 *									destination.
	 */
	public void copyFolder(String srcFolderName, String destFolderName) 
										throws FolderExistsException
	{
		ctrl.copyFolder(srcFolderName,destFolderName) ;
	}
	
	/**
	 *will copy one message from one folder to another. The Message with message-number
	 *<code>msgNumber</code> will be copied from the folder <code>srcFolderName</code>
	 *to the folder <code>destFolderName</code>. 
	 *@param srcFolderName is the full-path-name of the sourceFOlder.
	 *@param destFolderName is the full-paht-name of the destination folder.
	 *@param msgNumber is the number of the message in the source folder.
	 *@throws IndexOutOfBoundsException if the msgNumber is "out of bounds" :)
	 *@throws NonExistentFolderException if one of the folder does not exists.
	 */
	public void copyMessage(String srcFolderName, int msgNumber, String destFolderName) 
				throws NonExistentFolderException , IndexOutOfBoundsException
	{
		ctrl.copyMessage(srcFolderName,msgNumber,destFolderName) ;
	}
	
	/**
	 *deletes the message with <code>msgNumber</code> in the folder 
	 *<code>folderName</code> X expunge does not work X.
	 *@param folderName is the full-path-name of the folder containing the message.
	 *@param msgNumber is the number of the message in the folder.
	 *@throws NonExistentFolderException if the folder does not exists.
	 *@throws IndexOutOfBoundsException if the message-number is out of range(invalid).
	 */
	public void deleteMessage(String folderName, int msgNumber) 
					throws NonExistentFolderException , IndexOutOfBoundsException
	{
		deleteMessage(folderName, msgNumber) ;
	}
	


	/**
	 *
	 *This constructor is ONLY for testing the Fassade-class without the implementation
	 *of a Control-class.	Testing the GUI.
	 *DO NOT DEPEND IN THIS METHOD.
	 */
	Fassade () {}
	
	/**
	 *This constructor creates a Fassade-Object with Control and LocalStore 
	 *as argument.
	 */
	Fassade(smail.Control ctrl, smail.localStore.LocalStore store) 
	{
		this.ctrl 	= ctrl;	
		this.store 	= store;
	}

	/**
	 *Exits the Programm.
	 *After calling this the Control will clean up settings and try shut down properly.
	 */
	public void exit ()
	{	
		ctrl.shutdown() ;
	}
	
	
	/** 
	 *calls the related method in the control 
	 */
//	public void receive (JFrame download)
	public void receive ()
	{	
//		ctrl.receive(download) ;
		ctrl.receive() ;
	}
	
	/**
	 * calls the related method in the control
	 */
//	public void send (JFrame upload)
	public void send ()
	{	
//		ctrl.send(upload) ;
		ctrl.send();
	}
	
	public FolderContent getFolderContent(String path)
	{
		return(ctrl.getFolderContent(path));
	}
	
	/**
	 *Passes pseudo-message through to the control, to convert and store newly 
	 *created message.
	 *
	 *@param pMsg An E-mail message in gui format
	 *@see smail.gui.MessageContent
	 */
	public void storeNewMail(MessageContent pMsg)
	{
		try
		{
			ctrl.storeNewMail(pMsg);
		}
		catch (NonExistentFolderException e)
		{
			String msg = "There was thrown an exception on opening a folder." ;
			msg += "\nThe massage is :\n" ;
			msg += e.getMessage() ;
			System.out.println( msg ) ;
			e.printStackTrace();
//			call(msg) ;
		}
	}
	
	/**
	 * passes an Editor-Result (Message) to the Control and defines a folder
	 * where to store this (pseudo-)mail.
	 *
	 *@param pMsg  This a E-mail message in a format used by the gui to
	 *			to pass a message to the program.
	 *@param folderName This string defines the folder in which to store the message.
	 *			This has to be a full named folder (full path concatenated at the beginning of 
	 *			the foldername
	 *@see smail.gui.MessageContent
	 *@see smail.localStore.LocalFolder
	 */
	public void storeNewMail (MessageContent pMsg , String folderPath)
	{
		try
		{
			ctrl.storeNewMail(pMsg , folderPath) ;
		}
		catch (NonExistentFolderException e)
		{
			String msg = "N error occued while opening a folder." ;
			msg += "\nThe massage is :\n" ;
			msg += e.getMessage() ;
			System.out.println( e ) ;
			
//			call(msg) ;
		}
	}
	/**
	 *This method is temporarily added because of inconsistencies.
	 *Later this method is replaced by <code>storeNewMail(...)</code>
	 *in the same class.
	 */
	public void storeNewMessage (MessageContent pMsg , String folderPath)
	{
		try
		{
			ctrl.storeNewMail(pMsg , folderPath) ;
		}
		catch (NonExistentFolderException e)
		{
			String msg = "An error occured while opening a folder." ;
			msg += "\nThe massage is :\n" ;
			msg += e.getMessage() ;
			call(msg) ;
		}
	}
		
	/**Returns (pseudo-message formatted) mail which belongs to the passed 
	 *folder (full-named) and the message-number.
	 *
	 *@param msgnum The number of the message in the folder
	 *@param folder The name (full path) of the folder where the messsage is stored
	 *@returns If no exception was thrown this returns the spezified message converted to the MessageContent-format 
	 *@throws This happens if a message does not exists.
	 */
	public MessageContent openMail(int msgnum , String folder)
	{
		try
		{
			return ctrl.getMail(msgnum,folder);
		}
		catch (Exception e)
		{
			String m = "Fassade" ;
			m += "The ErrorMessage is :\n" ;
			m += e.getMessage() ;
			System.out.println( m ) ;
			e.printStackTrace();	
		}
		return(null);
		
	}
	
	/**
	 *Returns a String-Representation of the defaultFolder
	 */
	public String getDefaultFolder()
	{
		try
		{
			return store.getDefaultFolder().getFullName();
		}
		catch(MessagingException mExc)
		{
			System.out.println("EXC bei holen des DefaultFolder von LocalStore(store) "+
						":: Control.getDefaultFolder()") ;
						return null ;
		}
	}
	
	//get headers from all messages in a folder in a gui usefull-representation
	public void getHeaderInfo() 
	{
		call("getHeaderInfo()") ;
	}
	//is klar
	public void saveAttachment()
	{
		call("saveAttachment()") ;
	}
	
	//is klar
	public void openAttachment()
	{
		call("openAttachment()") ;
	}
	
	//save an already existing mail which now has been changed
	public void updateMail() 
	{
		call("updateMail()") ;
	}
	// ??
	public void getAdress()
	{
		call("getAdress()") ;
	}
	
	public void getFolderContent()
	{
		call("getFolder") ;
	}
	
	//Eintrag im Verzeichnis loeschen
	public void removeEntry()
	{
		call("removeEntry()") ;
	}	
	
	//Adressbuch : Adresse hinzufuegen
	public void addAdress()
	{
		call("addAdress()") ;
	}
	
	//Adresse entfernen
	public void removeAdress()
	{
		call("remAdress") ;
	}
	
	//bestehende Adresse aendern
	public void updateAdress()
	{
		call("updateAdress()") ;
	}
	
	//Entraege einer Adresse holen
	public void getAdressInfo ()
	{
		call("getAdressInfo()") ;
	}
	
	//Filter erstellen
	public void newFilter ()
	{
		call("newFilter()") ;
	}
	//Filter entfernen
	public void removeFilter ()
	{
		call("remFilter()") ;
	}
	
	//bestehenden FIlter aendern
	public void updateFilter ()
	{
		call("updateFilter()") ;
	}
	//Filter aktivieren
	public void activateFilter ()
	{
		call("activateFilter()") ;
	}
	//Filter Info holen
	public void getFilterInfo ()
	{
		call("getFilterInfo()") ;
	}
	//Benutzer Einstellungen holen
/*	public void getClientProps ()
	{
		call("getClientProps()") ;
	}
	// s.o.
	public void setClientProps ()
	{
		call("setClientProps()") ;
	}
*/	
	/**
	 *this adds a new e-mail account to the properties and the runtime 
	 *account management.
	 *@param name is the username of the account on the mailserver
	 *@param smtp is the smtp-mailserver-address 
	 *@param pop3 is the pop3 mailserver-address
	 *@param accountName is the name the user gives this account
	 *@see smail.server.AccountData
	 *@see smail.server.Account
	 */
	public void addAccount (String accountName, String address ,String username,  String smtpHost, String pop3Host ) throws smail.server.InvalidAccountName
	{
//		call("addAccount()") ;
		ctrl.addAccount(accountName,address,username,smtpHost,pop3Host) ;
	}
	/**
	 *removes the account spezified by the passed <code>name</code> immediately.
	 *@param accountName is the name (identifier) of the account to be deleted.
	 */
	public void removeAccount (String accountName)
	{
		ctrl.removeAccount(accountName) ;
	}
	
	/**
	 *updates any change on an account. The account is identified by the accountName
	 *and all changes will made upon this account.
	 *@param name is the username of the mailserver-account
	 *@param smtp is the smtp-mail server address
	 *@param pop3 is the pop3-mail server address
	 *@param accountName is the identifier of the account to change.
	 *@see smail.server.AccountData
	 *@see smail.server.Account
	 */
	public void updateAccount (String accountName, String address ,String username,  String smtpHost, String pop3Host )
	{
		ctrl.changeAccount(accountName, address , username, smtpHost, pop3Host ) ;
	}
	
	/**
	 *this returns all Accounts stored in the AccountData-class
	 *@see smail.server.AccountData
	 *@see smail.server.Account
	 */
	public smail.server.Account[] getAllAccounts() 
	{
			return ctrl.getAllAccounts() ;
	}


	//opens configuration-settings
	public void editConfig ()
	{
		call("editConfig()") ;
	}
	
	/**
	 *forces the program to store all properties to the properties file immedialtely.
	 *Usually properties are stored on exiting the program. Calling this method 
	 *will assure that changed settings are written and saved to the properties file.
	 */
/*	public void saveConfig ()
	{
		ctrl.storePropertiesNow("saveConfig()") ;
	}
*/	
	/**
	 *No longer Deprecated ! This method is called when there is an attempt to
	 *open a folder that does not exists.
	 *@param m The message that is to be displayed.
	 */
	private void call(String m)
	{
		Popup p = new Popup(m) ;
		p.show() ;

	}
	
	
}
