/**
 *This class supplies important information about the content of a folder.
 *Two fields hold the names of all subFolders and the messages in a folder.
 *The subFolders are stored as Strings representing the full path name of 
 *a folder.<br>
 *Message-informations are stored as @see smail.MessageHeader objects.
 */

package smail ;

import smail.localStore.LocalFolder ;
import javax.mail.* ;
import javax.mail.internet.* ;


public class FolderContent
{
	private String[] subFolderNames = new String [0]  ;
	private MessageHeader[] msgHeader = new MessageHeader[0];
	private String rootFolderName = "";




        // Default Konstruktor just to test the GUI with Dummy-Classes

        // could be removed when GUI doesn't need any Dummies to show Messages

        public FolderContent()

        {
        }



	
	/**
	 *constructs a new FolderContent-object representing the contents of a folder.
	 *The folder is spezified by the string-parameter (the full path name).
	 *The fields of the resulting object contain the information about the names
	 *(full path names) and the header informations of all contained messages.
	 *The messages are hold by an string-array where the array-index is the same
	 *as the message-number in a LocalFolder. Request a message needs the parent-folder
	 *and the array-index has to be passed.
	 *@param rootFolder is the full path from the root down to the requested folder.
	 *@see smail.MessageHeader
	 */	 
	public FolderContent(javax.mail.Folder rootFolder)
	{
		try
		{
			LocalFolder rootLFolder = (LocalFolder)rootFolder;
			rootFolderName = rootLFolder.getFullName() ;
///*DEBUG*/			System.out.println( "FolderCOntent :: name is :"+rootFolderName ) ;
			javax.mail.Folder[] subF = rootLFolder.list() ;
			subFolderNames = new String[subF.length] ;
			for (int i = 0 ;subF != null && i<subF.length ;i++ )
			{
				subFolderNames[i] = ((LocalFolder)subF[i]).getFullName() ;
			}
			try
			{
			
				if ( !rootLFolder.isOpen() ) {
					(rootLFolder).open(javax.mail.Folder.READ_WRITE) ;
				}
				int msgCount = rootLFolder.getMessageCount();
				msgHeader = new MessageHeader[msgCount];
				for (int k = 0 ; k<msgCount ; k++) {
///*DEBUG*/			System.out.println( "(FolderContent.java:69) :: message-index is :"+k+" adding on to fit API" ) ;
					msgHeader[k] = new MessageHeader((smail.localStore.LocalMessage)rootLFolder.getMessage(k+1));
				}
///*DEBUG*/		System.out.println( "FOlderContent :: found "+msgCount+" messages " ) ;
				rootLFolder.close(false) ;
			}
			catch (IllegalStateException e)
			{
				System.out.println( "FolderContent.FolderContent() :: opened folder in illegal state" ) ;
			}			
		}
		catch (javax.mail.MessagingException mex)
		{
			System.out.println("messgeing exc in FolderContent.FolderContent() :: !") ;
			System.out.println(mex.getMessage()) ;
			mex.printStackTrace() ;
		}		 
	}

	/**
	 *is used to remind the folder this FolderContent is bound to.
	 *@returns the full path String of the folder this FolderContent 
	 *was instantiated on.
	 */
	public String getThisRoot()
	{
		return rootFolderName ;
	}
	
	/**
	 *is used to obtain the subFolder of the folder passed on initialization
	 *of a new FolderContent-object. The subFolder are addressed by the full
	 *path in String representation.
	 *@returns the subFolder-names of the folder passed on instantiation.
	 */	 
	public String[] getSubFolder()
	{
		return subFolderNames ;
	}
	
	/**
	 *returns the MessageHeader-objects of all messages in this folder.
	 *The messages can be addressed by the messageNumber-field of a 
	 *MessageHeader-object and the folder where the corresponding Message 
	 *is stored. The folder can obtained by the 
	 *getContainingFolderName()-method in MessageHeader-class.
	 *@see smail.MessageHeader
	 */
	public MessageHeader[] getMessageHeader()
	{
		return msgHeader ;	
	}		
}
