/**
 *Objects of this class are used to store header informations of messages.
 *Most informations are used by the gui to show a message pre-view in a table.<br>
 *<h2>It is very important to check whether a field is null or not.</h2>
 *<br>
 *All fields should be self explainable. folderPosition gives the message number
 *of a message-object in a folder.
 *@see smail.localStore.LocalMessage
 *@see javax.mail.Message ;
 */
package smail ;

import javax.mail.MessagingException ;

public class MessageHeader
{
	public String[] 	to = new String[0] ; 
	public String 		from = "" ;
	public String[] 	cc = new String[0] ;
	public String[] 	bcc = new String[0] ;
	public String 		subject = "" ;
	public int 			folderPosition = -1 ;
	public boolean 		attachment = false ;
	public boolean 		read = false;
	public boolean 		answered  = false ;
	private String 		folder = "";
		
	/** gives the date the message was received */
	public String rDate  = "";
	
	/** gives the date this message was sent */
	public String sDate = "";
	

        // Default Konstruktor just to test the GUI with Dummy-Classes
        // could be removed when GUI doesn't need any Dummies to show Messages
        public MessageHeader()
        {

        }



	/**
	 *initializes a new MessageHeader-object and tries to set all fields 
	 *with the corresponding Message informations.
	 */
	public MessageHeader(javax.mail.Message msg)
	{
		getAddresses(msg) ;

//		if (msg.isSet(Flags.Flag.SEEN)
//		{
		try
		{
			subject = msg.getSubject() ;
			read = ( msg.isSet(javax.mail.Flags.Flag.SEEN) ) ;
			answered = ( msg.isSet(javax.mail.Flags.Flag.ANSWERED) ) ;
			if(msg.getReceivedDate()!=null)
				rDate = msg.getReceivedDate().toString();
			if(msg.getSentDate()!=null)
				sDate = msg.getSentDate().toString() ;
		}
		catch (MessagingException mexc)
		{
			System.out.println( "MessagingExc in MessageHeader.MessageHeader() !" ) ;
		}
		folderPosition = msg.getMessageNumber() ;
		try {
//			System.out.println( "MessageHeader.java:70	ist msg gleich null ?"+(msg == null) ) ;
//			System.out.println( "MessageHeader.java:70  ist folder gliech null ?"+(msg.getFolder() == null )) ;
//			System.out.println( "MessageHeader.java:70	folder name "+msg.getFolder().getName() ) ;
			folder = msg.getFolder().getFullName() ;
//			System.out.println( "						"+folder ) ;
		} catch (Exception e) {
//			System.out.println( "					caught on :"+e.getMessage() ) ;
		}
	}
	
	/**
	 *returns the folder where this Message was obtained from.The result is 
	 *the full path of the folder.
	 */	 
	public String getContainingFolderPath()
	{
		return folder ;
	}
	
	private void getAddresses(javax.mail.Message msg)
	{
		javax.mail.Address[] tmpAddr ;
		try
		{
			this.from = (msg.getFrom())[0].toString() ;
		}
		catch (MessagingException mexc)
		{
			System.out.println( "Exc in MessageHeader.Constructor() :: no success getting FROM addresses ") ;
		}
		
		
		/* recipientType TO */
		try
		{
			tmpAddr = msg.getRecipients(javax.mail.Message.RecipientType.TO) ;
			if (tmpAddr != null)
			{
				to = new String[tmpAddr.length] ;
				for (int i = 0 ; i<tmpAddr.length ; i++)
				{
					to[i] = tmpAddr[i].toString() ;
				}
			}
		}
		catch (MessagingException mexc)
		{
			System.out.println( "Exc in MessageHeader.Constructor() :: no success getting TO addresses ") ;
		}
		/* recipientType CC */
		try
		{
			tmpAddr = msg.getRecipients(javax.mail.Message.RecipientType.CC) ;
			if (tmpAddr != null)
			{
				cc = new String[tmpAddr.length] ;
				for (int i = 0 ; i<tmpAddr.length ; i++)
				{
					cc[i] = tmpAddr[i].toString() ;
				}
			}
		}
		catch (MessagingException mexc)
		{
			System.out.println( "Exc in MessageHeader.Constructor() :: no success getting CC addresses ") ;
		}

		/* recipientType BCC */
		try
		{
			tmpAddr = msg.getRecipients(javax.mail.Message.RecipientType.BCC) ;
			if (tmpAddr != null)
			{
				bcc = new String[tmpAddr.length] ;
				for (int i = 0 ; i<tmpAddr.length ; i++)
				{
					bcc[i] = tmpAddr[i].toString() ;
				}
			}
		}
		catch (MessagingException mexc)
		{
			System.out.println( "Exc in MessageHeader.Constructor() :: no success getting BCC addresses ") ;
		}
		
		try
		{
			subject = msg.getSubject() ;
		}
		catch (MessagingException mexc)
		{
			System.out.println( "Exc in MessageHeader.Constructor() :: no success getting subject") ;
		}	
	}
	
}
