package smail.filter;

/** This class represents a action of an filter<P> */
public class Action{

    /** constant for type-declaration of the actual action<br>
     * This constant is used for a action, which deletes a message when the 
     * condition <br> is true (the test method returned a true)
     */
    public static final int DELETE    = 0;
    
    /** constant for type-declaration of the actual action<br>
     * This constant is used for a action, which moves a message to the 
     * destination-folder <br>
     * when the condition is true (the test method returned a true)
     */
    public static final int MOVE      = 1;
        
    /** the type of the action (see constants above) */
    private int                 type;
    
    /** the full-name of the destination-folder, for actions, which moves 
     * messages <br> into a specified folder
     */
    private java.lang.String    destination = "";
        
    /** creates a ne action-object (which dont move any messages into a folder)
     * @param type the type of the action
     */
    public Action (int type) throws smail.filter.NoSuchTypeException{ 
        if (type >= 0 && type <= 1)
            this.type = type;
        else throw new smail.filter.NoSuchTypeException();
    }
        
    /** creates a new action-object (which move messages into the destination-
     * folder)
     * @param type the type of the action
     * @param dest the full-name of the destination-folder
     */
    public Action (int type, java.lang.String dest) throws smail.filter.NoSuchTypeException { 
        
        if (type >= 0 && type <= 1)
            this.type = type;
        else throw new smail.filter.NoSuchTypeException();
        
        this.type        = type; 
        this.destination = dest;
    }
    
    /** set the type of the action */
    public void setType (int type)  throws smail.filter.NoSuchTypeException{
        if (type >= 0 && type <= 1)
            this.type = type;
        else throw new smail.filter.NoSuchTypeException();
    }
    
    /** to get the type of the action */
    public int getType () { return this.type; }
    
    /** set the full-name of the destination-folder of the action */
    public void setDestination (java.lang.String dest) {
        this.destination = dest;
    }

    /** to get the full-name of the destination-folder */
    public java.lang.String getDestination() { return this.destination; }
    
}