package smail.filter;

/** This class represents a Manager for the filters, which works on a given store.
 * It gets the filter from the PropertiesManager of the programm.
 * @see smail.utils.PropertiesManager
 * @see  javax.mail.Store
 */
public class FilterManager implements smail.utils.Configurable {
    
    /** The store on which the FilterManager is working on */
    private javax.mail.Store    store;
    
    /** The filters of the type PRE which the FilterManger gets from the PropertiesManager
      * @see smail.filter.Filter.PRE
     */
//    private Filter[]    PreFilter = new Filter[0] ;

    /** The filters of the type POST which the FilterManger gets from the PropertiesManager
     * @see smail.filter.Filter.POST
     */
//    private Filter[]    PostFilter = new Filter[0] ;
    
    /** The filters of the type MANUALLY which the FilterManger gets from the PropertiesManager
     * @see smail.filter.Filter.MANUALLY
     */
//    private Filter[]  ManuallyFilter = new Filter[0] ;
    

	private Filter[] allFilter = new Filter[0] ;

    /** creates a new FilterManager
     * @param store the store on which the FilterManager works on
     */
    public FilterManager(javax.mail.Store store) {
        this.store = store;
        
        // register myself as PropertyListener
        smail.utils.PropertiesManager.addPropertiesListener((smail.utils.Configurable)this);
        
        // get the filters from the ProprtiesManager
        
        // get the information how many filters exists
        int count = 0;
        try { 
            count = java.lang.Integer.parseInt(smail.utils.PropertiesManager.get("smail.filter.count"));
            // get all the filters
            for (int i=0; i<count; i++) {
				// initialiases the needed variables
                java.lang.String filterName             = "";
                int filterType                          = -1;
                int filterCondCount                     = 0 ;
                smail.filter.Condition[] conds;
                smail.filter.Action action              = null;

                // get the informations from PropertiesManager
                filterName = smail.utils.PropertiesManager.get("smail.filter." + i + ".name");
                
                // active or not
                String str = smail.utils.PropertiesManager.get("smail.filter." + i + ".active");
                boolean active = (str.equals("true")) ? true : false;
                
                // gets the type of the filter
                filterType = java.lang.Integer.parseInt(smail.utils.PropertiesManager.get("smail.filter." + i + ".type"));
                // how many Condition ?
                filterCondCount = java.lang.Integer.parseInt(smail.utils.PropertiesManager.get("smail.filter." + i + ".condition.count"));

                // create a new Array of Condition-objects
                conds = new smail.filter.Condition[filterCondCount];
                for (int j=0; j < filterCondCount; j++) {

                    // get the type of the condition
                    java.lang.String filterCondType = smail.utils.PropertiesManager.get("smail.filter." + i + ".condition." + j + ".type");
					
                      // which type of Condition ?
                    if (filterCondType.equals("subject")) {
                            java.lang.String keyword = smail.utils.PropertiesManager.get("smail.filter." + i + ".condition." + j + ".keyword");
                            // create a new SubjectCondition
                            conds[j] = new smail.filter.SubjectCondition(keyword);
                    }
                    else if (filterCondType.equals("from")) { 
                            java.lang.String keyword = smail.utils.PropertiesManager.get("smail.filter." + i + ".condition." + j + ".keyword");
                            // create a new FromCondition
                            conds[j] = new smail.filter.FromCondition(keyword);
                    }
                    else if (filterCondType.equals("date")) { 
                            int type = java.lang.Integer.parseInt(smail.utils.PropertiesManager.get("smail.filter." + i + ".condition." + j + ".dateType"));
                            java.lang.String date = smail.utils.PropertiesManager.get("smail.filter." + i + ".condition." + j + ".date");
                            // create a new DateCondition
                            try {conds[j] = new smail.filter.DateCondition(type,new java.util.Date(date)); }
                            catch (smail.filter.NoSuchTypeException e) { System.out.println("NoSuchTypeException in FilterManager"); }
                    }
                }
                
                // get the type of the action
                int filterActionType = java.lang.Integer.parseInt(smail.utils.PropertiesManager.get("smail.filter." + i + ".action.type"));
                // which type of action ?
                if (filterActionType == smail.filter.Action.MOVE) { 
                    java.lang.String filterActionDest = smail.utils.PropertiesManager.get("smail.filter." + i + ".action.destination");
                    // create a new action-object
                    try { action = new smail.filter.Action(filterActionType, filterActionDest); }
                    catch (NoSuchTypeException e) { System.out.println("NoSuchTypeException in FilterManager"); }
                }
                else if (filterActionType == smail.filter.Action.DELETE) { 
                    
                    // create a new action-object
                    try { action = new smail.filter.Action(filterActionType); }
                    catch (NoSuchTypeException e) { System.out.println("NoSuchTypeException in FilterManager"); }
                }

                // create a new filter-object
                try {
                    smail.filter.Filter newFilter = new Filter(filterType, filterName, action, conds);
                    
                    // set the activity of true
                    newFilter.setActivity(active);
                    
                    // set the filterId
                    newFilter.setFilterId(i);
                    
                    // append the new filter
					this.addFilter(newFilter) ;
//                    switch (filterType) {
//                        case(smail.filter.Filter.PRE)  : this.addFilter(newFilter);break;
//                        case(smail.filter.Filter.POST) : this.addFilter(newFilter);break;
//                        case(smail.filter.Filter.MANUALLY)  : this.addFilter(newFilter);break;
//                        case(smail.filter.Filter.REC_MAN)  : {
//                            this.addFilter(newFilter);
//                            this.addFilter(newFilter);
//                        }
//                    }
                    
                }
                catch (smail.filter.NoSuchTypeException e) { System.out.println("NoSuchTypeException in FilterManager"); }
                catch (smail.filter.NameAlreadyExistException e) { System.out.println("NameAlreadyExistException in FilterManager"); }
            }
        }
        catch (NumberFormatException e) { System.out.println("NumberFormatException in FilterManager"); }
    }
    
    /** This function applies a given filter on a given folder
     * @param folder the folder on which the filter applies
     * @param filter the filter which is applied
     */
    public void applyFilter (javax.mail.Folder folder, smail.filter.Filter filter) throws FilterNotActiveException {
        
        // filter active ?
        if ( ! filter.getActivity() ) throw new smail.filter.FilterNotActiveException();
        
        smail.filter.Condition[] conditions;
        
        if (filter.getType() == smail.filter.Action.DELETE) {
            try {
                javax.mail.Message[] msgs = folder.getMessages();
                conditions = filter.getCondition();
               
                for (int i=0; msgs[i] != null && i < msgs.length; i++) {
                    boolean test = true;
                    for (int j=0; j < conditions.length; j++)
                        test = test && conditions[j].test(msgs[i]);
                    
                msgs[i].setFlag(javax.mail.Flags.Flag.DELETED, test);
                folder.expunge();
                }
            }
            catch (javax.mail.MessagingException e) {
                System.out.println("applyFilter in FilterManager :: MessagingException");
				System.out.println( "message is :"+e.getMessage() ) ;
				e.printStackTrace();
            }
        }
        else if (filter.getType() == smail.filter.Action.MOVE) {
            try {
                javax.mail.Message[] msgs = folder.getMessages();
                conditions = filter.getCondition();
                javax.mail.Message[] toAppendMsg = new javax.mail.Message[1];
                
                for (int i=0; msgs[i] != null && i < msgs.length; i++) {
                    javax.mail.Folder destFolder = this.store.getFolder(filter.getAction().getDestination());
                    
                    boolean test = true;
                    for (int j=0; j < conditions.length; j++)
                        test = test && conditions[j].test(msgs[i]);
                    
                    if (test) {
                        toAppendMsg[0] = msgs[i];
                        destFolder.appendMessages(toAppendMsg);
                        msgs[i].setFlag(javax.mail.Flags.Flag.DELETED, true);
                    }
                }
                folder.expunge();
            }
            catch (javax.mail.FolderNotFoundException e) {
                System.out.println("applyFilter in FilterManager :: FolderNotFoundException");
                System.out.println( "message is :"+e.getMessage() ) ;
                e.printStackTrace();
				
            }
            catch (javax.mail.MessagingException e) {
                System.out.println("applyFilter in FilterManager :: MessagingException");
                System.out.println( "message is :"+e.getMessage() ) ;
                e.printStackTrace();
				
            }                 
        }
    }    
    
    /** gives a Vector containing all the filter of the type PRE
     * @see smail.filter.Filter.PRE
     */
//    public Filter[] getAllPreFilter() { return this.PreFilter ; }
    
    /** gives a Vector containing all the filter of the type POST
     * @see smail.filter.Filter.POST
     */
//    public Filter[] getAllPostFilter() { return this.PostFilter; }
    
    /** gives a Vector containing all the filter of the type MANUALLY
     * @see smail.filter.Filter.MANUALLY
     */
//    public Filter[] getAllManuallyFilter() { return this.ManuallyFilter; }
    
    /** get a filter identified with the type and the name */
    public smail.filter.Filter getFilter (String name) throws NoSuchNameException
    {
		synchronized(allFilter)
		{
			for (int i = 0 ; i<allFilter.length ; i++)
			{
				if (allFilter[i].getName().equals(name))
				{
					return allFilter[i] ;
				}
			}
			throw new smail.filter.NoSuchNameException();
		}
    }
    
	public Filter getFilter(int filterID) throws WrongFilterIDException
	{
		synchronized(allFilter)
		{

			if (allFilter[filterID].getFilterId() == filterID)
			{
				return allFilter[filterID] ;
			}
			throw new WrongFilterIDException() ;
//			for (int i = 0 ; i<allFilter.length ; i++)
//			{
//				if (allFilter[i].getName().equals(name))
//				{
//					return allFilter[i] ;
//				}
//			}
//			throw new smail.filter.NoSuchNameException();		
		}
	}
	
	public synchronized void removeFilter(int filterID)
	{
		try
		{
			this.removeFilter( this.getFilter(filterID).getName() );
		}
		catch (WrongFilterIDException e)
		{
			// XXX
		}
		catch (NoSuchNameException e)
		{
			// XXX
		}
	}
	
    /** remove a filter identified with the type and the name */
    public synchronized void removeFilter (String name) throws NoSuchNameException
    {
		synchronized(allFilter)
		{

	        // hold the filter for later
	        smail.filter.Filter filter = this.getFilter(name);//trowns NoSuchName
	            
	            //unregister the filter
				Filter[] temp = new Filter[allFilter.length-1];
				System.arraycopy( allFilter, 0, temp, 0, filter.getFilterId() ) ;
				System.arraycopy( allFilter, filter.getFilterId()+1, temp, filter.getFilterId(), allFilter.length-filter.getFilterId()-1 );
				allFilter = temp ;
				//adjust filter ID's
				for (int k = 0 ; k<allFilter.length ; k++)
				{
					allFilter[k].setFilterId(k) ;
				}
				
	            // get the filterId
//	            int filterId = filter.getFilterId();
//	            
//	            smail.utils.PropertiesManager.set("smail.filter." + filterId + ".name", "");
//	            smail.utils.PropertiesManager.set("smail.filter." + filterId + ".active", "");
//	            smail.utils.PropertiesManager.set("smail.filter." + filterId + ".type", "");
//	            smail.utils.PropertiesManager.set("smail.filter." + filterId + ".action.type", "");
//	            smail.utils.PropertiesManager.set("smail.filter." + filterId + ".action.destination", "");
//	            smail.utils.PropertiesManager.set("smail.filter." + filterId + ".condition.count", "");
//	            
//	            // get the count of Conditions
//	            int condCount = filter.getCondition().length;
//	            
//	            // for all Conditions
//	            for (int i=0; i< condCount; i++) { 
//	                smail.utils.PropertiesManager.set("smail.filter." + filterId + ".condition." + i + ".keyword", "");
//	                smail.utils.PropertiesManager.set("smail.filter." + filterId + ".condition." + i + ".dateType", "");
//	                smail.utils.PropertiesManager.set("smail.filter." + filterId + ".condition." + i + ".date", "");
//	            }
		}//END SYNCHRO
    }
    
      /** automatically saves a pre-generated filter in the equivalent Vector of the FilterManager 
        and register the filter to the ProprtiesManager
       */
    public synchronized void addFilter(smail.filter.Filter filter) throws NameAlreadyExistException {
        try {
			smail.filter.Filter findFilter = this.getFilter( filter.getName() );
        }
            catch (NoSuchNameException e) { 

				synchronized(allFilter)
				{
					smail.filter.Filter[] temp = new Filter[allFilter.length+1];
					System.arraycopy( allFilter, 0, temp, 0, allFilter.length );
					temp[allFilter.length] = filter;
					filter.setFilterId(allFilter.length);
					allFilter = temp;
				}			 
             return; 
        }   
        // Exception is not catched -> Name already exists !
        throw new smail.filter.NameAlreadyExistException();
                   
     }
    
	public Filter[] getAllFilter()
	{
		synchronized(allFilter)
		{
			return allFilter ;
		}
	}
	
	public void exiting()
	{
		int ftw ;
		Filter filter ;
		for (ftw = 0 ; ftw< this.allFilter.length ; ftw++ )
		{
			filter = allFilter[ftw] ;
			// set the name
			smail.utils.PropertiesManager.set("smail.filter." + ftw + ".name", filter.getName());
			
			// set the Activity
			smail.utils.PropertiesManager.set("smail.filter." + ftw + ".active", java.lang.String.valueOf(filter.getActivity()) );
			
			// set the type of the filter
			smail.utils.PropertiesManager.set("smail.filter." + ftw + ".type", java.lang.String.valueOf(filter.getType()) );
			
			// set the count of conditions
			smail.utils.PropertiesManager.set("smail.filter." +  ftw+ ".condition.count", java.lang.String.valueOf(filter.getCondition().length));
			// for all conditions
			for (int condN = 0; condN < filter.getCondition().length;condN++) {
			    
			    // get the Condition
			    smail.filter.Condition temp = filter.getCondition()[condN];
			    
			    // which type of Condition ?
			    if (temp instanceof SubjectCondition) {
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".type", "subject");
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".keyword", ((smail.filter.SubjectCondition) temp).getKeyword());
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".dateType", "");
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".date", "");
			    }
			    else if (temp instanceof FromCondition) {
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".type", "from");
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".keyword", ((smail.filter.FromCondition) temp).getKeyword());
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".dateType", "");
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".date", "");
			    }
			    else if (temp instanceof DateCondition) {
			            smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".type", "date");
			            smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".keyword", "");
			            smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".dateType", java.lang.String.valueOf(((smail.filter.DateCondition) temp).getType()));
			            smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".condition." + condN + ".date", ((smail.filter.DateCondition) temp).getDate().toString());
			    }
			}
			
			// which type of action ?
			if (filter.getAction().getType() == smail.filter.Action.DELETE) {
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".action.type", java.lang.String.valueOf(filter.getAction().getType()) );
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".action.destination", "");
			}
			else if (filter.getAction().getType() == smail.filter.Action.MOVE) {
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".action.type", java.lang.String.valueOf(filter.getAction().getType()) );
			        smail.utils.PropertiesManager.set("smail.filter." +  ftw + ".action.destination", filter.getAction().getDestination());
			}
		}	// END FOR-BLOCK
		smail.utils.PropertiesManager.set("smail.filter.count", ftw+"") ;
		// unregister all trash filter in property manager
		
		String trashFilter ;
		int k = ftw ;
		while (null != smail.utils.PropertiesManager.get("smail.filter." + k + ".name"))
		{
		    smail.utils.PropertiesManager.set("smail.filter." + k + ".name", "");
		    smail.utils.PropertiesManager.set("smail.filter." + k + ".active", "");
		    smail.utils.PropertiesManager.set("smail.filter." + k + ".type", "");
		    smail.utils.PropertiesManager.set("smail.filter." + k + ".action.type", "");
		    smail.utils.PropertiesManager.set("smail.filter." + k + ".action.destination", "");
//		    smail.utils.PropertiesManager.set("smail.filter." + k + ".condition.count", "");
		    
			String countString = smail.utils.PropertiesManager.get("smail.filter." + k + ".condition.count") ;		    
			try
			{
				int conditionCount = Integer.parseInt(countString) ;
				for (int j = 0 ; j<conditionCount ; j++ )
				{
					smail.utils.PropertiesManager.set("smail.filter." + k + ".condition." + j + ".type","") ;
					smail.utils.PropertiesManager.set("smail.filter." + k + ".condition." + j + ".keyword", "");
					smail.utils.PropertiesManager.set("smail.filter." + k + ".condition." + j + ".dateType", "");
					smail.utils.PropertiesManager.set("smail.filter." + k + ".condition." + j + ".date", "");
				}
				smail.utils.PropertiesManager.set("smail.filter." + k + ".condition.count", "") ;
			}
			catch (NumberFormatException e)
			{
				System.out.println( "FilterManager.exiting() :: NumberFormatException" ) ;
			}
		k++ ;		    
		}
//		System.out.println( "FilterManager.exiting() :: deleted "+(k-ftw)+" Filter from PropertiesManager" ) ; /*DEBUG*/
	}
	
	public void toggleActivity(int filterID)
	{
		try
		{
			synchronized(allFilter)
			{
				if (filterID < 0 || filterID >= allFilter.length)
					return  ;
			}	
			boolean active = getFilter(filterID).getActivity() ;
			getFilter(filterID).setActivity( !active ) ;
		}
		catch (WrongFilterIDException e)
		{
			System.out.println( "FilterManager.toggleActivity() :: caught Wrong ID Exc" ) ;
		}
	}			
	
    /**
     * This is called when a property has changed.
     */
    public void propertyChanged(smail.utils.PropertyChangedEvent event) {
        //...
    }
    
}