// EditorFrame.java

package smail.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

/** The EditorFrame class displays a window where the user can create a new message.
 * There are given some tools to help him by this work, such as cut, copy, paste,
 * find, replace, go to line, add a signature or use an addressbook.
 * <br><br>
 * The to-field has allways be filled with an information, otherwise the message could'nt
 * be sent or moved to the draftfolder.
 */


public class EditorFrame extends JFrame
{
    smail.Fassade fassade = null;

    smail.utils.PropertiesManager propMan = smail.utils.PropertiesManager.getInstance();
    String sPropertieFile = "";
    String sFromId = "";
    String sFromText = "";

    boolean bCcToggle = true;  // true means cc, false means bcc
    boolean bSmileToggle = false;
    boolean bEncryptToggle = false;

    int iIndexOld = 0;
    String sFindOld = "";
    String sReplaceOld = "";

    Vector vToAddresses = new Vector();
    Vector vCcAddresses = new Vector();
    Vector vBccAddresses = new Vector();
    Vector vAttachments = new Vector();

    JPanel contentPane;

    // property-File for chosing language (editor_lang_en for english, editor_lang_de for german)
    private ResourceBundle resource = ResourceBundle.getBundle("smail/gui/editor_lang");

    // getting the language specific textes for buttons, menus, etc from the property-file
    private String title = resource.getString("titleEditor_key");

    private String sMenuMessage = resource.getString("menuMessage_key");
    private String sMenuMessageSend = resource.getString("menuMessageSend_key");
    private String sMenuMessageDraft = resource.getString("menuMessageDraft_key");
    private String sMenuMessageClear = resource.getString("menuMessageClear_key");
    private String sMenuMessageClose = resource.getString("menuMessageClose_key");

    private String sMenuEdit = resource.getString("menuEdit_key");
    private String sMenuEditCut = resource.getString("menuEditCut_key");
    private String sMenuEditCopy = resource.getString("menuEditCopy_key");
    private String sMenuEditPaste = resource.getString("menuEditPaste_key");
    private String sMenuEditSelectAll = resource.getString("menuEditSelectAll_key");
    private String sMenuEditFind = resource.getString("menuEditFind_key");
    private String sMenuEditReplace = resource.getString("menuEditReplace_key");
    private String sMenuEditGoToLine = resource.getString("menuEditGoToLine_key");

    private String sMenuAttachment = resource.getString("menuAttachment_key");
    private String sMenuAttachmentAdd = resource.getString("menuAttachmentAdd_key");
    private String sMenuAttachmentRemove = resource.getString("menuAttachmentRemove_key");
    private String sMenuAttachmentRemoveAll = resource.getString("menuAttachmentRemoveAll_key");

    private String sMenuSignature = resource.getString("menuSignature_key");
    private String sMenuSignatureNew = resource.getString("menuSignatureNew_key");

    private String sMenuAddressbook = resource.getString("menuAddressbook_key");
    private String sMenuAddressbookOpen = resource.getString("menuAddressbookOpen_key");

    private String sMenuOptions = resource.getString("menuOptions_key");
    //private String sMenuOptionsHTML = resource.getString("menuOptionsHTML_key");
    private String sMenuOptionsEncrypt = resource.getString("menuOptionsEncrypt_key");
    private String sMenuOptionsConfiguration = resource.getString("menuOptionsConfiguration_key");

    private String sMenuHelp = resource.getString("menuHelp_key");
    private String sMenuHelpAbout = resource.getString("menuHelpAbout_key");

    //Popup Menus
    private String sPopupToRemove = resource.getString("popupToRemove_key");
    private String sPopupToRemoveAll = resource.getString("popupToRemoveAll_key");
    private String sPopupToEdit = resource.getString("popupToEdit_key");
    private String sPopupToTransfer = resource.getString("popupToTransfer_key");

    private String sPopupCcRemove = resource.getString("popupCcRemove_key");
    private String sPopupCcRemoveAll = resource.getString("popupCcRemoveAll_key");
    private String sPopupCcEdit = resource.getString("popupCcEdit_key");
    private String sPopupCcTransfer = resource.getString("popupCcTransfer_key");

    private String sPopupAttachmentAdd = resource.getString("popupAttachmentAdd_key");
    private String sPopupAttachmentRemove = resource.getString("popupAttachmentRemove_key");
    private String sPopupAttachmentRemoveAll = resource.getString("popupAttachmentRemoveAll_key");

    // Tooltips for Buttons
    private String sTooltipSend = resource.getString("tooltipSend_key");
    private String sTooltipDraft = resource.getString("tooltipDraft_key");
    private String sTooltipCut = resource.getString("tooltipCut_key");
    private String sTooltipCopy = resource.getString("tooltipCopy_key");
    private String sTooltipPaste = resource.getString("tooltipPaste_key");
    private String sTooltipAttachment = resource.getString("tooltipAttachment_key");
    private String sTooltipSignature = resource.getString("tooltipSignature_key");
    private String sTooltipAddressbook = resource.getString("tooltipAddressbook_key");
    private String sTooltipSmile = resource.getString("tooltipSmile_key");
    private String sTooltipHelp = resource.getString("tooltipHelp_key");

    private String sLabelTo = resource.getString("labelTo_key");
    private String sLabelFrom = resource.getString("labelFrom_key");
    private String sLabelSubject = resource.getString("labelSubject_key");

    private String sLabelError = resource.getString("labelError_key");
    private String sTitleError = resource.getString("titleError_key");


    // Menu
    JMenuBar jMenuBar1 = new JMenuBar();

    JMenu jMenuMessage = new JMenu(sMenuMessage);
    JMenuItem jMenuMessageSend = new JMenuItem(sMenuMessageSend);
    JMenuItem jMenuMessageDraft = new JMenuItem(sMenuMessageDraft);
    JMenuItem jMenuMessageClear = new JMenuItem(sMenuMessageClear);
    JMenuItem jMenuMessageClose = new JMenuItem(sMenuMessageClose);

    JMenu jMenuEdit = new JMenu(sMenuEdit);
    JMenuItem jMenuEditCut = new JMenuItem(sMenuEditCut);
    JMenuItem jMenuEditCopy = new JMenuItem(sMenuEditCopy);
    JMenuItem jMenuEditPaste = new JMenuItem(sMenuEditPaste);
    JMenuItem jMenuEditSelectAll = new JMenuItem(sMenuEditSelectAll);
    JMenuItem jMenuEditFind = new JMenuItem(sMenuEditFind);
    JMenuItem jMenuEditReplace = new JMenuItem(sMenuEditReplace);
    JMenuItem jMenuEditGoto = new JMenuItem(sMenuEditGoToLine);

    JMenu jMenuAttachment = new JMenu(sMenuAttachment);
    JMenuItem jMenuAttachmentAdd = new JMenuItem(sMenuAttachmentAdd);
    JMenuItem jMenuAttachmentRemove = new JMenuItem(sMenuAttachmentRemove);
    JMenuItem jMenuAttachmentRemoveAll = new JMenuItem(sMenuAttachmentRemoveAll);

    JMenu jMenuSignature = new JMenu(sMenuSignature);
    JMenuItem jMenuSignatureCreate = new JMenuItem(sMenuSignatureNew);

    JMenu jMenuAddressbook = new JMenu(sMenuAddressbook);
    JMenuItem jMenuAddressbookOpen = new JMenuItem(sMenuAddressbookOpen);

    JMenu jMenuOptions = new JMenu(sMenuOptions);
    //JMenuItem jMenuOptionsHTML = new JMenuItem(sMenuOptionsHTML);
    JCheckBoxMenuItem jMenuOptionsEncrypt = new JCheckBoxMenuItem(sMenuOptionsEncrypt);
    //JMenuItem jMenuOptionsEncrypt = new JMenuItem(sMenuOptionsEncrypt);
    JMenuItem jMenuOptionsConfiguration = new JMenuItem(sMenuOptionsConfiguration);



    JMenu jMenuHelp = new JMenu(sMenuHelp);
    JMenuItem jMenuHelpHelp = new JMenuItem(sMenuHelp);
    JMenuItem jMenuHelpAbout = new JMenuItem(sMenuHelpAbout);


    // Popup Menus
    JPopupMenu jPopupMenuAttachments = new JPopupMenu();
    JMenuItem jPopupMenuAttachmentsAdd = new JMenuItem(sPopupAttachmentAdd);
    JMenuItem jPopupMenuAttachmentsRemove = new JMenuItem(sPopupAttachmentRemove);
    JMenuItem jPopupMenuAttachmentsRemoveAll = new JMenuItem(sPopupAttachmentRemoveAll);

    JPopupMenu jPopupMenuTo = new JPopupMenu();
    JMenuItem jPopupMenuToRemove = new JMenuItem(sPopupToRemove);
    JMenuItem jPopupMenuToRemoveAll = new JMenuItem(sPopupToRemoveAll);
    JMenuItem jPopupMenuToEdit = new JMenuItem(sPopupToEdit);
    JMenuItem jPopupMenuToTransfer = new JMenuItem(sPopupToTransfer);

    JPopupMenu jPopupMenuCc = new JPopupMenu();
    JMenuItem jPopupMenuCcRemove = new JMenuItem(sPopupCcRemove);
    JMenuItem jPopupMenuCcRemoveAll = new JMenuItem(sPopupCcRemoveAll);
    JMenuItem jPopupMenuCcEdit = new JMenuItem(sPopupCcEdit);
    JMenuItem jPopupMenuCcTransfer = new JMenuItem(sPopupCcTransfer);

    JPopupMenu jPopupMenuText = new JPopupMenu();
    JMenuItem jPopupMenuTextCut = new JMenuItem(sMenuEditCut);
    JMenuItem jPopupMenuTextCopy = new JMenuItem(sMenuEditCopy);
    JMenuItem jPopupMenuTextPaste = new JMenuItem(sMenuEditPaste);
    JMenuItem jPopupMenuTextReplace = new JMenuItem(sMenuEditReplace);
    JMenuItem jPopupMenuTextSelectAll = new JMenuItem(sMenuEditSelectAll);

    // Buttons
    JToolBar jToolBar1Message = new JToolBar();
    JToolBar jToolBar2Edit = new JToolBar();
    JToolBar jToolBar3 = new JToolBar();
    JToolBar jToolBar4 = new JToolBar();
    JToolBar jToolBar5Help = new JToolBar();

    JButton jButton1Send = new JButton();
    JButton jButton2Draft = new JButton();
    JButton jButton3Cut = new JButton();
    JButton jButton4Copy = new JButton();
    JButton jButton5Paste = new JButton();
    JButton jButton6Attachment = new JButton();
    JButton jButton7Signature = new JButton();
    JButton jButton8Addressbook = new JButton();
    JButton jButton9Smile = new JButton();
    JButton jButton10Help = new JButton();


    // oberer Bereich

    JLabel jLabel1To = new JLabel(sLabelTo);
    JLabel jLabel2From = new JLabel(sLabelFrom);
    JLabel jLabel3Subject = new JLabel(sLabelSubject);

    JTextField jTextFieldTo = new JTextField();
    JTextField jTextFieldCc = new JTextField();
    JTextField jTextFieldSubject = new JTextField();
    JComboBox jComboBoxFrom = new JComboBox();
    JScrollPane jScrollPaneTo = new JScrollPane();
    JScrollPane jScrollPaneCc = new JScrollPane();
    JList jListTo = new JList();
    JList jListCc = new JList();
    JButton jButtonCc = new JButton();

    // unterer Bereich
    JSplitPane jSplitPane1 = new JSplitPane();
    JScrollPane jScrollPane1 = new JScrollPane();
    JScrollPane jScrollPane2 = new JScrollPane();
    JTextPane jTextPane1 = new JTextPane();
    JList jListAttachments = new JList();

    JLabel statusBar = new JLabel();



    /**Creates an editor window to write a new message
     * @param fassade the fassade, which gets the editor's result by a MessageContent
     * @param sPropertieFile (not yet realised, only dummy information) the url of the propertie information
     * @param sFromId (not yet realised, only dummy information) the id of the person who sends this message
     * @param sFromText (not yet realised, only dummy information) the address of the person who sends this message
     */
    public EditorFrame(smail.Fassade fassade, String sPropertieFile, String sFromId, String sFromText)
    {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        this.fassade = fassade;
        this.sPropertieFile = sPropertieFile;
        this.sFromId = sFromId;
        //this.sFromText = sFromText;

        smail.server.Account[] accountArray = fassade.getAllAccounts();
        for (int iCounter = 0; iCounter < accountArray.length; iCounter++)
        {
            smail.server.Account ac = accountArray[iCounter];

            if (ac != null)
            {
              String sAddress  = ac.username;
              jComboBoxFrom.addItem(sAddress);
            }
        }
        if(jComboBoxFrom.getItemCount()==0)
        {
          jComboBoxFrom.addItem("Dummy@bla.de");
          jComboBoxFrom.setSelectedItem("Dummy@bla.de");
        }
        else
        {
          jComboBoxFrom.setSelectedItem(sFromText);
        }

        //jComboBoxFrom.addItem(sFromText);

        //Center the window
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        if (frameSize.height > screenSize.height)
        {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width)
        {
            frameSize.width = screenSize.width;
        }
        setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
        setVisible(true);
    }


    /**Creates an editor window to write a new message when called by hyperlink
     * @param fassade the fassade, which gets the editor's result by a MessageContent
     * @param sPropertieFile (not yet realised, only dummy information) the url of the propertie information
     * @param sFromId (not yet realised, only dummy information) the id of the person who sends this message
     * @param sFromText (not yet realised, only dummy information) the address of the person who sends this message
     */
    public EditorFrame(smail.Fassade fassade, String sPropertieFile, String sFromId, String sFromText, String sTo)
    {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        this.fassade = fassade;
        this.sPropertieFile = sPropertieFile;
        this.sFromId = sFromId;
        //this.sFromText = sFromText;
        smail.server.Account[] accountArray = fassade.getAllAccounts();
        for (int iCounter = 0; iCounter < accountArray.length; iCounter++)
        {
            smail.server.Account ac = accountArray[iCounter];

            if (ac != null)
            {
              String sAddress  = ac.username;
              jComboBoxFrom.addItem(sAddress);
            }
        }
        if(jComboBoxFrom.getItemCount()==0)
        {
          jComboBoxFrom.addItem("Dummy@bla.de");
          jComboBoxFrom.setSelectedItem("Dummy@bla.de");
        }
        else
        {
          jComboBoxFrom.setSelectedItem(sFromText);
        }

        vToAddresses.add(sTo);
        jListTo.setListData(vToAddresses);


        //Center the window
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        if (frameSize.height > screenSize.height)
        {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width)
        {
            frameSize.width = screenSize.width;
        }
        setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
        setVisible(true);
    }




    // Hilfsfunktion zur Vermeidung von doppelten Re:
    String correctSubject(String sSubject,String sReplyComment)
    {
      String sTemp = sSubject.trim();

      if (sTemp.indexOf(sReplyComment)==0)
      {
        sTemp =  sTemp.substring(sReplyComment.length()).trim();
      }
      return sTemp;
    }

    // Hilfsfunktion zum ERzeugen von Quotingzeichen
    void addQuotingToken(String sToken)
    {
        javax.swing.text.DefaultStyledDocument doc = (javax.swing.text.DefaultStyledDocument) jTextPane1.getStyledDocument();
        String sText = "";

        try
        {
          sText = doc.getText(0,doc.getLength());
        }
        catch(Exception ex){System.out.println("EditorFrame - Document holen  "+ex);}


        // erste stelle mit token fuellen
        try
        {
          doc.insertString(0, sToken, null);
        }
        catch (Exception ex1)
        {
          System.out.println("Fehler beim einfuegen: 1 "+ex1);
        }

        // alle weiteren stellen mit token fuellen
        int iCorrection = 0 + sToken.length();
        int iCounter = 0;

        while (iCounter < sText.length())
        try
        {
          if ((iCounter > 0)&&(sText.charAt(iCounter-1) == '\n'))
          {
            System.out.println(iCounter);
            doc.insertString(iCounter + iCorrection, sToken, null);
            iCorrection = iCorrection + sToken.length();
          }
          iCounter = iCounter + 1;
        }
        catch (Exception ex1)
        {
          System.out.println("Fehler beim einfuegen: 2 "+ex1);
        }
    }

    /**Creates an editor window when reply or reply to all was chosen, certain textfields will be still filled
     * @param fassade the fassade, which gets the editor's result by a MessageContent
     * @param sPropertieFile (not yet realised, only dummy information) the url of the propertie information
     * @param sFromId (not yet realised, only dummy information) the id of the person who sends this message
     * @param sFromText (not yet realised, only dummy information) the address of the person who sends this message
     */
    public EditorFrame(smail.Fassade fassade, String sPropertieFile, String sFromId, String sFromText, MessageContent content)
    {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        this.fassade = fassade;
        this.sPropertieFile = sPropertieFile;
        this.sFromId = sFromId;

        // i hope, it's allways the person who wrote that mail...
        String sTo = ""+(content.getToAddresses()).elementAt(0);

        vToAddresses = content.getToAddresses();
        jListTo.setListData(vToAddresses);

        smail.server.Account[] accountArray = fassade.getAllAccounts();
        for (int iCounter = 0; iCounter < accountArray.length; iCounter++)
        {
            smail.server.Account ac = accountArray[iCounter];

            if (ac != null)
            {
              String sAddress  = ac.username;
              jComboBoxFrom.addItem(sAddress);
            }
        }

        if(jComboBoxFrom.getItemCount()==0)
        {
          jComboBoxFrom.addItem("Dummy@bla.de");
          jComboBoxFrom.setSelectedItem("Dummy@bla.de");
        }
        else
        {
          jComboBoxFrom.setSelectedItem(sFromText);
        }

        String sTemp = correctSubject(content.getSubject(),"Re:");
        jTextFieldSubject.setText("Re: "+sTemp);

        // content.getText() will be "", when quoting should be excluded
        if (content.getText().length() > 0)
        {
            jTextPane1.setText(content.getText());
            addQuotingToken("> ");
            jTextPane1.setText("On "+content.getDate()+"  "+sTo+" wrote: \n\n"+jTextPane1.getText());
        }


        //Center the window
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        if (frameSize.height > screenSize.height)
        {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width)
        {
            frameSize.width = screenSize.width;
        }
        setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
        setVisible(true);
    }


    /**Creates an editor window when calld from Otutbox or Draft to all was chosen, certain textfields will be still filled
     * @param fassade the fassade, which gets the editor's result by a MessageContent
     * @param sPropertieFile (not yet realised, only dummy information) the url of the propertie information
     * @param sFromId (not yet realised, only dummy information) the id of the person who sends this message
     * @param sFromText (not yet realised, only dummy information) the address of the person who sends this message
     */
    public EditorFrame(smail.Fassade fassade, String sPropertieFile, String sFromId, String sFromText, MessageContent content, int iStatus)
    {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        this.fassade = fassade;
        this.sPropertieFile = sPropertieFile;
        this.sFromId = sFromId;

        // i hope, it's allways the person who wrote that mail...
        String sTo = ""+(content.getToAddresses()).elementAt(0);

        vToAddresses = content.getToAddresses();
        jListTo.setListData(vToAddresses);

        smail.server.Account[] accountArray = fassade.getAllAccounts();
        for (int iCounter = 0; iCounter < accountArray.length; iCounter++)
        {
            smail.server.Account ac = accountArray[iCounter];

            if (ac != null)
            {
              String sAddress  = ac.username;
              jComboBoxFrom.addItem(sAddress);
            }
        }

        if(jComboBoxFrom.getItemCount()==0)
        {
          jComboBoxFrom.addItem("Dummy@bla.de");
          jComboBoxFrom.setSelectedItem("Dummy@bla.de");
        }
        else
        {
          jComboBoxFrom.setSelectedItem(sFromText);
        }

        //String sTemp = correctSubject(content.getSubject(),"Re:");
        jTextFieldSubject.setText(content.getSubject());

        // content.getText() will be "", when quoting should be excluded
        if (content.getText().length() > 0)
        {
            jTextPane1.setText(content.getText());
            //addQuotingToken("> ");
            //jTextPane1.setText("On "+content.getDate()+"  "+sTo+" wrote: \n\n"+jTextPane1.getText());
        }


        //Center the window
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        if (frameSize.height > screenSize.height)
        {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width)
        {
            frameSize.width = screenSize.width;
        }
        setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
        setVisible(true);
    }



    /**Creates an editor window when forward was chosen, certain textfields will be still filled
     * @param fassade the fassade, which gets the editor's result by a MessageContent
     * @param sPropertieFile (not yet realised, only dummy information) the url of the propertie information
     * @param sFromId (not yet realised, only dummy information) the id of the person who sends this message
     * @param sFromText (not yet realised, only dummy information) the address of the person who sends this message
     */

    public EditorFrame(smail.Fassade fassade, String sPropertieFile, String sFromId, String sFromText, MessageContent content, String sForward)
    {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        this.fassade = fassade;
        this.sPropertieFile = sPropertieFile;
        this.sFromId = sFromId;

        smail.server.Account[] accountArray = fassade.getAllAccounts();
        for (int iCounter = 0; iCounter < accountArray.length; iCounter++)
        {
            smail.server.Account ac = accountArray[iCounter];

            if (ac != null)
            {
              String sAddress  = ac.username;
              jComboBoxFrom.addItem(sAddress);
            }
        }
        if(jComboBoxFrom.getItemCount()==0)
        {
          jComboBoxFrom.addItem("Dummy@bla.de");
          jComboBoxFrom.setSelectedItem("Dummy@bla.de");
        }
        else
        {
          jComboBoxFrom.setSelectedItem(sFromText);
        }


        String sTemp = correctSubject(content.getSubject(),"Fwd:");
        jTextFieldSubject.setText("Fwd: "+content.getSubject());
        jTextPane1.setText(" ==================BEGIN FORWARDED MESSAGE================== \n\n"+content.getText()+"\n\n===================END FORWARDED MESSAGE===================");
        vAttachments = content.getAttachments();
        jListAttachments.setListData(vAttachments);

        //Center the window
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        if (frameSize.height > screenSize.height)
        {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width)
        {
            frameSize.width = screenSize.width;
        }
        setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
        setVisible(true);

    }


    // By changing the size of the editor, some components, like Textfields, Lists, etc.
    // will change their size too. This is realized here.
    void contentPane_ancestorResized(HierarchyEvent e)
    {
        int hoehe = getSize().height;
        int breite = getSize().width;
        int divLoc = jSplitPane1.getDividerLocation();
        int divSize = jSplitPane1.getDividerSize();

        jSplitPane1.setBounds(new Rectangle(10, 185, breite-20, hoehe-185-70));
        jSplitPane1.setDividerLocation(hoehe-550 +(220));
        //jTextFieldTo.setText(""+divLoc);

        int untererBereich = jTextPane1.getSize().height;

        jTextFieldTo.setBounds(new Rectangle(50, 40, (breite/2)-60, 20));
        jTextFieldCc.setBounds(new Rectangle((breite/2)+80, 40, (breite/2)-90, 20));
        jTextFieldSubject.setBounds(new Rectangle(75, 155, breite-90, 20));

        jComboBoxFrom.setBounds(new Rectangle(75, 130, breite-90, 20));
        //jListTo.setBounds(new Rectangle(10, 65, (breite/2)-20, 55));
        //jListCc.setBounds(new Rectangle((breite/2)+10, 65, (breite/2)-20, 55));
        jScrollPaneTo.setBounds(new Rectangle(10, 65, (breite/2)-20, 55));
        jScrollPaneCc.setBounds(new Rectangle((breite/2)+10, 65, (breite/2)-20, 55));

        jButtonCc.setBounds(new Rectangle((breite/2)+10, 40, 65, 20));
    }


    // Cc/Bcc-Einstellung
    void jButtonCc_actionPerformed(ActionEvent e)
    {
        if (bCcToggle == false)
        {
            jButtonCc.setText("Cc:");

            if(jTextFieldCc.getText().length()>0)
            {
              vBccAddresses.addElement(jTextFieldCc.getText());
              jTextFieldCc.setText("");
            }

            jListCc.setListData(vCcAddresses);
            bCcToggle = true;
            jTextFieldCc.requestFocus();
        }
        else
        {
            jButtonCc.setText("Bcc:");

            if(jTextFieldCc.getText().length()>0)
            {
              vCcAddresses.addElement(jTextFieldCc.getText());
              jTextFieldCc.setText("");
            }

            jListCc.setListData(vBccAddresses);
            bCcToggle = false;
            jTextFieldCc.requestFocus();
        }
    }

    // auf dem Smiley-Button wird durch anklicken der Smiley veraendert
    // entspricht lachender Smiley: Smiley-umwandlung ist aktiv
    //            trauriger Smiley: Smiley-umwandlung ist inaktiv
    void jButton9Smile_actionPerformed(ActionEvent e)
    {
        if (bSmileToggle == false)
        {
            jButton9Smile.setText(":-)");
            bSmileToggle = true;
        }
        else
        {
            jButton9Smile.setText(":-(");
            bSmileToggle = false;
        }
    }

    // Automatische Smiley-Umwandlung ein :-) wird in *smile* ersetzt
    void jTextPane1_keyReleased(KeyEvent e)
    {
        if (bSmileToggle == true)
        {
            replace(":-)", "*smile*");
            replace(":-(", "*cry*");
        }
    }



    // Type <enter> to get the String of the TextField into the List

    // Cc- Eintraege
    void jTextFieldCc_actionPerformed(ActionEvent e)
    {
        if (bCcToggle == true)
        {
            vCcAddresses.addElement(jTextFieldCc.getText());
            jTextFieldCc.setText("");
            jListCc.setListData(vCcAddresses);
            jTextFieldCc.requestFocus();
        }
        else
        {
            vBccAddresses.addElement(jTextFieldCc.getText());
            jTextFieldCc.setText("");
            jListCc.setListData(vBccAddresses);
            jTextFieldCc.requestFocus();
        }
    }

    // to call from the addressbook
    /** This procedure adds an e-mail-address to the cc-list
     *  @param address is an object that contains name and e-mailaddress
     *  @see Address
     */
    public void addAddressCc(Address address)
    {
        if (bCcToggle == true)
        {
            vCcAddresses.addElement(address.getEmail());
            jListCc.setListData(vCcAddresses);
        }
        else
        {
            bCcToggle = true;
            jButtonCc.setText("Cc:");
            vCcAddresses.addElement(address.getEmail());
            jListCc.setListData(vCcAddresses);
        }
    }

    // procedure to call from the addressbook
    /** This procedure adds an e-mail-address to the bcc-list
     *  @param address is an object that contains name and e-mailaddress
     *  @see Address
     */
    public void addAddressBcc(Address address)
    {
        if (bCcToggle == false)
        {
            vBccAddresses.addElement(address.getEmail());
            jListCc.setListData(vBccAddresses);
        }
        else
        {
            bCcToggle = false;
            jButtonCc.setText("Bcc:");
            vBccAddresses.addElement(address.getEmail());
            jListCc.setListData(vBccAddresses);
        }
    }


    // To-Eintraege
    void jTextFieldTo_actionPerformed(ActionEvent e)
    {
        vToAddresses.addElement(jTextFieldTo.getText());
        jTextFieldTo.setText("");
        jListTo.setListData(vToAddresses);
        jTextFieldTo.requestFocus();
    }

    // procedure to call from the addressbook
    /** This procedure adds an e-mail-address to the to-list
     *  @param address is an object that contains name and e-mailaddress
     *  @see Address
     */
    public void addAddressTo(Address address)
    {
        vToAddresses.addElement(address.getEmail());
        jListTo.setListData(vToAddresses);
    }

    // Subject (press enter go to the textpane
    void jTextFieldSubject_actionPerformed(ActionEvent e)
    {
        jTextPane1.requestFocus();
    }


    /** returns the content of the message which was written by the editor
    *  @see smail.gui.MessageContent
    */
    public MessageContent getMessageContent()
    {
        MessageContent result;
        boolean bCcAddresses = false;
        boolean bBccAddresses = false;
        boolean bAttachments = false;

        if(vCcAddresses.size()>0) bCcAddresses = true;
        if(vBccAddresses.size()>0) bBccAddresses = true;
        if(vAttachments.size()>0) bAttachments = true;

        //public MessageContent(String sFrom,Vector vToAddresses, Vector vCcAdresses, Vector vBccAdresses, boolean bCcFlag, String sSubject, String sText, Vector vAttachments, boolean  bCcAddresses, boolean bBccAddresses, boolean  bAttachments)
        result = new MessageContent(""+jComboBoxFrom.getSelectedItem(),vToAddresses, vCcAddresses,vBccAddresses,jTextFieldSubject.getText(), jTextPane1.getText(), vAttachments, bCcAddresses, bBccAddresses, bAttachments);
        return result;
    }


    // Component initialization
    private void jbInit() throws Exception
    {
        // image holen
        Image image = new ImageIcon(getClass().getResource("Images/icon16.png")).getImage();
        this.setIconImage(image);

        contentPane = (JPanel) this.getContentPane();
        contentPane.setLayout(null);

        this.setTitle(title);

        // Menu
        this.setJMenuBar(jMenuBar1);

        // Message

        // Send
        jMenuMessageSend.setAccelerator(javax.swing.KeyStroke.getKeyStroke(83, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuMessageSend.addActionListener(new java.awt.event.ActionListener() {public void actionPerformed(ActionEvent e) {jMenuMessageSend_actionPerformed(e);}});

        // Draft
        jMenuMessageDraft.setAccelerator(javax.swing.KeyStroke.getKeyStroke(68, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuMessageDraft.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuMessageDraft_actionPerformed(e);}});

        // Clear
        jMenuMessageClear.setAccelerator(javax.swing.KeyStroke.getKeyStroke(48, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuMessageClear.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuMessageClear_actionPerformed(e);}});

        // Close
        jMenuMessageClose.setAccelerator(javax.swing.KeyStroke.getKeyStroke(81, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuMessageClose.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){jMenuMessageClose_actionPerformed(e);}});

        // Edit
        // Cut
        jMenuEditCut.setAccelerator(javax.swing.KeyStroke.getKeyStroke(88, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuEditCut.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditCut_actionPerformed(e);}});

        // Copy
        jMenuEditCopy.setAccelerator(javax.swing.KeyStroke.getKeyStroke(67, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuEditCopy.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditCopy_actionPerformed(e);}});

        // Paste
        jMenuEditPaste.setAccelerator(javax.swing.KeyStroke.getKeyStroke(86, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuEditPaste.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditPaste_actionPerformed(e);}});

        // Select All
        jMenuEditSelectAll.setAccelerator(javax.swing.KeyStroke.getKeyStroke(65, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuEditSelectAll.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditSelectAll_actionPerformed(e);}});

        // Find
        jMenuEditFind.setAccelerator(javax.swing.KeyStroke.getKeyStroke(70, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuEditFind.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditFind_actionPerformed(e);}});

        // Replace
        jMenuEditReplace.setAccelerator(javax.swing.KeyStroke.getKeyStroke(82, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuEditReplace.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditReplace_actionPerformed(e);}});

        // Go to line
        jMenuEditGoto.setAccelerator(javax.swing.KeyStroke.getKeyStroke(71, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuEditGoto.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditGoto_actionPerformed(e);}});


        // Attachment
        // Add Attachment
        jMenuAttachmentAdd.setAccelerator(javax.swing.KeyStroke.getKeyStroke(65, java.awt.event.KeyEvent.ALT_MASK, false));
        jMenuAttachmentAdd.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuAttachmentAdd_actionPerformed(e);}});

        // Remove Attachment
        jMenuAttachmentRemove.setAccelerator(javax.swing.KeyStroke.getKeyStroke(82, java.awt.event.KeyEvent.ALT_MASK, false));
        jMenuAttachmentRemove.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuAttachmentRemove_actionPerformed(e);}});

        // Remove All Attachments
        //jMenuAttachmentRemoveAll.setAccelerator(javax.swing.KeyStroke.getKeyStroke(82, java.awt.event.KeyEvent.ALT_MASK, false));
        jMenuAttachmentRemoveAll.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuAttachmentRemoveAll_actionPerformed(e);}});

        // Signature
        // Create New Signature
        jMenuSignatureCreate.setAccelerator(javax.swing.KeyStroke.getKeyStroke(83, java.awt.event.KeyEvent.ALT_MASK, false));
        jMenuSignatureCreate.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuSignatureCreate_actionPerformed(e);}});

        jMenuSignature.add(jMenuSignatureCreate);
        int iMenuCounter = 0;
        // counter holen, bzw erzeugen
        if(propMan.get("smail.gui.signature.counter") != null)
        {
            try
            {
                iMenuCounter = Integer.parseInt(propMan.get("smail.gui.signature.counter"));
            }
            catch(Exception ex)
            {
                System.out.println("smail.gui.EditorFrame - Signature - no counter in Property");
            }
        }

        System.out.println(""+iMenuCounter);

        for(int iCounter = 1; iCounter <= iMenuCounter; iCounter++)
        {
          JMenuItem sTemp1 = new JMenuItem(propMan.get("smail.gui.signature.description"+iCounter));
          jMenuSignature.add(sTemp1);

          sTemp1.addActionListener
          (new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuItem1_actionPerformed(e);        }});

        }



        //Addressbook
        // Open Addressbook
        jMenuAddressbookOpen.setAccelerator(javax.swing.KeyStroke.getKeyStroke(66, java.awt.event.KeyEvent.CTRL_MASK, false));
        jMenuAddressbookOpen.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuAddressbookOpen_actionPerformed(e);}});

        // Options
        //jMenuOptions.setText("Options");
        //jMenuOptionsHTML.setText("HTML");
        //jMenuOptionsEncrypt.setText("Encrypt Message");
        jMenuOptionsEncrypt.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuOptionsEncrypt_actionPerformed(e);}});
        //jMenuOptionsConfiguration.setText("Configuratione");

        // Help
        jMenuHelpHelp.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1,0, false));
        jMenuHelpHelp.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){jMenuHelpHelp_actionPerformed(e);}});
        // About
        jMenuHelpAbout.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){jMenuHelpAbout_actionPerformed(e);}});



        // Add Menus to Menubar
        jMenuBar1.add(jMenuMessage);
        jMenuBar1.add(jMenuEdit);
        jMenuBar1.add(jMenuAttachment);
        jMenuBar1.add(jMenuSignature);
        jMenuBar1.add(jMenuAddressbook);
        jMenuBar1.add(jMenuOptions);
        jMenuBar1.add(jMenuHelp);


        // Add MenuItems to Menus
        jMenuMessage.add(jMenuMessageSend);
        jMenuMessage.add(jMenuMessageDraft);
        jMenuMessage.add(jMenuMessageClear);
        jMenuMessage.add(jMenuMessageClose);

        jMenuEdit.add(jMenuEditCut);
        jMenuEdit.add(jMenuEditCopy);
        jMenuEdit.add(jMenuEditPaste);
        jMenuEdit.add(jMenuEditSelectAll);
        jMenuEdit.add(jMenuEditFind);
        jMenuEdit.add(jMenuEditReplace);
        jMenuEdit.add(jMenuEditGoto);

        jMenuAttachment.add(jMenuAttachmentAdd);
        jMenuAttachment.add(jMenuAttachmentRemove);
        jMenuAttachment.add(jMenuAttachmentRemoveAll);

        //jMenuSignature.add(jMenuSignatureCreate);

        jMenuAddressbook.add(jMenuAddressbookOpen);

        //jMenuOptions.add(jMenuOptionsHTML);
        jMenuOptions.add(jMenuOptionsEncrypt);
        jMenuOptions.add(jMenuOptionsConfiguration);

        jMenuHelp.add(jMenuHelpHelp);
        jMenuHelp.add(jMenuHelpAbout);


        // Popup Menu

        //jPopupMenuToRemove.setText("Remove");
        jPopupMenuToRemove.addActionListener(new java.awt.event.ActionListener() {public void actionPerformed(ActionEvent e) {jPopupMenuToRemove_actionPerformed(e);}});
        //jPopupMenuToRemoveAll.setText("Remove All");
        jPopupMenuToRemoveAll.addActionListener(new java.awt.event.ActionListener() {public void actionPerformed(ActionEvent e) {jPopupMenuToRemoveAll_actionPerformed(e);}});
        //jPopupMenuToEdit.setText("Edit");
        jPopupMenuToEdit.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuToEdit_actionPerformed(e);}});
        //jPopupMenuToTransfer.setText("Transfer to Cc/Bcc Field");
        jPopupMenuToTransfer.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuToTransfer_actionPerformed(e);}});
        jListTo.addMouseListener(new java.awt.event.MouseAdapter() {public void mousePressed(MouseEvent e) {jListTo_mousePressed(e);}});

        //jPopupMenuCcRemove.setText("Remove");
        jPopupMenuCcRemove.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuCcRemove_actionPerformed(e);}});
        //jPopupMenuCcRemoveAll.setText("Remove All");
        jPopupMenuCcRemoveAll.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuCcRemoveAll_actionPerformed(e);}});
        //jPopupMenuCcEdit.setText("Edit");
        jPopupMenuCcEdit.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuCcEdit_actionPerformed(e);}});
        //jPopupMenuCcTransfer.setText("Transfer to To Field");
        jPopupMenuCcTransfer.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuCcTransfer_actionPerformed(e);}});
        jListCc.addMouseListener(new java.awt.event.MouseAdapter(){public void mousePressed(MouseEvent e){jListCc_mousePressed(e);}});


        //jPopupMenuAttachmentsAdd.setText("Add");
        jPopupMenuAttachmentsAdd.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuAttachmentAdd_actionPerformed(e);}});
        //jPopupMenuAttachmentsRemove.setText("Remove");
        jPopupMenuAttachmentsRemove.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuAttachmentRemove_actionPerformed(e);}});
        //jPopupMenuAttachmentsRemoveAll.setText("Remove All");
        jPopupMenuAttachmentsRemoveAll.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuAttachmentRemoveAll_actionPerformed(e);}});
        jListAttachments.addMouseListener(new java.awt.event.MouseAdapter() {public void mousePressed(MouseEvent e) {jListAttachments_mousePressed(e);}});

        jTextPane1.addMouseListener(new java.awt.event.MouseAdapter(){public void mousePressed(MouseEvent e){jTextPane1_mousePressed(e);}});
        jPopupMenuTextCut.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditCut_actionPerformed(e);}});
        jPopupMenuTextCopy.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditCopy_actionPerformed(e);}});
        jPopupMenuTextPaste.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditPaste_actionPerformed(e);}});
        jPopupMenuTextReplace.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditReplace_actionPerformed(e);}});
        jPopupMenuTextSelectAll.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditSelectAll_actionPerformed(e);}});


        jPopupMenuTo.add(jPopupMenuToRemove);
        jPopupMenuTo.add(jPopupMenuToRemoveAll);
        jPopupMenuTo.add(jPopupMenuToEdit);
        jPopupMenuTo.add(jPopupMenuToTransfer);
        jPopupMenuCc.add(jPopupMenuCcRemove);
        jPopupMenuCc.add(jPopupMenuCcRemoveAll);
        jPopupMenuCc.add(jPopupMenuCcEdit);
        jPopupMenuCc.add(jPopupMenuCcTransfer);
        jPopupMenuAttachments.add(jPopupMenuAttachmentsAdd);
        jPopupMenuAttachments.add(jPopupMenuAttachmentsRemove);
        jPopupMenuAttachments.add(jPopupMenuAttachmentsRemoveAll);
        jPopupMenuText.add(jPopupMenuTextCut);
        jPopupMenuText.add(jPopupMenuTextCopy);
        jPopupMenuText.add(jPopupMenuTextPaste);
        jPopupMenuText.add(jPopupMenuTextSelectAll);
        jPopupMenuText.add(jPopupMenuTextReplace);

        // Buttons
        jToolBar1Message.setBorderPainted(false);
        jToolBar2Edit.setBorderPainted(false);
        jToolBar3.setBorderPainted(false);
        jToolBar4.setBorderPainted(false);
        jToolBar5Help.setBorderPainted(false);


        // Setting Images, Tooltips,Sizes, ActionListeners to the Buttons
        jButton1Send.setIcon(new ImageIcon(EditorFrame.class.getResource("Images/send.png")));
        jButton1Send.setToolTipText(sTooltipSend);
        jButton1Send.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuMessageSend_actionPerformed(e);}});

        jButton2Draft.setIcon(new ImageIcon(EditorFrame.class.getResource("Images/save.png")));
        jButton2Draft.setToolTipText(sTooltipDraft);
        jButton2Draft.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuMessageDraft_actionPerformed(e);}});

        jButton3Cut.setIcon(new ImageIcon(EditorFrame.class.getResource("Images/cut.png")));
        jButton3Cut.setToolTipText(sTooltipCut);
        jButton3Cut.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditCut_actionPerformed(e);}});

        jButton4Copy.setIcon(new ImageIcon(EditorFrame.class.getResource("Images/copy.png")));
        jButton4Copy.setToolTipText(sTooltipCopy);
        jButton4Copy.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditCopy_actionPerformed(e);}});

        jButton5Paste.setIcon(new ImageIcon(EditorFrame.class.getResource("Images/paste.png")));
        jButton5Paste.setToolTipText(sTooltipPaste);
        jButton5Paste.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuEditPaste_actionPerformed(e);}});

        jButton6Attachment.setIcon(new ImageIcon(EditorFrame.class.getResource("Images/attachment.png")));
        jButton6Attachment.setToolTipText(sTooltipAttachment);
        jButton6Attachment.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuAttachmentAdd_actionPerformed(e);}});

        jButton7Signature.setIcon(new ImageIcon(EditorFrame.class.getResource("Images/signature.png")));
        jButton7Signature.setToolTipText(sTooltipSignature);
        jButton7Signature.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuSignatureCreate_actionPerformed(e);}});

        jButton8Addressbook.setIcon(new ImageIcon(EditorFrame.class.getResource("Images/addressbok.png")));
        jButton8Addressbook.setToolTipText(sTooltipAddressbook);
        jButton8Addressbook.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuAddressbookOpen_actionPerformed(e);}});

        jButton9Smile.setText(":-)");
        jButton9Smile.setToolTipText(sTooltipSmile);
        jButton9Smile.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jButton9Smile_actionPerformed(e);}});

        jButton10Help.setIcon(new ImageIcon(EditorFrame.class.getResource("Images/help.png")));
        jButton10Help.setToolTipText(sTooltipHelp);
        jButton10Help.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuHelpHelp_actionPerformed(e);}});

        // Adding Buttons to the Toolbars
        jToolBar1Message.add(jButton1Send, null);
        jToolBar1Message.add(jButton2Draft, null);
        jToolBar2Edit.add(jButton3Cut, null);
        jToolBar2Edit.add(jButton4Copy, null);
        jToolBar2Edit.add(jButton5Paste, null);
        jToolBar3.add(jButton6Attachment, null);
        jToolBar3.add(jButton7Signature, null);
        jToolBar3.add(jButton8Addressbook, null);
//        jToolBar4.add(jButton9Smile, null);
        jToolBar5Help.add(jButton10Help, null);


        // oberer Bereich

        // Labels
        //jLabel1To.setText("To:");
        //jLabel2From.setText("From:");
        //jLabel3Subject.setText("Subject");
        jButtonCc.setText("Cc:");

        // TextFields
        // Color, Editability
        jTextFieldTo.setBackground(Color.white);
        jTextFieldTo.setEditable(true);
        jTextFieldTo.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jTextFieldTo_actionPerformed(e);}});

        jTextFieldCc.setBackground(Color.white);
        jTextFieldCc.setEditable(true);
        jTextFieldCc.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jTextFieldCc_actionPerformed(e);}});

        jTextFieldSubject.setBackground(Color.white);
        jTextFieldSubject.setEditable(true);
        jTextFieldSubject.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jTextFieldSubject_actionPerformed(e);}});

        jButtonCc.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jButtonCc_actionPerformed(e);}});

        jComboBoxFrom.setBackground(Color.white);
        jComboBoxFrom.setEditable(false);

        // Lists
        jListTo.setBackground(Color.white);
        jListTo.setBorder(BorderFactory.createEtchedBorder());

        jListCc.setBackground(Color.white);
        jListCc.setBorder(BorderFactory.createEtchedBorder());

        jScrollPaneTo.getViewport().add(jListTo, null);
        jScrollPaneCc.getViewport().add(jListCc, null);

        // unterer Bereich
        jSplitPane1.setOrientation(JSplitPane.VERTICAL_SPLIT);
        //jSplitPane1.setBounds(new Rectangle(20, 193, 638, 306));
        //jSplitPane1.setDividerLocation(200);

        jTextPane1.setBackground(Color.white);
        jTextPane1.setSelectionColor(SystemColor.black);
        jTextPane1.setEditable(true);
        jTextPane1.setBorder(BorderFactory.createEtchedBorder());
        jTextPane1.setFont(new java.awt.Font("SansSerif", 0, 13));
        jTextPane1.addKeyListener(new java.awt.event.KeyAdapter(){public void keyReleased(KeyEvent e){jTextPane1_keyReleased(e);}});


        jListAttachments.setBackground(Color.white);
        jListAttachments.setBorder(BorderFactory.createEtchedBorder());

        jScrollPane2.getViewport().add(jListAttachments, null);
        jScrollPane1.getViewport().add(jTextPane1, null);
        jSplitPane1.add(jScrollPane1, JSplitPane.TOP);
        jSplitPane1.add(jScrollPane2, JSplitPane.BOTTOM);


        // set Focus
        jTextFieldSubject.setNextFocusableComponent(jTextPane1);
        jTextFieldTo.setNextFocusableComponent(jTextFieldCc);
        jTextFieldCc.setNextFocusableComponent(jTextFieldSubject);



        // Sizes of the components
        this.setSize(new Dimension(601, 550));
        statusBar.setBounds(new Rectangle(0, 376, 541, 17));

/*        jToolBar1Message.setBounds(new Rectangle(10, 0,60, 30));
        jToolBar2Edit.setBounds(new Rectangle(90, 0, 90, 30));
        jToolBar3.setBounds(new Rectangle(200, 0, 90, 30));
        jToolBar4.setBounds(new Rectangle(310, 0, 60, 30));
        jToolBar5Help.setBounds(new Rectangle(390, 0, 35, 30));
*/
        jToolBar1Message.setBounds(new Rectangle(10, 0,80, 30));
        jToolBar2Edit.setBounds(new Rectangle(100, 0, 110, 30));
        jToolBar3.setBounds(new Rectangle(220, 0, 110, 30));
        jToolBar4.setBounds(new Rectangle(340, 0, 50, 30));
        jToolBar5Help.setBounds(new Rectangle(400, 0, 50, 30));


        jButton1Send.setPreferredSize(new Dimension(27, 27));
        jButton2Draft.setPreferredSize(new Dimension(27, 27));
        jButton3Cut.setPreferredSize(new Dimension(27, 27));
        jButton4Copy.setPreferredSize(new Dimension(27, 27));
        jButton5Paste.setPreferredSize(new Dimension(27, 27));
        jButton6Attachment.setPreferredSize(new Dimension(27, 27));
        jButton7Signature.setPreferredSize(new Dimension(27, 27));
        jButton8Addressbook.setPreferredSize(new Dimension(27, 27));
        jButton9Smile.setPreferredSize(new Dimension(27, 30));
        jButton10Help.setPreferredSize(new Dimension(27, 27));

        jLabel1To.setBounds(new Rectangle(10, 40, 50, 20));
        jLabel2From.setBounds(new Rectangle(10, 130, 50, 20));
        jLabel3Subject.setBounds(new Rectangle(10, 155, 50, 20));

        jTextFieldTo.setBounds(new Rectangle(50, 40, 240, 20));
        jTextFieldCc.setBounds(new Rectangle(380, 40, 210, 20));
        jTextFieldSubject.setBounds(new Rectangle(75, 155, 520, 20));

        jComboBoxFrom.setBounds(new Rectangle(75, 130, 520, 20));
        //jListTo.setBounds(new Rectangle(10, 65, 280, 55));
        //jListCc.setBounds(new Rectangle(310, 65, 280, 55));
        jScrollPaneTo.setBounds(new Rectangle(10, 65, 280, 55));
        jScrollPaneCc.setBounds(new Rectangle(310, 65, 280, 55));
        jButtonCc.setBounds(new Rectangle(310, 40, 65, 20));

        jSplitPane1.setBounds(new Rectangle(10, 185, 580, 280));
        jSplitPane1.setDividerLocation(220);



        contentPane.addHierarchyBoundsListener(new java.awt.event.HierarchyBoundsAdapter() {
            public void ancestorResized(HierarchyEvent e) {
                contentPane_ancestorResized(e);
            }
        });

        // Add components to the contentPane

        // ToolBars
        contentPane.add(jToolBar1Message, null);
        contentPane.add(jToolBar2Edit, null);
        contentPane.add(jToolBar3, null);
        contentPane.add(jToolBar4, null);
        contentPane.add(jToolBar5Help, null);
        // Labels, Button
        contentPane.add(jLabel1To, null);
        contentPane.add(jLabel2From, null);
        contentPane.add(jLabel3Subject, null);
        contentPane.add(jButtonCc, null);
        // Textfields, Combobox
        contentPane.add(jTextFieldTo, null);
        contentPane.add(jTextFieldCc, null);
        contentPane.add(jTextFieldSubject, null);
        contentPane.add(jComboBoxFrom, null);

        // Lists
        //contentPane.add(jListTo, null);
        //contentPane.add(jListCc, null);
        contentPane.add(jScrollPaneTo, null);
        contentPane.add(jScrollPaneCc, null);

        contentPane.add(jSplitPane1, null);

        contentPane.add(statusBar, null);
    }



    // Performances of the Actions
    // Prozeduren zu den einzelnen Menupunkten


    // Menu Message
    // Message -> Send
    void jMenuMessageSend_actionPerformed(ActionEvent e)
    {
        if (jTextFieldTo.getText().length() > 0)
        {
            vToAddresses.addElement(jTextFieldTo.getText());
            jTextFieldTo.setText("");
            jListTo.setListData(vToAddresses);
        }

        if (jTextFieldCc.getText().length() > 0)
        {
            if(bCcToggle == true)
            {
              vCcAddresses.addElement(jTextFieldCc.getText());
              jTextFieldCc.setText("");
              jListCc.setListData(vCcAddresses);
            }
            else
            {
              vBccAddresses.addElement(jTextFieldCc.getText());
              jTextFieldCc.setText("");
              jListCc.setListData(vBccAddresses);
            }

        }

        if (vToAddresses.size() > 0)
        {
            MessageContent content = getMessageContent();
            if (bEncryptToggle == true)
            {
              content.setEncryptBoolean(true);
            }
            else
            {
              content.setEncryptBoolean(false);
            }

            // Damit ich das Editorergebnis testen kann
            try
            {
                System.out.println("---------------------------");
                System.out.println("To: ");
                for(int iCounter=0; iCounter < content.getToAddresses().size(); iCounter++)
                {
                    System.out.println(""+content.getToAddresses().elementAt(iCounter));
                }
                System.out.println("---------------------------");
                System.out.println("Cc-Boolean: "+content.getCcAddressesBoolean());
                System.out.println("Cc: ");
                for(int iCounter=0; iCounter < content.getCcAddresses().size(); iCounter++)
                {
                    System.out.println(""+content.getCcAddresses().elementAt(iCounter));
                }
                System.out.println("---------------------------");
                System.out.println("Bcc-Boolean: "+content.getBccAddressesBoolean());
                System.out.println("Bcc: ");
                for(int iCounter=0; iCounter < content.getBccAddresses().size(); iCounter++)
                {
                    System.out.println(""+content.getBccAddresses().elementAt(iCounter));
                }
                System.out.println("---------------------------");
                System.out.println("From:     "+content.getFrom());
                System.out.println("---------------------------");
                System.out.println("Subject:  "+content.getSubject());
                System.out.println("---------------------------");
                System.out.println("Text:     "+content.getText());
                System.out.println("---------------------------");
                System.out.println("Attachment-Boolean: "+content.getAttachmentsBoolean());
                System.out.println("Attachments: ");
                for(int iCounter=0; iCounter < content.getAttachments().size(); iCounter++)
                {
                    System.out.println(""+content.getAttachments().elementAt(iCounter));
                }
                System.out.println("---------------------------");
                System.out.println("Encrypt-Boolean: "+content.getEncryptBoolean());

                System.out.println("===========================");
            }
            catch (Exception ex)
            {
    	        System.out.println ("unknown status of editor (DELETE or WORKING)");
            }

            // Ende der Testausgabe

            // Zum Testen auskommentieren, gibt noch Fehler weil Fassade nicht wirklich uebergeben wird (null)
            fassade.storeNewMail(content);
            dispose();
        }
        else
        {
            ErrorFrame error = new ErrorFrame();
            error.jLabelInfo.setText(sLabelError);
            error.setTitle(sTitleError);
            error.jButtonOk.requestFocus();
            error.show();
        }
    }

    // Message -> Draft
    void jMenuMessageDraft_actionPerformed(ActionEvent e)
    {
        if (jTextFieldTo.getText().length() > 0)
        {
            vToAddresses.addElement(jTextFieldTo.getText());
            jTextFieldTo.setText("");
            jListTo.setListData(vToAddresses);
        }

        if(bCcToggle == true)
        {
            vCcAddresses.addElement(jTextFieldCc.getText());
            jTextFieldCc.setText("");
            jListCc.setListData(vCcAddresses);
        }
        else
        {
            vBccAddresses.addElement(jTextFieldCc.getText());
            jTextFieldCc.setText("");
            jListCc.setListData(vBccAddresses);
        }


        if (vToAddresses.size() > 0)
        {
            MessageContent content = getMessageContent();
            fassade.storeNewMail(content,"");
            dispose();
        }
        else
        {
            ErrorFrame error = new ErrorFrame();
            error.jLabelInfo.setText(sLabelError);
            error.setTitle(sTitleError);
            error.show();
        }
    }

    // Message -> Clear
    void jMenuMessageClear_actionPerformed(ActionEvent e)
    {
        jTextFieldTo.setText("");
        jTextFieldCc.setText("");
        jTextFieldSubject.setText("");
        vToAddresses.removeAllElements();
        jListTo.setListData(vToAddresses);
        vCcAddresses.removeAllElements();
        vBccAddresses.removeAllElements();
        jListCc.setListData(vCcAddresses);
        vAttachments.removeAllElements();
        jListAttachments.setListData(vAttachments);
        jTextPane1.setText("");
        jTextPane1.requestFocus();
    }

    // Message -> Close
    public void jMenuMessageClose_actionPerformed(ActionEvent e)
    {
        dispose();
    }

    // Menu Edit
    // Edit -> Cut
    void jMenuEditCut_actionPerformed(ActionEvent e)
    {
        jTextPane1.cut();
        jTextPane1.requestFocus();
    }

    // Edit -> Copy
    void jMenuEditCopy_actionPerformed(ActionEvent e)
    {
        jTextPane1.copy();
        jTextPane1.requestFocus();
    }

    // Edit -> Paste
    void jMenuEditPaste_actionPerformed(ActionEvent e)
    {
        jTextPane1.paste();
        jTextPane1.requestFocus();
    }

    // Edit -> Select All
    void jMenuEditSelectAll_actionPerformed(ActionEvent e)
    {
        jTextPane1.selectAll();
        jTextPane1.requestFocus();
    }


    // (Hilfsprozedur) Edit -> Find
    void find(String sFind)
    {
        int iIndex = -2;
        String sText = "";

        javax.swing.text.DefaultStyledDocument doc = (javax.swing.text.DefaultStyledDocument) jTextPane1.getStyledDocument();

        try
        {
            sText = doc.getText(0,doc.getLength());
        }
        catch(Exception e){}

        iIndex = sText.indexOf(sFind,iIndexOld);
        if (iIndex > -1)
        {
            jTextPane1.select(iIndex,iIndex + sFind.length());

            iIndexOld = iIndex + 1;
            sFindOld = sFind;
        }
        else
        {
            iIndexOld = 0;
            iIndex = 0;
        }
    }

    // Edit -> Find
    void jMenuEditFind_actionPerformed(ActionEvent e)
    {
        FindFrame find = new FindFrame(sFindOld);
        find.show();

        if(find.bFindStatus == true)
        {
            String sFind = find.sFind;
            int iIndex = -2;

            // bei neuer Textsuche wird wieder am Anfang des Textes gesucht
            if (sFindOld.equals(sFind)== false)
            {
                iIndex = 0;
                iIndexOld = 0;
            }
            find(sFind);
        }
    }

    // (Hilfsprozedur) Edit -> Replace
    void replace(String sFind, String sReplace)
    {
        int iIndex = -2;
        String sNewText = "";
        String sText = "";

        javax.swing.text.DefaultStyledDocument doc = (javax.swing.text.DefaultStyledDocument) jTextPane1.getStyledDocument();

        try
        {
            sText = doc.getText(0,doc.getLength());
        }
        catch(Exception e){}

        iIndex = sText.indexOf(sFind,iIndexOld);

        if (iIndex > -1)
        {
            sNewText = sText.substring(0,iIndex) + sReplace + sText.substring(iIndex+sFind.length());

            jTextPane1.setText(sNewText);
            sFindOld = sFind;
            sReplaceOld = sReplace;
        }
    }

    // Edit -> Replace
    void jMenuEditReplace_actionPerformed(ActionEvent e)
    {
        ReplaceFrame replace = new ReplaceFrame(sFindOld, sReplaceOld);
        replace.show();

        if(replace.bReplaceStatus == true)
        {
            String sFind = replace.sFind;
            String sReplace = replace.sReplace;
            int iIndex = -2;

            if (sFindOld.equals(sFind)== false)
            {
                iIndex = 0;
                iIndexOld = 0;
            }
            replace(sFind, sReplace);
        }
    }


    // Edit -> Go to line
    void jMenuEditGoto_actionPerformed(ActionEvent e)
    {
        GoToLineFrame goToLine = new GoToLineFrame();
        goToLine.show();

        if(goToLine.bGoToLineStatus == true)
        {
            int iLine = goToLine.iLine;
            int iIndex = 0;
            int iCounter = 1;

            String sText ="";

            javax.swing.text.DefaultStyledDocument doc = (javax.swing.text.DefaultStyledDocument) jTextPane1.getStyledDocument();

            try
            {
                sText = doc.getText(0,doc.getLength());
            }
            catch(Exception ex){}

            while((iIndex != -1)&&(iCounter <iLine))
            {
                iIndex = sText.indexOf("\n",iIndex + 1);
                iCounter = iCounter + 1;
            }

            if(iIndex > 0)
                jTextPane1.setCaretPosition(iIndex + 1);
            else
                jTextPane1.setCaretPosition(0);
        }
    }


    // Menu Attachment
    // Attachment -> Add Attachment
    void jMenuAttachmentAdd_actionPerformed(ActionEvent e)
    {
        FileDialog fd = new FileDialog(this, "Load Attachment", FileDialog.LOAD);
        fd.show();

        String sDirectory = fd.getDirectory();
        String sFile = fd.getFile();

        if ((sFile!=null)&&(sDirectory!=null))
        {
        	vAttachments.addElement(sDirectory+sFile);
        	jListAttachments.setListData(vAttachments);
        }
    }


    // Attachment -> Remove Attachment
    void jMenuAttachmentRemove_actionPerformed(ActionEvent e)
    {
        int iIndex = jListAttachments.getSelectedIndex();
        if (iIndex != -1)
        {
          vAttachments.removeElementAt(iIndex);
          jListAttachments.setListData(vAttachments);
        }
    }


    // Attachment -> Remove All Attachments
    void jMenuAttachmentRemoveAll_actionPerformed(ActionEvent e)
    {
        vAttachments.removeAllElements();
        jListAttachments.setListData(vAttachments);
    }


    // Menu Signature
    // Signature -> Create New Signature
    void jMenuSignatureCreate_actionPerformed(ActionEvent e)
    {
        SignatureFrame signature = new SignatureFrame();
        signature.show();

        if(signature.bSignatureStatus == true)
        {
            String sDescription = signature.sDescription;
            String sText = signature.sText;

            // create a new MenuItem
            JMenuItem jMenuNewSignature = new JMenuItem();
            // the new menuitem gets the text of the description
            jMenuNewSignature.setText(sDescription);
            // add new item to the menu "Signature"
            jMenuSignature.add(jMenuNewSignature);

            //jTextPane1.setText(sText);

            jMenuNewSignature.addActionListener
            (new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jMenuItem1_actionPerformed(e);        }});
        }
    }


    void jMenuItem1_actionPerformed(ActionEvent e)
    {
        // System.out.println("Menue: "+e.getActionCommand());
        String sText = jTextPane1.getText();
        String sSignature = propMan.get("smail.gui.signature."+e.getActionCommand());
        jTextPane1.setText(sText+"\n"+sSignature);
    }



    // Menu Addressbook
    // Addressbook -> Open Addressbook
    void jMenuAddressbookOpen_actionPerformed(ActionEvent e)
    {
        Addressbook book = new Addressbook(this, fassade);
	System.out.println("Addressbook opened with Addressbook(this)");
        book.show();
    }

    // Menu Options
    // Options -> Encrypt Message
    void jMenuOptionsEncrypt_actionPerformed(ActionEvent e)
    {
        if (bEncryptToggle == false)
        {
            System.out.println("Encrypt auf true gesetzt, es wird verschluesselt");
            bEncryptToggle = true;
        }
        else
        {
            System.out.println("Encrypt auf false gesetzt, es wird nicht verschluesselt");
            bEncryptToggle = false;
        }

    }



    // Menu Help

    // Help -> Help
    public void jMenuHelpHelp_actionPerformed(ActionEvent e)
    {
        String sTempResource = smail.gui.EditorFrame.class.getResource("help.html").toString();//.substring(5);
        Internet browser = new Internet(sTempResource);
        browser.show(sTempResource);
    }


    // Help -> About
    public void jMenuHelpAbout_actionPerformed(ActionEvent e)
    {
        final JFrame f = new AboutFrame();
    }




    /**Overridden so we can exit when window is closed*/
    protected void processWindowEvent(WindowEvent e)
    {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING)
        {
            jMenuMessageClose_actionPerformed(null);
        }
    }


    // Popup Menu Attachments
    void jListAttachments_mousePressed(MouseEvent e)
    {
      if ((e.getModifiers() != InputEvent.BUTTON1_MASK))
      {
        System.out.println("Nicht die linke maustaste");
        jPopupMenuAttachments.show(e.getComponent(), e.getX(), e.getY());
      }
      else
      {
        System.out.println("linke maustaste");
      }
    }



    // Popup Menu To Field
    void jListTo_mousePressed(MouseEvent e)
    {
      if ((e.getModifiers() != InputEvent.BUTTON1_MASK))
      {
        System.out.println("Nicht die linke maustaste");
        jPopupMenuTo.show(e.getComponent(), e.getX(), e.getY());
      }
      else
      {
        System.out.println("linke maustaste");
      }
    }

    // Popup To -> Remove
    void jPopupMenuToRemove_actionPerformed(ActionEvent e)
    {
      int iIndex = jListTo.getSelectedIndex();
      if(iIndex != -1)
      {
          vToAddresses.removeElementAt(iIndex);
          jListTo.setListData(vToAddresses);
      }
      jTextFieldTo.requestFocus();
    }

    // Popup To -> Remove All
    void jPopupMenuToRemoveAll_actionPerformed(ActionEvent e)
    {
      vToAddresses.removeAllElements();
      jListTo.setListData(vToAddresses);
      jTextFieldTo.requestFocus();
    }

    // Popup to -> Edit
    void jPopupMenuToEdit_actionPerformed(ActionEvent e)
    {
      int iIndex = jListTo.getSelectedIndex();

      if(iIndex != -1)
      {
          jTextFieldTo.setText((String)vToAddresses.elementAt(iIndex));
          vToAddresses.removeElementAt(jListTo.getSelectedIndex());
          jListTo.setListData(vToAddresses);
      }
      jTextFieldTo.requestFocus();
    }

    // Popup to -> Transfer to cc/bcc Field
    void jPopupMenuToTransfer_actionPerformed(ActionEvent e)
    {
      int iIndex = jListTo.getSelectedIndex();

      if(iIndex != -1)
      {
        if(bCcToggle == true)
        {
          vCcAddresses.addElement((String)vToAddresses.elementAt(iIndex));
          vToAddresses.removeElementAt(iIndex);
          jListTo.setListData(vToAddresses);
          jListCc.setListData(vCcAddresses);
          jTextFieldCc.requestFocus();
        }
        else
        {
          vBccAddresses.addElement((String)vToAddresses.elementAt(iIndex));
          vToAddresses.removeElementAt(iIndex);
          jListTo.setListData(vToAddresses);
          jListCc.setListData(vBccAddresses);
          jTextFieldCc.requestFocus();
        }
      }
    }

    // Popup Menu Cc Field
    void jListCc_mousePressed(MouseEvent e)
    {
      if ((e.getModifiers() != InputEvent.BUTTON1_MASK))
      {
        System.out.println("Nicht die linke maustaste");
        jPopupMenuCc.show(e.getComponent(), e.getX(), e.getY());
      }
      else
      {
        System.out.println("linke maustaste");
      }
    }

    // Popup Cc -> Remove
    void jPopupMenuCcRemove_actionPerformed(ActionEvent e)
    {
      int iIndex = jListCc.getSelectedIndex();

      if(iIndex != -1)
      {
        if(bCcToggle == true)
        {
          vCcAddresses.removeElementAt(iIndex);
          jListCc.setListData(vCcAddresses);
          jTextFieldCc.requestFocus();
        }
        else
        {
          vBccAddresses.removeElementAt(iIndex);
          jListCc.setListData(vBccAddresses);
          jTextFieldCc.requestFocus();
        }
      }
    }

    // Popup Cc -> Remove All
    void jPopupMenuCcRemoveAll_actionPerformed(ActionEvent e)
    {
      if(bCcToggle == true)
      {
        vCcAddresses.removeAllElements();
        jListCc.setListData(vCcAddresses);
        jTextFieldCc.requestFocus();
      }
      else
      {
        vBccAddresses.removeAllElements();
        jListCc.setListData(vBccAddresses);
        jTextFieldCc.requestFocus();
      }
    }

    // Popup Cc -> Edit
    void jPopupMenuCcEdit_actionPerformed(ActionEvent e)
    {
      int iIndex = jListCc.getSelectedIndex();

      if(iIndex != -1)
      {
        if(bCcToggle == true)
        {
          jTextFieldCc.setText((String)vCcAddresses.elementAt(iIndex));
          vCcAddresses.removeElementAt(iIndex);
          jListCc.setListData(vCcAddresses);
          jTextFieldCc.requestFocus();
        }
        else
        {
          jTextFieldCc.setText((String)vBccAddresses.elementAt(iIndex));
          vBccAddresses.removeElementAt(iIndex);
          jListCc.setListData(vBccAddresses);
          jTextFieldCc.requestFocus();
        }
      }
    }


    // Popup Cc -> Transfer
    void jPopupMenuCcTransfer_actionPerformed(ActionEvent e)
    {
      int iIndex = jListCc.getSelectedIndex();

      if(iIndex != -1)
      {
        if(bCcToggle == true)
        {
          vToAddresses.addElement((String)vCcAddresses.elementAt(iIndex));
          vCcAddresses.removeElementAt(iIndex);
          jListCc.setListData(vCcAddresses);
          jListTo.setListData(vToAddresses);
          jTextFieldTo.requestFocus();
        }
        else
        {
          vToAddresses.addElement((String)vBccAddresses.elementAt(iIndex));
          vBccAddresses.removeElementAt(iIndex);
          jListCc.setListData(vBccAddresses);
          jListTo.setListData(vToAddresses);
          jTextFieldTo.requestFocus();
        }
      }
    }


    // Popup Menu Text Field
    void jTextPane1_mousePressed(MouseEvent e)
    {
      if ((e.getModifiers() != InputEvent.BUTTON1_MASK))
      {
        System.out.println("Nicht die linke maustaste");
        jPopupMenuText.show(e.getComponent(), e.getX(), e.getY());
      }
      else
      {
        System.out.println("linke maustaste");
      }
    }

}
