// FindFrame.java

package smail.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

/** The FindFrame class displays a dialog window where the user can set a word or character, which he would like
 * to find in the textpane.	.
 * After he had cliqued on the OK-button the found string will be marked in the editor's textpane. If the user
 * repeats this action, the textfield of this window will contain the word, which have been looked for before.
 * If he doesn't change this entry, the next string (if it exists) will be find in the textpane.
 * <br><br>
 * Since it is a modal dialog, when the application calls its show method to display the dialog, it
 * blocks the rest of the application until the user has cliqued on a button or closed this window.
 */

public class FindFrame extends JDialog
{
  /** the information if the user has pressed the OK-button (true) or canceled (false)  */
  public boolean bFindStatus = false;

  /** the expression you want to find in the text */
  public String sFind = "";


  // property-File for chosing language (editor_lang_en for english, editor_lang_de for german)
  private ResourceBundle resource = ResourceBundle.getBundle("smail/gui/editor_lang");

  // getting the language specific textes for buttons, menus, etc from the property-file
  private String title = resource.getString("titleFind_key");

  private String sButtonOK = resource.getString("buttonOk_key");
  private String sButtonCancel = resource.getString("buttonCancel_key");
  private String sLabelFind = resource.getString("labelFind_key");


  JPanel contentPane;
  JLabel jLabelFind = new JLabel(sLabelFind);
  JTextField jTextFieldFind = new JTextField();
  JButton jButtonOk = new JButton(sButtonOK);
  JButton jButtonCancel = new JButton(sButtonCancel);


  /**Creates a dialog window for finding a certain expression in the editor's textpane
  	* @param sFind the expression you want to find in the text
    */
  public FindFrame(String sFind)
  {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
    jTextFieldFind.setText(sFind);
  }



  /**Component initialization*/
  private void jbInit() throws Exception
  {
    setModal(true);

    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(null);

    this.setTitle(title);
    this.setSize(new Dimension(300, 130));
    this.setResizable(false);


    //jLabelFind.setText("Find");
    jLabelFind.setBounds(new Rectangle(15, 18, 83, 22));

    jTextFieldFind.setBackground(Color.white);
    jTextFieldFind.setEditable(true);
    jTextFieldFind.setBounds(new Rectangle(58, 18, 224, 22));
    jTextFieldFind.addActionListener(new FindFrame_jTextFieldFind_actionAdapter(this));

    //jButtonOk.setText("OK");
    jButtonOk.setBounds(new Rectangle(15, 64, 115, 27));
    jButtonOk.addActionListener(new FindFrame_jButtonOk_actionAdapter(this));

    //jButtonCancel.setText("Cancel");
    jButtonCancel.setBounds(new Rectangle(164, 64, 115, 27));
    jButtonCancel.addActionListener(new FindFrame_jButtonCancel_actionAdapter(this));

    contentPane.add(jLabelFind, null);
    contentPane.add(jTextFieldFind, null);
    contentPane.add(jButtonOk, null);
    contentPane.add(jButtonCancel, null);

    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = getSize();
    if (frameSize.height > screenSize.height)
    {
      frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width)
    {
      frameSize.width = screenSize.width;
    }
    setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

  }


  /**Overridden so we can exit when window is closed*/
  protected void processWindowEvent(WindowEvent e)
  {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      // System.exit(0);
    }
  }


  // Button OK
  // gets the text to search from the textfield and sets it to the global variable sFind
  void jButtonOk_actionPerformed(ActionEvent e)
  {
    bFindStatus = true;
    sFind = jTextFieldFind.getText();
    dispose();
  }

  // Button Cancel
  // closes the window without doing anything
  void jButtonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  // by typing enter in the Textfiel find go to the OK Button
  void jTextFieldFind_actionPerformed(ActionEvent e)
  {
    jButtonOk.requestFocus();
  }
}


class FindFrame_jButtonOk_actionAdapter implements java.awt.event.ActionListener
{
  FindFrame adaptee;

  FindFrame_jButtonOk_actionAdapter(FindFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonOk_actionPerformed(e);
  }
}

class FindFrame_jButtonCancel_actionAdapter implements java.awt.event.ActionListener
{
  FindFrame adaptee;

  FindFrame_jButtonCancel_actionAdapter(FindFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonCancel_actionPerformed(e);
  }
}

class FindFrame_jTextFieldFind_actionAdapter implements java.awt.event.ActionListener {
  FindFrame adaptee;

  FindFrame_jTextFieldFind_actionAdapter(FindFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jTextFieldFind_actionPerformed(e);
  }
}
