// GoToLineFrame.java

package smail.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;


/** The GoToLineFrame class displays a dialog window where the user can set a line number.
 * After he had cliqued on the OK-button the cursor of the editor will go to the chosen line.
 * If there is no sence in the input of the user, the cursor will be set at the first line of the textpane.<br><br>
 * Since it is a modal dialog, when the application calls its show method to display the dialog, it
 * blocks the rest of the application until the user has cliqued on a button or closed this window.
 */

public class GoToLineFrame extends JDialog
{
  /** the information if the user has pressed the OK-button (true) or canceled (false)  */
  public boolean bGoToLineStatus = false;
  /** the line to which you want to go */
  public int iLine = 0;


  // property-File for chosing language (editor_lang_en for english, editor_lang_de for german)
  private ResourceBundle resource = ResourceBundle.getBundle("smail/gui/editor_lang");

  // getting the language specific textes for buttons, menus, etc from the property-file
  private String title = resource.getString("titleGoToLine_key");

  private String sButtonOK = resource.getString("buttonOk_key");
  private String sButtonCancel = resource.getString("buttonCancel_key");
  private String sLabelGoToLine = resource.getString("labelGoToLine_key");



  JPanel contentPane;
  JLabel jLabelGoToLine = new JLabel(sLabelGoToLine);
  JTextField jTextFieldGoToLine = new JTextField();
  JButton jButtonOk = new JButton(sButtonOK);
  JButton jButtonCancel = new JButton(sButtonCancel);


  /**Creates a dialog window to let the cursor go to a special line of the editor's textpane*/
  public GoToLineFrame()
  {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }


  /**Component initialization*/
  private void jbInit() throws Exception
  {
    setModal(true);

    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(null);

    this.setTitle(title);
    this.setSize(new Dimension(300, 130));
    this.setResizable(false);

    jTextFieldGoToLine.setBackground(Color.white);
    jTextFieldGoToLine.setEditable(true);
    jTextFieldGoToLine.setBounds(new Rectangle(110, 15, 170, 22));
    jTextFieldGoToLine.addActionListener(new GoToLineFrame_jTextFieldGoToLine_actionAdapter(this));

    //jLabelGoToLine.setText("Go to line:");
    jLabelGoToLine.setBounds(new Rectangle(15, 15, 100, 22));

    //jButtonOk.setText("OK");
    jButtonOk.setBounds(new Rectangle(15, 60, 115, 27));
    jButtonOk.addActionListener(new GoToLineFrame_jButtonOk_actionAdapter(this));

    //jButtonCancel.setText("Cancel");
    jButtonCancel.setBounds(new Rectangle(165, 60, 115, 27));
    jButtonCancel.addActionListener(new GoToLineFrame_jButtonCancel_actionAdapter(this));

    contentPane.add(jLabelGoToLine, null);
    contentPane.add(jTextFieldGoToLine, null);
    contentPane.add(jButtonOk, null);
    contentPane.add(jButtonCancel, null);


    // Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = getSize();
    if (frameSize.height > screenSize.height)
    {
      frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width)
    {
      frameSize.width = screenSize.width;
    }
    setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

}



  //Overridden so we can exit when window is closed
  protected void processWindowEvent(WindowEvent e)
  {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      //System.exit(0);
    }
  }



  // Button OK
  // gets the number and sets it to the global variable iLine
  void jButtonOk_actionPerformed(ActionEvent e)
  {
    bGoToLineStatus = true;
    String sLine = jTextFieldGoToLine.getText();

    try
    {
      iLine = Integer.parseInt(sLine);
    }
    catch(Exception ex)
    {
      iLine = 0;
    }

    dispose();
  }


  // Button Cancel
  // closes the window without doing anything
  void jButtonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  // by typing enter in the Textfield go to the OK Button
  void jTextFieldGoToLine_actionPerformed(ActionEvent e)
  {
    jButtonOk.requestFocus();
  }
}


class GoToLineFrame_jButtonOk_actionAdapter implements java.awt.event.ActionListener
{
  GoToLineFrame adaptee;

  GoToLineFrame_jButtonOk_actionAdapter(GoToLineFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonOk_actionPerformed(e);
  }
}

class GoToLineFrame_jButtonCancel_actionAdapter implements java.awt.event.ActionListener
{
  GoToLineFrame adaptee;

  GoToLineFrame_jButtonCancel_actionAdapter(GoToLineFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonCancel_actionPerformed(e);
  }
}

class GoToLineFrame_jTextFieldGoToLine_actionAdapter implements java.awt.event.ActionListener {
  GoToLineFrame adaptee;

  GoToLineFrame_jTextFieldGoToLine_actionAdapter(GoToLineFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jTextFieldGoToLine_actionPerformed(e);
  }
}
