// ReplaceFrame.java

package smail.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

/** The ReplaceFrame class displays a dialog window where the user can set a word or character, which he would like
 * to find in the textpane and another word or character which should replace the first expression.	.
 * After he had cliqued on the OK-button the found string will be replaced in the editor's textpane. If the user
 * repeats this action, the textfield of this window will contain the words, which have been used before.
 * If he doesn't change this entry, the next string (if it exists) will be replaced in the textpane.
 * <br><br>
 * Since it is a modal dialog, when the application calls its show method to display the dialog, it
 * blocks the rest of the application until the user has cliqued on a button or closed this window.
 */


public class ReplaceFrame extends JDialog
{
  /** the information if the user has pressed the OK-button (true) or canceled (false)  */
  boolean bReplaceStatus = false;
  /** the expression you want to find and replace in the text (the old expression)*/
  public String sFind = "";
  /** the expression you want to set in for the found expression (the new expression)*/
  public String sReplace = "";

  // property-File for chosing language (editor_lang_en for english, editor_lang_de for german)
  private ResourceBundle resource = ResourceBundle.getBundle("smail/gui/editor_lang");

  // getting the language specific textes for buttons, menus, etc from the property-file
  private String title = resource.getString("titleReplace_key");

  private String sButtonOK = resource.getString("buttonOk_key");
  private String sButtonCancel = resource.getString("buttonCancel_key");
  private String sLabelFind = resource.getString("labelReplaceFind_key");
  private String sLabelReplace = resource.getString("labelReplace_key");


  JPanel contentPane;
  JLabel jLabelFind = new JLabel(sLabelFind);
  JLabel jLabelReplace = new JLabel(sLabelReplace);
  JTextField jTextFieldFind = new JTextField();
  JTextField jTextFieldReplace = new JTextField();
  JButton jButtonOk = new JButton(sButtonOK);
  JButton jButtonCancel = new JButton(sButtonCancel);


  /**Creates a dialog window for finding and replacing a certain expression in the editor's textpane
  	* @param sFind the expression you want to find and replace in the text (the old expression)
  	* @param sReplace the expression you want to set in for the found expression (the new expression)
    */

  public ReplaceFrame(String sFind, String sReplace)
  {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
    jTextFieldFind.setText(sFind);
    jTextFieldReplace.setText(sReplace);
  }


  /**Component initialization*/
  private void jbInit() throws Exception
  {
    setModal(true);

    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(null);

    this.setSize(new Dimension(300, 130));
    this.setTitle(title);
    this.setResizable(false);

//    jLabelFind.setText("Text to find:");
    jLabelFind.setBounds(new Rectangle(15, 15, 89, 22));

//    jLabelReplace.setText("Replace with:");
    jLabelReplace.setBounds(new Rectangle(15, 41, 85, 22));

    jTextFieldFind.setBackground(Color.white);
    jTextFieldFind.setEditable(true);
    jTextFieldFind.setBounds(new Rectangle(100, 15, 185, 22));
    jTextFieldFind.addActionListener(new ReplaceFrame_jTextFieldFind_actionAdapter(this));

    jTextFieldReplace.setBackground(Color.white);
    jTextFieldReplace.setEditable(true);
    jTextFieldReplace.setBounds(new Rectangle(100, 41, 185, 22));
    jTextFieldReplace.addActionListener(new ReplaceFrame_jTextFieldReplace_actionAdapter(this));

    //jButtonOk.setText("OK");
    jButtonOk.setBounds(new Rectangle(15, 75, 115, 27));
    jButtonOk.addActionListener(new ReplaceFrame_jButtonOk_actionAdapter(this));

    //jButtonCancel.setText("Cancel");
    jButtonCancel.setBounds(new Rectangle(170, 75, 115, 27));
    jButtonCancel.addActionListener(new ReplaceFrame_jButtonCancel_actionAdapter(this));

    contentPane.add(jLabelFind, null);
    contentPane.add(jLabelReplace, null);
    contentPane.add(jTextFieldFind, null);
    contentPane.add(jTextFieldReplace, null);
    contentPane.add(jButtonCancel, null);
    contentPane.add(jButtonOk, null);


    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = getSize();
    if (frameSize.height > screenSize.height) {
      frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width) {
      frameSize.width = screenSize.width;
    }
    setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
  }


  /**Overridden so we can exit when window is closed*/
  protected void processWindowEvent(WindowEvent e)
  {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      //System.exit(0);
    }
  }


  // Button OK
  // gets the text to search and replace from the textfields and sets them to the global variables sFind and sReplace
  void jButtonOk_actionPerformed(ActionEvent e)
  {
    bReplaceStatus = true;
    sFind = jTextFieldFind.getText();
    sReplace = jTextFieldReplace.getText();
    dispose();
  }


  // Button Cancel
  // closes the window without doing anything
  void jButtonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  // by typing enter in the Textfield find go to Textfield replace
  void jTextFieldFind_actionPerformed(ActionEvent e)
  {
    jTextFieldReplace.requestFocus();
  }

  // by typing enter in the Textfield replace go to the OK Button
  void jTextFieldReplace_actionPerformed(ActionEvent e)
  {
    jButtonOk.requestFocus();
  }
}


class ReplaceFrame_jButtonOk_actionAdapter implements java.awt.event.ActionListener
{
  ReplaceFrame adaptee;

  ReplaceFrame_jButtonOk_actionAdapter(ReplaceFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonOk_actionPerformed(e);
  }
}

class ReplaceFrame_jButtonCancel_actionAdapter implements java.awt.event.ActionListener
{
  ReplaceFrame adaptee;

  ReplaceFrame_jButtonCancel_actionAdapter(ReplaceFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonCancel_actionPerformed(e);
  }
}

class ReplaceFrame_jTextFieldFind_actionAdapter implements java.awt.event.ActionListener {
  ReplaceFrame adaptee;

  ReplaceFrame_jTextFieldFind_actionAdapter(ReplaceFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jTextFieldFind_actionPerformed(e);
  }
}

class ReplaceFrame_jTextFieldReplace_actionAdapter implements java.awt.event.ActionListener
{
  ReplaceFrame adaptee;

  ReplaceFrame_jTextFieldReplace_actionAdapter(ReplaceFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jTextFieldReplace_actionPerformed(e);
  }
}
