// SignatureFrame.java

package smail.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

/** The SignatureFrame class displays a dialog window where the user can create a new signature.
 *  Every signature has to get an description. By this description the user will find his signature
 * as menuitem of the menu <i>signature </i> of the editor.
 * After he had cliqued on the OK-button the created signature will be apended to the editor's menu.
 * <br><br>
 * Since it is a modal dialog, when the application calls its show method to display the dialog, it
 * blocks the rest of the application until the user has cliqued on a button or closed this window.
 */

public class SignatureFrame extends JDialog
{
  smail.utils.PropertiesManager propMan = smail.utils.PropertiesManager.getInstance();

  /** the information if the user has pressed the OK-button (true) or canceled (false)  */
  public boolean bSignatureStatus = false;
  /** the name you want to give your signature, by this name a menuitem will appear in the menu signature of the editor */
  public String sDescription = "";
  /** your signature*/
  public String sText = "";

  // property-File for chosing language (editor_lang_en for english, editor_lang_de for german)
  private ResourceBundle resource = ResourceBundle.getBundle("smail/gui/editor_lang");

  // getting the language specific textes for buttons, menus, etc from the property-file
  private String title = resource.getString("titleSignature_key");

  private String sButtonOK = resource.getString("buttonOk_key");
  private String sButtonCancel = resource.getString("buttonCancel_key");
  private String sLabelDescription = resource.getString("labelSignatureDescription_key");
  private String sTextDescription = resource.getString("textSignatureDescription_key");

  private String sMenuEditCut = resource.getString("menuEditCut_key");
  private String sMenuEditCopy = resource.getString("menuEditCopy_key");
  private String sMenuEditPaste = resource.getString("menuEditPaste_key");
  private String sMenuEditSelectAll = resource.getString("menuEditSelectAll_key");


  JPanel contentPane;
  JLabel jLabelDescription = new JLabel(sLabelDescription);
  JTextField jTextFieldDescription = new JTextField(sTextDescription);
  JScrollPane jScrollPane = new JScrollPane();
  JTextArea jTextArea1 = new JTextArea();
  JButton jButtonOk = new JButton(sButtonOK);
  JButton jButtonCancel = new JButton(sButtonCancel);

  JPopupMenu jPopupMenuText = new JPopupMenu();
  JMenuItem jPopupMenuTextCut = new JMenuItem(sMenuEditCut);
  JMenuItem jPopupMenuTextCopy = new JMenuItem(sMenuEditCopy);
  JMenuItem jPopupMenuTextPaste = new JMenuItem(sMenuEditPaste);
  JMenuItem jPopupMenuTextSelectAll = new JMenuItem(sMenuEditSelectAll);

  /**Creates a dialog window to create a signature*/
  public SignatureFrame()
  {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }
  /**Component initialization*/
  private void jbInit() throws Exception
  {
    setModal(true);

    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(null);

    this.setSize(new Dimension(600, 350));
    this.setTitle("Signature");
    this.setResizable(false);

    jLabelDescription.setBounds(new Rectangle(15, 15, 78, 18));

    jTextFieldDescription.setBounds(new Rectangle(100, 15, 200, 25));
    jTextFieldDescription.requestFocus();
    jTextFieldDescription.selectAll();
    jTextFieldDescription.addActionListener(new SignatureFrame_jTextFieldDescription_actionAdapter(this));

    jTextArea1.setBorder(BorderFactory.createEtchedBorder());
    jTextArea1.setFont(new java.awt.Font("SansSerif", 0, 13));
    jScrollPane.setBounds(new Rectangle(12, 66, 575, 176));
    jScrollPane.getViewport().add(jTextArea1, null);

    // Ok
    jButtonOk.setBounds(new Rectangle(162, 261, 112, 25));
    jButtonOk.addActionListener(new SignatureFrame_jButtonOk_actionAdapter(this));

    // Cancel
    jButtonCancel.setBounds(new Rectangle(328, 260, 112, 25));
    jButtonCancel.addActionListener(new SignatureFrame_jButtonCancel_actionAdapter(this));


    // Popup
    jTextArea1.addMouseListener(new java.awt.event.MouseAdapter(){public void mousePressed(MouseEvent e){jTextArea1_mousePressed(e);}});

    jPopupMenuTextCut.setAccelerator(javax.swing.KeyStroke.getKeyStroke(88, java.awt.event.KeyEvent.CTRL_MASK, false));
    jPopupMenuTextCut.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuEditCut_actionPerformed(e);}});

    jPopupMenuTextCopy.setAccelerator(javax.swing.KeyStroke.getKeyStroke(67, java.awt.event.KeyEvent.CTRL_MASK, false));
    jPopupMenuTextCopy.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuEditCopy_actionPerformed(e);}});

    jPopupMenuTextPaste.setAccelerator(javax.swing.KeyStroke.getKeyStroke(86, java.awt.event.KeyEvent.CTRL_MASK, false));
    jPopupMenuTextPaste.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuEditPaste_actionPerformed(e);}});

    jPopupMenuTextSelectAll.setAccelerator(javax.swing.KeyStroke.getKeyStroke(65, java.awt.event.KeyEvent.CTRL_MASK, false));
    jPopupMenuTextSelectAll.addActionListener(new java.awt.event.ActionListener(){public void actionPerformed(ActionEvent e){jPopupMenuEditSelectAll_actionPerformed(e);}});

    jPopupMenuText.add(jPopupMenuTextCut);
    jPopupMenuText.add(jPopupMenuTextCopy);
    jPopupMenuText.add(jPopupMenuTextPaste);
    jPopupMenuText.add(jPopupMenuTextSelectAll);


    contentPane.add(jLabelDescription, null);
    contentPane.add(jTextFieldDescription, null);
    contentPane.add(jScrollPane, null);
    contentPane.add(jButtonCancel, null);
    contentPane.add(jButtonOk, null);


    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = getSize();
    if (frameSize.height > screenSize.height)
    {
      frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width)
    {
      frameSize.width = screenSize.width;
    }
    setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
  }

  /**Overridden so we can exit when window is closed*/
  protected void processWindowEvent(WindowEvent e)
  {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      dispose();
    }
  }

  // Button OK
  void jButtonOk_actionPerformed(ActionEvent e)
  {
    int iCounter = 0;

    bSignatureStatus = true;
    sDescription = jTextFieldDescription.getText();
    sText = jTextArea1.getText();

    // counter holen, bzw erzeugen
    if(propMan.get("smail.gui.signature.counter") != null)
    {
        try
        {
            iCounter = Integer.parseInt(propMan.get("smail.gui.signature.counter"));
        }
        catch(Exception ex)
        {
            System.out.println("smail.gui.SignatureFrame - no counter in Property");
        }
    }

    // testen ob Description schon vorhanden
    boolean bDescriptionExists = false;
    for(int iCounter2 =1; iCounter2 <= iCounter; iCounter2++)
    {
        String sDescriptionOld = propMan.get("smail.gui.signature.description"+iCounter2);
        if(sDescription.equals(sDescriptionOld)== true)
        {
            bDescriptionExists = true;
            ErrorFrame error = new ErrorFrame();
            error.jLabelInfo.setText("Please chose another Description");
            error.setTitle("Description exits already");
            error.jButtonOk.requestFocus();
            error.show();

            jTextFieldDescription.requestFocus();
            jTextFieldDescription.selectAll();
            return;
        }
    }



    System.out.println(propMan.get("smail.gui.signature.counter"));
    iCounter = iCounter +1;
    propMan.set("smail.gui.signature.counter",""+iCounter);

    //key zusammenbauen description mit count basteln
    String sDescriptionKey = "smail.gui.signature.description"+iCounter;

    // description setzen
    propMan.set(sDescriptionKey,sDescription);
    // key fuer text basteln
    String sTextKey = "smail.gui.signature."+sDescription;
    // text setzen
    propMan.set(sTextKey,sText);
    propMan.exiting();
    dispose();

  }

  // Button Cancel
  void jButtonCancel_actionPerformed(ActionEvent e)
  {
    dispose();
  }

  void jTextFieldDescription_actionPerformed(ActionEvent e)
  {
      jTextArea1.requestFocus();
  }


    // Popup Menu TextArea
    void jTextArea1_mousePressed(MouseEvent e)
    {
      if ((e.getModifiers() != InputEvent.BUTTON1_MASK))
      {
        System.out.println("Nicht die linke maustaste");
        jPopupMenuText.show(e.getComponent(), e.getX(), e.getY());
      }
      else
      {
        System.out.println("linke maustaste");
      }
    }

   // Popup Menu -> Cut
    void jPopupMenuEditCut_actionPerformed(ActionEvent e)
    {
        jTextArea1.cut();
        jTextArea1.requestFocus();
    }

   // Popup Menu -> Copy
    void jPopupMenuEditCopy_actionPerformed(ActionEvent e)
    {
        jTextArea1.copy();
        jTextArea1.requestFocus();
    }
   // Popup Menu -> Paste
    void jPopupMenuEditPaste_actionPerformed(ActionEvent e)
    {
        jTextArea1.paste();
        jTextArea1.requestFocus();
    }


   // Popup Menu -> Select All
    void jPopupMenuEditSelectAll_actionPerformed(ActionEvent e)
    {
        jTextArea1.selectAll();
        jTextArea1.requestFocus();
    }

}
class SignatureFrame_jButtonOk_actionAdapter implements java.awt.event.ActionListener
{
  SignatureFrame adaptee;

  SignatureFrame_jButtonOk_actionAdapter(SignatureFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonOk_actionPerformed(e);
  }
}

class SignatureFrame_jButtonCancel_actionAdapter implements java.awt.event.ActionListener
{
  SignatureFrame adaptee;

  SignatureFrame_jButtonCancel_actionAdapter(SignatureFrame adaptee)
  {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e)
  {
    adaptee.jButtonCancel_actionPerformed(e);
  }
}

class SignatureFrame_jTextFieldDescription_actionAdapter implements java.awt.event.ActionListener {
  SignatureFrame adaptee;

  SignatureFrame_jTextFieldDescription_actionAdapter(SignatureFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jTextFieldDescription_actionPerformed(e);
  }
}
