package smail.gui.filterGui ;

import javax.swing.* ;
import javax.swing.table.* ;
import java.awt.event.* ;
import java.awt.Dimension ;
import java.awt.GridBagConstraints ;
import java.awt.Insets ;
import smail.filter.* ;
import java.util.Vector ; 
/**
 *This class represents the table model for the filter overview
 *in the beginning of the FilterGuy Dialog. It made of two columns :<br>
 *1. row shows if this filter is activated <br>
 *2. row shows the user defined name of a filter rule
 */

class FilterTableModel extends AbstractTableModel /* implements java.awt.event.ActionListener */
{
	/**
	 *this follows an example of the java-tutorial
	 *@see java.swing.table.AbstractTableModel
	 */
	protected String[] colNames = new String[] 
			{	"active"  , "Filter type", "Filter name"  } ;
			
	/**
	 *this follows an example of the java-tutorial
	 *@see java.swing.table.AbstractTableModel
	 */
	protected Class[] colClass = new Class[]
			{	Boolean.class , String.class , String.class	} ;
	
	private FilterManager fm ;


	/* this method is used for debugging the process of making a new filter */
	private void printFilter(Filter filter)
	{
		System.out.println( "beginn adding this filter :" ) ;
		System.out.print( "		FilterType : ") ;
		switch (filter.getType())
		{
			case smail.filter.Filter.REC_MAN:
				System.out.println( "REC_MAN"+"("+filter.getType()+")" ) ;
				break;
			case smail.filter.Filter.MANUALLY :
				System.out.println( "MANUALLY"+"("+filter.getType()+")" ) ;
				break;
			case smail.filter.Filter.PRE :
				System.out.println( "PRE"+"("+filter.getType()+")" ) ;
				break;
			case smail.filter.Filter.POST :
				System.out.println( "POST"+"("+filter.getType()+")" ) ;
				break;
			default:
				System.out.println( "UNKNOWN"+"("+filter.getType()+")" ) ;
				break;
		}	
		System.out.print( "		ActionType :" ) ;
		switch (filter.getAction().getType())
		{
			case smail.filter.Action.DELETE :
				System.out.println( "DELETE"+"("+filter.getAction().getType()+")" ) ;
				break;
			case smail.filter.Action.MOVE :
				System.out.println( "MOVE"+"("+filter.getAction().getType()+")" ) ;
				break;
			default:
				System.out.println( "UNKNOWN"+"("+filter.getAction().getType()+")" ) ;
				break;
		}
		if (filter.getAction().getType() == smail.filter.Action.MOVE)
		{
			System.out.print( filter.getAction().getDestination() ) ;
		}
		Condition[] cond = filter.getCondition() ;
		for (int i = 0 ; cond != null && i<cond.length ; i++ )
		{
			System.out.println( "		filter condition (not array index):"+(i+1) ) ;
			System.out.println( "		type : "+cond[i].getConditionType() ) ;
		}
	}
	
	void addFilterToModel(Filter newFilter, boolean overwrite)
	{
		try
		{
			if ( !overwrite ) {
				fm.addFilter(newFilter) ;
			} else {
				try {
					Filter delFilter = fm.getFilter( newFilter.getName() );
					fm.removeFilter( delFilter.getFilterId() );
				} catch (NoSuchNameException exc) {
					//				
				}
				fm.addFilter(newFilter);
			}
			
		}
		catch (NameAlreadyExistException exc)
		{
			System.out.println( "FilterTableModel.addFilterToModel() :: Name already exists !" ) ;
			smail.Popup p = new smail.Popup("Sorry , there is a Filter with the same name.",true) ;
		}
		fireTableDataChanged() ;
	}
	
	void removeFilterFromModel(int index)
	{

		fm.removeFilter(index) ;
		fireTableDataChanged() ;
		
	}
	
	Filter getFilter(int index) throws WrongFilterIDException 
	{
		return fm.getFilter(index) ;
	}
	
	void toggleActivity(int index)
	{
		fm.toggleActivity(index) ;
		fireTableDataChanged() ;
	}
	
	FilterTableModel(smail.filter.FilterManager fm)
	{
		super() ;
		this.fm = fm ;
	}
	
	public boolean isCellEditable(int row,int column)
	{
		if(column==0)
		{
			return true;
		}	
		return false;	
	}
	/**
	 *@see javax.swing.table.AbstractTableModel
	 */
	public int getColumnCount() 
	{
		return 3 ;
	}
	
	/**
	 *@see javax.swing.table.AbstractTableModel
	 */
	public int getRowCount()
	{
		synchronized(fm)
		{
				return fm.getAllFilter().length ;
		}
	}
	
	/**
	 *@see javax.swing.table.AbstractTableModel
	 */
	public String getColumnName(int column)
	{
		return colNames[column] ;
	}
	
	/**
	 *@see javax.swing.table.AbstractTableModel
	 */
	public Class getColumnClass(int column)
	{
		return colClass[column] ;
	}



	public void setValueAt(Object value, int row, int column)/* ADJUST THIS TO THE RIGHT FILTER */
	{
		switch (column)
		{
			case 0 :
				try 
				{
					fm.getFilter(row).setActivity(((Boolean)value).booleanValue());
				}
				catch(smail.filter.WrongFilterIDException wfide)
				{
					wfide.printStackTrace();	
				}
				
				break;
			default :break;
		}
		fireTableDataChanged();
	}
	
	/**
	 *@see javax.swing.table.AbstractTableModel
	 */
	public Object getValueAt(int row, int column)/* ADJUST THIS TO THE RIGHT FILTER */
	{
		try
		{
			switch (column)
			{
				case 0 :
					return new Boolean( fm.getFilter(row).getActivity() ) ;
				case 1 :
					{
						switch ( fm.getFilter(row).getType() )
						{
							case Filter.MANUALLY:
								return "MANUALLY" ;
							case Filter.POST:
								return "POST" ;
							case Filter.PRE:
								return "PRE" ;
							default:
								return "this type is unknown :"+fm.getFilter(row).getType() ;
						}
					}
				case 2 :
					{
						return ( fm.getFilter(row).getName() ) ;
					}
				default:
					System.out.println( "wrong column called in filterTableModel" ) ;
					return null ;		
			}
		}
		catch (WrongFilterIDException e)
		{
			System.out.println( "FilterTableModel.getValueAt("+row+"/"+column+") :: wrong filterID caught !" ) ;
			return null ;
		}
	}// END getValue			
}
