/*
 * localStore.java
 *
 * Created on 25. Mai 2001, 17:23
 */

package smail.localStore;

public class LocalStore extends javax.mail.Store
{
    /** names of the standard folders, that should be created if
     *  not found at startup */
    private static final String[] STANDARD_FOLDERNAMES =
    { "/", "/INBOX", "/OUTBOX", "/DRAFTS", "/TRASH", "/SENT"
    // TODO: remove this test stuff
      , "/INBOX/Uni", "/INBOX/Uni/SWP", "/INBOX/Uni/SWP/gui",
      "/INBOX/Uni/SWP/Fassade", "/INBOX/Uni/Zedat",
      "/INBOX/Mailinglists",
      "/INBOX/Mailinglists/Anti M$", "/INBOX/Mailinglists/Gimp User Group"
    };

    /** the path to the mbox files */
    private java.io.File storePath;
    
    private java.util.Hashtable folders;
    
    protected javax.mail.Session session;
   
    /** Creates a new localStore that meens creating the <CODE>STANDARD_FOLDERNAMES</CODE> e.g. /, INBOX, OUTBOX, DRAFTS,TRASH, SENT
     * @param s The session this <CODE>Localstore</CODE> belongs to.
     * @param un The path to write on disk.
     */
    public LocalStore(javax.mail.Session s, javax.mail.URLName un)
    {
        super(s, un);
        
        session = s;
        storePath = new java.io.File(un.getFile()).getAbsoluteFile();
//        System.out.println("Path to mail folders: " + storePath.getAbsolutePath()); // debug
        
        folders = new java.util.Hashtable();
        folders = readMBoxes(storePath, s);
        
        /* //debug
        java.util.Iterator it = folders.keySet().iterator();
        while (it.hasNext()) {
          String str = (String)it.next();
          MBox mbox = (MBox)folders.get(str);
          System.out.println(str + " -> " + mbox.mboxFilename.getAbsolutePath());
        }
        System.exit(0); */
    }
    
    /** Returns the default <CODE>Folder</CODE> here INBOX
     * @throws MessagingException If INBOX was not found.
     * @return returns the <CODE>Localfolder</CODE> INBOX
     */    
    public javax.mail.Folder getDefaultFolder() throws javax.mail.MessagingException
    {
        return (LocalFolder)getFolder("/INBOX");
    }

    /**
     * Returns the folder indicated by a string. The foldername has to be
     * absolute. (You may omit the leading '/') <P>
     * @return javax.mail.Folder, if folder can by found, else <CODE>null</CODE>
     * @param str The string to indicate the <CODE>Folder</CODE>.
     * @throws MessagingException 
     */
    public javax.mail.Folder getFolder(String str) throws javax.mail.MessagingException
    {
        // if name isn't absolute prepend a '/'
        if (!(str.charAt(0)=='/')) str = "/" + str;
        // if (str.endsWith("/") && str.length() != 1) str = str.substring(0, str.length()-1);
        
          
        // return folder if it exists or create a new one if not.
        
        // LocalFolder tmp = (LocalFolder)folders.get(str);
        MBox mbox = null;
        java.util.Iterator it = this.getFolderIterator();
        while (it.hasNext()) {
          String name = (String)it.next();
          if (name.equals(str)) {
//            System.out.println("LocalStore.getFolder(String): " + str); // debug
            return new LocalFolder(this, str, (MBox)folders.get(name));
          }
        }
//        System.out.println("LocalStore.getFolder(String): " + str + " (non existant!)");
        throw new javax.mail.MessagingException();
        //return new LocalFolder(this, str, null);
    }
    
    /** Returns a <CODE>Folder</CODE> indicated by a <CODE>javax.mail.URLName</CODE>.
     * @param un The <CODE>java.mail.URLName</CODE> to indecate the <CODE>Folder</CODE>.
     * @throws MessagingException
     * @return Returns a <CODE>javax.mail.Folder</CODE>.
     */    
    public javax.mail.Folder getFolder(javax.mail.URLName un) throws javax.mail.MessagingException
    {
        return getFolder(un.getFile());
    }
    
    /** Returns the Folder interator to walk thew Messages.
     * @return returns a <CODE>java.util.Iterator()</CODE>.
     */    
    protected java.util.Iterator getFolderIterator() {
      return folders.keySet().iterator();
    }
    
    private static java.util.Hashtable readMBoxes(java.io.File path, javax.mail.Session s) {
    // protected LocalStore(java.io.File path) {
      
      java.util.Hashtable folders = new java.util.Hashtable();
      
      // check if LocalStore was created with a filename instead of a pathname that
      // leads to the mbox files and a simple attempt to correct this.
      if (!path.isDirectory()) {
        System.err.println("FIXME: LocalStore: filename instead of pathname to mbox files");
        path = new java.io.File(path.getAbsolutePath());
      }
      
      // check if the path to the mailboxes already exists and create it if not
      if (!path.exists()) {
//        System.out.println("Path to mailboxes doesn't exist. Creating it now.");
        if (!path.mkdirs()) {
          // TODO: throwing an exception here?
          System.err.println("Warning: Can't create path! Continue at own risk!");
        }
      }
      
      // TODO: change into just checking if the folders exist and creating an empty one if not
//      System.out.println("Reading/creating standard folders..."); // debug
      for (int i=0; i < STANDARD_FOLDERNAMES.length; i++) {
//        System.out.println("Folder: " + STANDARD_FOLDERNAMES[i]); // debug
        try {
          folders.put(STANDARD_FOLDERNAMES[i],
            new MBox(new java.io.File(path.getAbsolutePath() + STANDARD_FOLDERNAMES[i] + ".mbox"), s, null));
        } catch (java.io.IOException ioe) {
          System.err.println("Error while reading/creating " + STANDARD_FOLDERNAMES[i]);
          ioe.printStackTrace();
        }
      }
      
      // now read all other mboxes and subfolders recursivly
      return readMBoxes(path, path.getAbsolutePath().length(), folders, s);
    }
    
    private static java.util.Hashtable readMBoxes(java.io.File path,
                                                  int rootPathLength,
                                                  java.util.Hashtable folders,
                                                  javax.mail.Session s) {

      // get all files in the current directory with the ending ".mbox"
      java.io.File[] mboxFiles = path.listFiles(new java.io.FileFilter() {
        public boolean accept(java.io.File name) {
          return (name.isFile() && name.getName().endsWith(".mbox"));
        }
      });
      
      // create a MBox for every file and put a (foldername, mbox) pair into the
      // folders table.
      for (int i=0; i<mboxFiles.length; i++) {
        // cut the rootPath from the pathname
        String name = mboxFiles[i].getAbsolutePath();
        name = name.substring(rootPathLength, name.length() - ".mbox".length());
        if (!folders.containsKey(name)) {
          try {
             folders.put(name, new MBox(mboxFiles[i], s, null));
          } catch (java.io.IOException ioe) {
            System.err.println("Error: can't read " + mboxFiles[i].getAbsolutePath());
            //ioe.printStackTrace();
          }
        }
      } 
      
      // get all subdirectories of the current directory
      java.io.File[] subFolders = path.listFiles(new java.io.FileFilter() {
        public boolean accept(java.io.File name) {
          return (name.isDirectory());
        }
      });
      
      // read all mboxes in the subdirectories recursivly
      for (int i=0; i<subFolders.length; i++) {
        folders = readMBoxes(subFolders[i], rootPathLength, folders, s);
      }
      
      return folders;
    }
    
    protected void deleteFolder(LocalFolder f) {
      String name = f.getFullName();
      java.util.Iterator iterator = folders.keySet().iterator();
      while (iterator.hasNext()) {
        String s = (String)iterator.next();
        if (s.equals(name)) {
//          System.out.println("LocalStore: Deleting folder: " + name);
          ((MBox)folders.get(s)).deleteMBox();
          iterator.remove();
          break;
        }
      }
    }
}
