/*
 * MBox.java
 *
 * Created on 9. Juni 2001, 22:32
 */

package smail.localStore;

/** This is an implementation to write the folders to the standardized mbox format.
 *
 * @author Andreas Rebenstorf {@link mailto:reb@rrgbr.de}
 * @version 0.1
 */
public class MBox
{
    java.io.File mboxFile;
    /** <CODE>index</CODE> is needed to access the mbox-file and to show mails in the gui.
     */    
    //public java.util.Vector index;
    public Index index;
    
    private javax.mail.Session session;
    private LocalFolder folder;
    
    private String indexFilename;// = "index";
    
    public void setFolder(LocalFolder f)
    {
    	folder = f;
    }
    
    
    /** Here it will be checked that the mbox-file exists, than it will be indexed.
     * Else it will be created a new mbox-file.
     * @param mboxFilename the mbox-filename on harddisk
     * @param s the session all messages belongs to.
     * @throws IOException is thown if something is wrong with accessing the harddisk-files.
     */
    public MBox(java.io.File mboxFile, javax.mail.Session s, LocalFolder f) throws java.io.IOException
    {
        
        this.mboxFile = mboxFile;
        this.folder = f;
        indexFilename=mboxFile.getAbsoluteFile()+".index";
        //System.out.println("---------------------mbox"+indexFilename);
        //index = new java.util.Vector();

        session = s;
        if (!mboxFile.exists())
        {
            java.io.File dir = mboxFile.getParentFile();
//            System.out.println(dir.getName()); // debug
            if (!dir.exists()) {
//              System.out.println("DEBUG smail.localStore.mbox: Creating dir " + dir); // debug
              dir.mkdirs();
            }
            mboxFile.createNewFile();
        }
        try
        {
            java.io.File indexFile = new java.io.File(indexFilename);
            if (indexFile.exists())
            {
                //if (indexFile.length()>0)
                //{
//                    System.out.println("DEBUG smail.localStore.mbox: Get a stored index file: "+indexFilename);
                    java.io.ObjectInputStream objIs = new java.io.ObjectInputStream(new java.io.FileInputStream(indexFile));
                    index = (Index)objIs.readObject();
                    objIs.close();
                //}
                //else index = new Index();
            }
            else
            {
                //System.out.println(indexFilename);
                new java.io.File(indexFilename).createNewFile();
                index = new Index(); // TODO: this belongs into makeIndex() or not?
                this.makeIndex();
                writeIndex();
            }

        }
        catch (java.lang.ClassNotFoundException e)
        {
            new java.io.File(indexFilename).createNewFile();
            index = new Index();
            this.makeIndex();
        }
    }
    
    private int countMessages(java.io.File MBoxFile) throws java.io.IOException
    {
        java.io.BufferedReader mboxBr1=new java.io.BufferedReader(new java.io.FileReader(mboxFile));
        String line;
        int messages=0;
        while ((line=mboxBr1.readLine())!=null) 
            if (line.startsWith("From "))
                messages++;
        //System.out.println(br.readLine());
        mboxBr1.close();
        return messages;
    }
    
    
    
    private void deleteMessage(int num) throws java.io.IOException, MessageNumberNotValidException
    {
        java.io.FileWriter fw = new java.io.FileWriter(mboxFile);
        java.io.BufferedReader mboxBr1 = new java.io.BufferedReader(new java.io.FileReader(mboxFile));
        if (num > index.index.size()) throw new MessageNumberNotValidException();
        int lineNumBegin = ((Integer)index.index.elementAt(num)).intValue();
        
        // last message deletion
        int lineNumEnd;
        if (num==index.index.size()) lineNumEnd = getLastLine();
        else lineNumEnd = ((Integer)index.index.elementAt(num+1)).intValue();
        
//        System.out.println(lineNumBegin+".."+lineNumEnd);
        
        String line;
        int lineNum=0;
        while ((line=mboxBr1.readLine())!=null)
        {
            if ((lineNum <= lineNumBegin) || (lineNum >= lineNumEnd))
            {
                fw.write(line+"\n");
                fw.flush();
            }
        lineNum++;
        }
        fw.close();
        mboxBr1.close();
    }
    
    private int getLastLine() throws java.io.IOException
    {
        java.io.BufferedReader mboxBr1=new java.io.BufferedReader(new java.io.FileReader(mboxFile));
        String line;
        int lineNum=0;
        while ((line=mboxBr1.readLine())!=null)
        {
            lineNum++;
        }
        mboxBr1.close();
        return lineNum;
    }

    /** get a single <CODE>Localmessage</CODE> indecated by a number.
     * @param num <CODE>Localmessage</CODE> number
     * @return the <CODE>Localmessage</CODE> will be returned in a String-Array for easy access to the gui.
     * The first String contains the date.
     * The second contains the from.
     * The third contains the subject.
     */    
    public String[] getMessage (int num)
    {
        String[] msgReturn = new String[3];
        IndexItem msg = (IndexItem)index.index.get(num);
        //System.out.println("Line 97: "+msg.date+" - "+msg.from+" - "+msg.lineNum+" - "+msg.subject);  //debug
        msgReturn[0] = msg.date;
        msgReturn[1] = msg.from;
        msgReturn[2] = msg.subject;
        return msgReturn;
    }
    
    // TODO: merge the code of both getMimeMessage methods and just use this signature.
    public LocalMessage getMimeMessage(int num) throws java.io.IOException {
      return getMimeMessage(num, session);
    }
    
    /** Get a single <CODE>LocalMessage</CODE> intecated by the MessageNumber.
     * @param num indecates the <CODE>LocalMessage</CODE>
     * @param s the session the <CODE>LocalMessage</CODE> belongs to.
     * @throws IOException will be thrown if something is wrong with accessing the harddisk-files.
     * @return Returns a <CODE>LocalMessage</CODE>
     */    
    public LocalMessage getMimeMessage(int num, javax.mail.Session s) throws java.io.IOException
    {
        try
        {
            java.io.BufferedReader mboxBr = new java.io.BufferedReader(new java.io.FileReader(mboxFile));
            IndexItem msg = (IndexItem)index.index.get(num);
//            System.out.println("getMimeMessage(): "+msg.date+" - "+msg.from+" - "+msg.lineNumStart+" - "+msg.subject);
            
            //jump to linenumber
            int toLineNum = msg.lineNumStart;//java.lang.Integer.parseInt(msg.lineNum);
            //System.out.println("Jump to line: "+toLineNum);
            for (int i=0; i<toLineNum; i++)
                //System.out.println("Jump: "+mboxBr.readLine());
                mboxBr.readLine();
                
            String text="";
            String line;
            java.io.ByteArrayInputStream ba;
            byte[] tempBa;
            
            while ((line=mboxBr.readLine())!=null && !(line.startsWith("From ")))
            {
                text=text+line+"\n";
            }
            //System.out.println("Text: "+text); //debug
            tempBa = text.getBytes();
            ba = new java.io.ByteArrayInputStream(tempBa);

            if(ba==null)
            {
            	System.out.println("MBOX: No Inputstream");
            }	
            if(this.folder==null)
            {
            	System.out.println("MBOX: No Current Folder");
            }	

            //contruct a new LocalMessage to get requiered date for Index
            LocalMessage lm = new LocalMessage(session, (java.io.InputStream)ba);
            lm.setMessageNumber(num+1);
            lm.saveChanges();
            
            mboxBr.close();
            return (lm);

        }
        catch (javax.mail.MessagingException e)
        {
            System.out.println(e);
        }
        return null;
    }
/*    
    
    LocalMessage convert(javax.mail.Session s)
    {
        java.util.Vector msgs = new java.util.Vector();
        java.io.BufferedReader mboxBr1 = new java.io.BufferedReader(new java.io.FileReader(mboxFilename));
        while (((line=mboxBr1.readLine())!=null) || (line.startWith("From ")))
        {
            java.io.InputStream is = new java.io.InputStream();
            javax.mail.internet.MimeMessage mm = new javax.mail.internet.MimeMessage(s, mboxBr1)
        }
    }
*/
/*    public LocalFolder getFolder (String folderName)
    {
        java.io.File file = new java.io.File(("smail"+java.io.File.separator+folderName));
        if (!file.exists()) throw new java.io.FileNotFoundException();
        java.io.FileReader fr = new java.io.FileReader();
        java.io.BufferedReader mboxBr = new java.io.BufferedReader(fr);
        
        String text="";
        String line;
        while (line=mboxBr.readLine())!=null)
            if (line.startsWith("From "))
            {
                while (!line.startsWith("From "))
                {
                    
            text = text+line;
            }
        byte[] tempBa = text.getBytes ();
        java.io.ByteArrayInputStream ba = new java.io.ByteArrayInputStream(tempBa);
        //java.io.StringReader textStream = new java.io.StringReader(text);
        return (new LocalMessage(s, ((java.io.InputStream)ba)));
        

    }
*/    
    
    
/*    void writeFolder (LocalFolder lf) throws java.io.IOException, javax.mail.MessagingException
    {
        String folderName = lf.getFullName();
        java.io.FileWriter fw = new java.io.FileWriter(new java.io.File((defaultPath+folderName)));
        
        LocalMessage[] msgs = (LocalMessage[])lf.getMessages();
        for (int i=0; i<msgs.length; i++)
        {
            fw.write("From "+msgs[i].getFrom().toString()+" "+msgs[i].getReceivedDate().toString());
            fw.write(msgs[i].toString());
            fw.flush();
        }
        fw.close();
    }
*/    
    /** Append new mails to mbox. Indexfile will be updated automatically.
     * @param msgs The Array of <CODE>LocalMessage</CODE>s to append.
     * @throws IOException Will be thrown if it's something wrong accessing the harddisk-files.
     * @throws MessagingException Something is wrong by creating a new <CODE>LocalMessage</CODE>.
     */    
    public void appendMessages(LocalMessage[] msgs) throws java.io.IOException, javax.mail.MessagingException
    {
        java.io.BufferedReader br = new java.io.BufferedReader(new java.io.FileReader(mboxFile));

        java.io.FileOutputStream fo = new java.io.FileOutputStream(mboxFile);
        java.io.OutputStreamWriter osw = new java.io.OutputStreamWriter(fo);
        
        String line;
        // TODO: try to use seek() here
        while ((line=br.readLine())!=null) // seek end of file
        {
            osw.write(line+"\n");
            osw.flush();
        }

        for (int i=0; i<msgs.length; i++)
        {
            //System.out.println("From "+msgs[i].getFrom().toString());
            //System.out.println(" "+msgs[i].getReceivedDate().toString());
            osw.write("From "+msgs[i].getFrom().toString()+" "+msgs[i].getReceivedDate().toString()+"\n");
            osw.flush();
            
            // TODO: filter this stream for lines starting with "From " and escaping
            //       them somehow
            msgs[i].writeTo(fo);
            fo.flush();
        }
        osw.close();
        fo.close();
        br.close();
        makeIndex();
    }
    
    /** Copy an Mbox physically.
     * @param filename the new filename of the copied mbox.
     * @throws IOException Will be thrown if it's something wrong accessing the harddisk-files.
     * @throws MessagingException Something is wrong by creating a new <CODE>LocalMessage</CODE>.
     */    
    public void copy(java.io.File filename) throws java.io.IOException, javax.mail.MessagingException
    {
        java.io.BufferedReader br = new java.io.BufferedReader(new java.io.FileReader(mboxFile));
        java.io.FileWriter fw = new java.io.FileWriter(filename);
        String line;
        while ((line=br.readLine())!=null)
        {
            fw.write(line+"\n");
            fw.flush();
        }
        br.close();
        fw.close();
    }
    
    
    private void showIndex()
    {
        IndexItem item;
        for (int i=0; i<index.index.size(); i++)
        {
            item = (IndexItem)index.index.get(i);
//            System.out.println("Index-Item at index "+i+": "+item.date+" "+item.from+" "+item.lineNumStart+" "+item.subject);
        }
    }
    
    private void makeIndex()
    {
        try
        {
//            if (mboxFilename.length()>0)
 //           {
            java.io.BufferedReader mboxBr = new java.io.BufferedReader(new java.io.FileReader(mboxFile));
            String text="";
            String line;
            int endOfWord;
            String fromAddress;
            String from = "";
            String date;
            String subject = "";
            boolean attachment = false;
            int lineNum=0;
            int startLineNum;
            int endLineNum;
            
            line=mboxBr.readLine();
            while (line!=null)
            {
                lineNum++;
                //System.out.println(lineNum+"-- "+line);
                if (line.startsWith("From "))
                {
                    endOfWord = line.indexOf(" ",5);
                    fromAddress = line.substring(5,endOfWord);
                    date = line.substring(endOfWord+1, line.length()-1);
                    startLineNum = lineNum;

                    while ((line=mboxBr.readLine())!=null && !(line.startsWith("From ")))
                    {
                        //System.out.println(lineNum+": "+line);
                        lineNum++;
                        if (line.startsWith("Subject: ")) subject = line.substring(9);
                        if (line.startsWith("From: ")) from = line.substring(6);
                        if (line.startsWith("Content-Type: ")) if (line.substring(14).startsWith("Multipart")) attachment = true;
                        //text=text+line+"\n";
                    }
                    endLineNum=lineNum-1;
                    
                    //now all date is here to make an new IndexItem
                    IndexItem item = new IndexItem(from, date, startLineNum, endLineNum, subject, attachment);
                    index.index.add(item);
                    //System.out.println("Added Mimemessage to Index at Number "+(index.size()-1)); //debug
                }
            }
            writeIndex();
            mboxBr.close();
        }
        catch (java.io.FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (java.io.IOException e)
        {
            e.printStackTrace();
        }
        catch (java.lang.IndexOutOfBoundsException e)
        {
            System.out.println("ERROR: MBox format error. Form title not correct.");
            e.printStackTrace();
        }
        
    }
    
    /** Writes mbox from the indexfile to the disk.
     * This is needed because if you delete a message in the folder, the message is only deleted in the indexfile and nothing is written on disk. After work it is needed to rewrite the mbox to the harddisk.
     * @throws IOException Will be thrown if it's something wrong accessing the harddisk-files.
     * @throws FileNotFoundException File to write not found.
     */    
    public void reWriteMbox() throws java.io.IOException, java.io.FileNotFoundException
    {
        java.io.FileWriter fw = new java.io.FileWriter(mboxFile);
        //java.io.BufferedReader mboxBr1 = new java.io.BufferedReader(new java.io.FileReader(mboxFilename));
        java.io.BufferedReader mboxBr1;
        int streamCurrentLine=0;
        
        String line;
        for (int i=0; i<index.index.size()-1;i++)
        {
            if (!(((IndexItem)index.index.get(i)).isDeleted))
            {
                mboxBr1 = new java.io.BufferedReader(new java.io.FileReader(mboxFile));

                for (streamCurrentLine=0; streamCurrentLine<((IndexItem)index.index.get(i)).lineNumStart-1; i++)
                    line=mboxBr1.readLine();
                while ((line=mboxBr1.readLine())!=null || streamCurrentLine < ((IndexItem)index.index.get(i)).lineNumEnd)
                {
                    fw.write(line+"\n");
                }
                mboxBr1.close();
            }
        }
        fw.flush();
        fw.close();
    }
    
    void writeIndex() throws java.io.FileNotFoundException, java.io.IOException
    {
            java.io.ObjectOutputStream objOs = new java.io.ObjectOutputStream(new java.io.FileOutputStream(new java.io.File(indexFilename)));
            objOs.writeObject(index);
    }
    
    public int getMessageCount() {
      return index.getNotDeletedItemCount();
    }
    
    public void deleteItem(int num)
    {
        ((IndexItem)index.index.get(num)).delete();
    }
    
    protected boolean deleteMBox() {
      return (mboxFile.delete() &&
              new java.io.File(indexFilename).delete() &&
              mboxFile.getParentFile().delete()
              );
    }
    
    /** Only for testing and debugging.
     * @param args nothing
     */    
    public static void main(String[] args)
    {
        try
        {
            java.io.File newFile = new java.io.File("Mail"+java.io.File.separator+"inbox_uni");

            java.util.Properties props = System.getProperties() ;
            javax.mail.Session session = javax.mail.Session.getDefaultInstance(props,null);
            //System.out.println(newFile.getAbsoluteFile());
            MBox bla = new MBox(newFile, session, null);

            //get a Message from index
            String[] strTemp = bla.getMessage(5);
//            System.out.println("returned message from getMessage(..): "+strTemp[0]+" - "+strTemp[1]+" - "+strTemp[2]);

            //look that index is correct
            bla.showIndex();

            //get a complete LocalMessage
            LocalMessage lm = bla.getMimeMessage(4, session);
//            System.out.println("main.getContent(): "+lm.getContent());
            
            //append a new Message
            LocalMessage[] lmArray = new LocalMessage[1];
            lmArray[0] = lm;
            //bla.appendMessages(lmArray);
            
            bla.writeIndex();
            
            //look that index is correct
            //bla.showIndex();
            
            //delelte the first
            //bla.deleteMessage(0);
            
            //look that index is correct
            //bla.showIndex();
            

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
}

class IndexItem implements java.io.Serializable
{
    String from;
    String date;
    int lineNumStart;
    int lineNumEnd;
    String subject;
    boolean attachment;
    boolean isDeleted;
    //String msgid;
    
    IndexItem(String from, String date, int lineNumStart, int lineNumEnd, String subject, boolean attachment)//, String msgid)
    {
        this.from = from;
        this.date = date;
        this.lineNumStart = lineNumStart;
        this.lineNumEnd = lineNumEnd;
        this.subject = subject;
        this.attachment = attachment;
        this.isDeleted = false;
    }
    
    public boolean isDeleted() {
      return isDeleted;
    }
    
    void delete()
    {
        isDeleted=true;
    }
}

class Index implements java.io.Serializable
{
    java.util.Vector index;
    
    Index()
    {
        index = new java.util.Vector();
    }
    
    protected int getNotDeletedItemCount() {
      int x = 0;
      java.util.Iterator it = index.iterator();
      while (it.hasNext()) {
        IndexItem ii = (IndexItem)it.next();
        if ( ! ii.isDeleted()) x++;
      }
      return x;
    }
}

// TODO: this should be removed since the mail api throws IndexOutOfBoundsException
//       in those cases.
class MessageNumberNotValidException extends Exception {}
