package smail.server ;

import java.util.Properties ;
import smail.utils.*  ;
/**
 *This class restores the accounts from propertiesmanager and stores accounts as
 *properties in the propertiesmanager(smail.properties file) . Note that this class
 *is no singleton so it is possible to have more than one instance of this class.
 *This is difficult because all instances will work with the same restored account.
 *Accounts are stored as (example with X as a number):<br><CODE>
 *	KEY							VALUE
 *smail.accountName.X		swp04_test_account
 *smail.username.X			swp04@swp.inf.fu-berlin.de
 *smail.pop3.X				swp.inf.fu-berlin.de
 *smail.smtp.X				swp.inf.fu-berlin.de
 *</CODE>
 */
public class AccountData // implements Configurable
{
	/**
	 *This holds the accounts. This is set to an account array with length 0 to assure 
	 *a non-null value and assure better handling.
	 */
	private static Account[] accounts = new Account[0] ;
	/**
	 *This field is little bit useless and was know how many accounts exists. The value
	 *of this field should be the same as <CODE>accounts.length</CODE>.
	 */
	private int amount = 0;

	/**
	 *This will initialize an instance ...
	 *First the property "smail.account.number" is get from @see smail.utils.ProertiesManager.
	 *Based on this value the accounts are read from @see smail.utils.ProertiesManager
	 *retrieving the appropiate properties (see above in class desription).
	 *After reading all Accounts they are available during runtime from this class.
	 */
	public AccountData()
	{
		String amountString = PropertiesManager.get("smail.account.number") ;
		if (amountString == null)
		{
			System.out.println( "No Accounts defined yet !" ) ;
		}
		else
		{
			try
			{
				amount = Integer.parseInt(amountString) ;
			}
			catch (NumberFormatException nfexc)
			{
				System.out.println( "malformed account number can not be parsed !") ;
				System.out.println( nfexc.getMessage() ) ;
			}
			String accountName ;	/* the username of an account */
			String smtpHost ;		/* the SMTP-server to get/send mails */
			String pop3Host ;		/* the POP3-server to get mails from */		
			String username ;		/* name to identify with on mail server */
			String address ;		/* e-mail address */
			accounts = new Account[amount] ;
//OLDCODE
//			for ( ; amount > 0  ;amount-- )
//			{
//				accountName = PropertiesManager.get("smail.account."+amount) ;
//				smtpHost = PropertiesManager.get("smail."+accountName+".smtp") ;
//				pop3Host = PropertiesManager.get("smail."+accountName+".pop3") ;
//				username = PropertiesManager.get("smail."+accountName+".address") ;
//				accounts[amount-1] = new Account(username,null,null
//													,null,smtpHost,pop3Host,accountName) ;
//			}

			for (int i = 0 ; i<amount  ;i++ )
			{
				accountName = PropertiesManager.get("smail.account."+i+".accountName") ;
				smtpHost = PropertiesManager.get("smail.account."+i+".smtp") ;
				pop3Host = PropertiesManager.get("smail.account."+i+".pop3") ;
				username = PropertiesManager.get("smail.account."+i+".username") ;
				address = PropertiesManager.get("smail.account."+i+".address") ;
				accounts[i] = new Account(accountName,address,username,smtpHost,pop3Host) ;
			}
		}
	}
 
 	/**
	 *returns an array of Account-object representing the accounts defined by the user.
	 *If no accounts are defined an Array with length 0 is returned.
	 */
 	public Account[] getAllAccounts()
	{
			return(accounts) ;
	}
	
	/**
	 *This method is deprecated and not functional, use @see #changeAccount() instead.
	 *will change the SMTP field in the passed account object. Dont think about changing
	 *this field directly in the account-object because this changes may not be stored.
	 *@param acc is the account to change the smtpoHost field in.
	 *@param newSmtp is the new smtp mail-host address.
	 */
	public void changeSMTP(Account acc , String newSmtp)
	{
		storeAccountsToProperties() ;
	}

	/**
	 *This method is deprecated and not functinal, use @see #changeAccount() instead.
	 *will change the pop3Host field in the passed account object. Dont think about changing
	 *this field directly in the account-object because this changes may not be stored.
	 *@param acc is the account to change the smtpoHost field in.
	 *@param newPop is the new POP3 mail-host address.
	 */
	public void changePop3(Account acc , String newPop)
	{
		storeAccountsToProperties() ;
	}

	/**
	 *changes ALL account informations of the account (except accountName) 
	 *identified by accountName. All parameters are used to change the 
	 *coresponding field in the matching account object. If null is passed in one 
	 *parameter the cresponding field is not updated. Rememer : an empty string i not 
	 *null.
	 *@see smail.server.Account
	 */
	public void changeAccount(String accountName,String address,String username,String smtp,String pop3)
	{
		if (accountName == null)
		{			
			return  ;
		}
		boolean found = false ;
		
		/* only find the account and change it */
		for (int i = 0 ;i<amount && !found ;i++ )
		{
			if (accounts[i].accountName.equals(accountName))
			{

				found = true ;
				if (address != null)
					accounts[i].address = address ;
				if (username != null)
					accounts[i].username = username ;
				if (smtp != null)
					accounts[i].smtpHost = smtp;
				if (pop3 != null)
					accounts[i].pop3Host = pop3;
			}
		}
		storeAccountsToProperties();
	}
	
	/**
	 *will remove the account with the account name accountName from the 
	 *account-object-list. If the accountName is not found NO exception is thrown
	 *and no change is made.
	 *@param accountName is the name of the account to remove from the accountlist.
	 */	 
	public void remove(String accountName)
	{
		//TODO : remove an account not just from array at runtime
		boolean found = false ;
		for (int i = 0 ; !found && i<amount ; i++ )
		{
			if (accounts[i].accountName.equals(accountName))
			{
				found = true ;
				Account[] temp = new Account[accounts.length-1] ;
				
				/* the next two lines will remove the account at index i */
				System.arraycopy(accounts,0 , temp, 0, i) ;
				System.arraycopy(accounts, i+1, temp, i, accounts.length-i-1) ;
				
				accounts = temp ;
				amount = accounts.length ;
				
				// delete from properties
				PropertiesManager.set("smail.account."+i+".accountName","");
				PropertiesManager.set("smail.account."+i+".username","");
				PropertiesManager.set("smail.account."+i+".address","");
				PropertiesManager.set("smail.account."+i+".smtp","");
				PropertiesManager.set("smail.account."+i+".pop3","");
//				PropertiesManager.set("smail.account."+i+".accountName","");
				
			}
		}
		storeAccountsToProperties() ;
/*DEBUG*/		System.out.println( "AccountData.java:180 :: removed account " ) ;
	}

	/*
	 *this stores all accounts currently hold as objects into proerties with the
	 *service of PropertiesManager. This updates the properties of all accounts.
	 */
	private void storeAccountsToProperties()
	{
		//TODO : store the accounts as properties with propertiesManager

		PropertiesManager.set("smail.account.number", accounts.length+"") ;
		int i = 0 ;
		for (; i<accounts.length ; i++ )
		{
			PropertiesManager.set("smail.account."+i+".accountName", accounts[i].accountName);
			PropertiesManager.set("smail.account."+i+".address", accounts[i].address) ;
			PropertiesManager.set("smail.account."+i+".username", accounts[i].username) ;
			PropertiesManager.set("smail.account."+i+".smtp", accounts[i].smtpHost) ;
			PropertiesManager.set("smail.account."+i+".pop3", accounts[i].pop3Host) ;
		}
		/* now setting all unneeded account properties to zero string */
		String removed ;
		i++ ;
		while ((removed = PropertiesManager.get("smail.account."+i+".accountName")) != null )
		{
			PropertiesManager.set("smail.account."+i+".accountName", "") ;
			PropertiesManager.set("smail.account."+i+".address", "") ;
			PropertiesManager.set("smail.account."+i+".username", "") ;
			PropertiesManager.set("smail.account."+i+".smtp", "") ;
			PropertiesManager.set("smail.account."+i+".pop3", "") ;
			i++ ;	
		}
	}
	
	
	/**
	 *this adds an account to the Accounts field and to the properties file of the program.
	 *Accounts are identiefied their accountName so an exception is thrown when the 
	 *accountName of the new account is allready bound to another account.
	 *@throws InvalidAccountException when the accountName already exists.
	 *@param name is the username(email-address) of the account.
	 *@param smtpHost is the host address where to send mails to.
	 *@param pop3Host is the host address where to receive mails from.
	 *@param accountName is the string to identify an account on.
	 */
	public void addAccount(String accountName, String address ,String username,  String smtpHost, String pop3Host ) throws InvalidAccountName
	{
//		if(accounts!=null)
//		{
			for (int i = 0 ; i<accounts.length ; i++ )
			{
				if (accounts[i].accountName!=null && 0 == accounts[i].accountName.compareTo(accountName))
				{
					throw new InvalidAccountName() ;
				}
			}
			Account newAcc = new Account(accountName, address , username, smtpHost, pop3Host ) ;
			Account[] temp = new Account[accounts.length+1] ;
			System.arraycopy(accounts,0,temp,0,amount) ;
			temp[accounts.length] = newAcc ;
			accounts = temp ;
			amount = accounts.length ;
			storeAccountsToProperties() ;
//		}
//		else
//		{
//			accounts = new Account[1];
//			accounts[0] = new Account(accountName, address , username, smtpHost, pop3Host ) ;
//		}
	}
}