package smail.utils ;

import java.util.Properties ;
import java.util.Vector ;
import smail.Control ;
import java.io.File ;
import java.io.FileOutputStream ;

/**
This class makes a first suggestion about handling our properties during runtime
and to make properties persistent and available in more than one session.
properties are stored in a file called <code>smail.properties</code> in the directory
spezified by the VM and the system property <code>user.home</code>.
Who does not know the value of this property may run the main-method of this class
and look for the line beginning with <code>user.home is:</code>.<br>
some examples :<p>
A class M will store some properties a,b and c with values 1,2 and 3.
now this class has to implement the interface Configurable to assure the
implementation of the method propertyChanged(PropertyChangedEvent pEvent)
<pre>
class M imlements Configurable
{
	...
	public void propertyChanged(PropertyChangedEvent pEvent)
	{
		//here follows the code handling the event
	}
	...
	//next the class must add itself to the property listeners in the 
	//PropertiesManager class :
	public M()
	{
		PropertiesManager.addPropertyListener(this) ;
		...
		//and it has to get all needed properties from the PropertiesManager
		//in the constructor to get them at instanciation-time.
		String xyzValue = PropertiesManager.get("smail.xyz") ;
		...
	}	
}
</pre>
The last step is important to initialize all fields related to some property.
It is possible to get properties without beeing notified about a change.
It is possible to change a property with the set()-method.If a property changes 
the PropertiesManager will call the propertyChanged-method with a 
PropertyChangedEvent as argument on every object registered as a listener. The 
PropertyChangedEvent contains key and value of the changed property. Note that
every changed property will cause to tell the change to every listener. It is 
expected that every listener cares for what he is interested in and what not and 
to react in a proper way like update all fields related to this property.<br>
Let us take a look on the propertyChanged(..)-method :<br>
<pre>
	public void propertyChanged(PropertyChangedEvent pEvent)
	{
		//the object is registered as a listener and is interested in the property
		//"smail.xyz"
		if (0 == pEvent.getKey().compareTo("smail.xyz"))
		{
			String value = pEvent.getValue() ;
			...//this code handles the distribution of the value in this class
		}
		else if (...)//other properties can be catched too
		{
			...
		}
		else
			return ;//do nothing if not interested in this property
	}
</pre>
This is only an example.
<pre>
</pre>
<pre>
</pre>
*/
public class PropertiesManager
{
	private static boolean exists = false ;
	private static Properties props = new Properties() ;
	private static Vector listeners = new Vector() ;
	private static PropertiesManager propMan = new PropertiesManager() ;
	//after the first instantiation this field will point to the property
	//used in this session
	private java.io.File propertyFile ;
	
	/**
	 *is used to initialize the instance on the first request. This version allows 
	 *only the property file in the "user.home"/smail directory. Further release may
	 *implement some mechanism to load a user defined property file.
	 */
	private PropertiesManager()
	{
//		if (0 == path.compareTo("default"))//load the default property file
//		{
			propertyFile = getDefaultPropertyFile() ;
//			if (Control.d)
//				System.out.println( "Default PropertyFile will be used" ) ;
//		}
//		else//load the spezified propertyfile
//		{
//			try
//			{
//				propertyFile = File(path) ;
//				if (!propertyFile.exists())
//					propertyFile = getDefaultPropertyFile() ;
//			}
//			catch (java.io.IOException ioexc)
//			{
//				System.out.println( "File not found ! Message is :" ) ;
//				System.out.println( ioexc.getMessage() ) ;				
//			}
//		}
		//some file is loaded now get the properties
		java.io.FileInputStream finstr = null ;//the scope of view forces to do this
		try
		{
			finstr = new java.io.FileInputStream(propertyFile) ;
			props.load((java.io.InputStream)finstr) ;
		}
		catch (java.io.FileNotFoundException fnfexc)
		{
			System.out.println( "serious error : property file not found" ) ;
			System.out.println( "Session properties can not be stored !!!" ) ;			
		}
		catch (java.io.IOException ioexc)
		{
			System.out.println( "serious error : properties could not load" ) ;
			System.out.println( "Session properties can not be stored !!!" ) ;			
		}
		//this extra try/catch is to separate the not so serios of being unable
		//to close an OutputStream
		try
		{
			finstr.close() ;
		}
		catch (java.io.IOException ioexc)
		{
			System.out.println( "file stream can not be closed !" ) ;
		}
		if ( ! props.getProperty("smail.isdefault").equals("2")) {
			props = DefaultProperties.getDefaults() ;
			System.out.println( "XX" ) ;
		}
		
		/* this sets the path of the smail-jar directory at runtime */
//		getRuntimeSmailPath() ;
		//i do not really know if this will hold the object in memory
		//This gives the chance to save our properties to the property file
//		System.runFinalization() ;
	}

//	private void getRuntimeSmailPath()
//	{
//		String smailRuntimePath = this.getClass().getResource("../../").getPath() ;
//		props.setProperty("smail.runtime.home",smailRuntimePath) ;
//	}

	
	/**
	 *returns the default property file placed in the directory named by the 
	 *system property "user.dir" plus the directory "smail". If the file do not
	 *exists all missing directories between the root and the smail directory and
	 *the smail directory itself will be created. Additionally after running this
	 *program the file "smail.properties" is placed in the smail directory.
	 */
	private java.io.File getDefaultPropertyFile()
	{
		File defDir = new File(System.getProperty("user.home"),"smail") ;

		if (defDir.exists())//directory exists so go and get the file
		{
			File defFile = new File(defDir,"smail.properties") ;

			if (defFile.exists())//file exists so we are finish
			{
				return(defFile);
			}
			else//no file so create any missing part until the file itself
			{
				try
				{
					java.io.FileOutputStream fileOut = new java.io.FileOutputStream(defFile);
					Properties defProps = DefaultProperties.getDefaults() ;
					defProps.store(fileOut,"temporarily property handling") ;
					defProps = null ;
					fileOut.flush() ;
					fileOut.close() ;
				}
				catch (java.io.FileNotFoundException fnfexc)
				{
					System.out.println( "default property file could not be created " ) ;
				}
				catch (java.io.IOException ioexc)
				{
					System.out.println( "no flushing / closing " ) ;
				}
				//this is dirty : defFile does not exist if an exception was 
				//caught above but what else should we do here ?
				return(defFile);				
			}	
		}//some directies do not exist so create them
		boolean success = defDir.mkdirs() ;
		if (success)
		{
			System.out.println( "missing dirs created" ) ;
			
			//make the property file
			File defFile = new File(defDir,"smail.properties") ;
			try
			{
				java.io.FileOutputStream fileOut = new java.io.FileOutputStream(defFile);
				Properties defProps = DefaultProperties.getDefaults() ;
				defProps.store(fileOut,"temporarily property handling") ;
				defProps = null ;
				fileOut.flush() ;
				fileOut.close() ;
			}
			catch (java.io.FileNotFoundException fnfexc)
			{
				System.out.println( "default property file was not found " ) ;
			}
			catch (java.io.IOException ioexc)
			{
				System.out.println( "no flushing closing " ) ;
			}
			//this is dirty : defFile does not exist if an exception was 
			//caught above
			return(defFile);				
		}
		else
		{
			//without any property file the programm can not run properly
			System.out.println( "can not create missing directories" ) ;
			System.exit(-1) ;
		}
		// ??? needed to compile
		return null ;
		
	}

	/**
	 *This returns the default and only instance of this class. Maybe this is a to
	 *complicated solution of such a little problem. The argument defines the 
	 *destination (as a string representation of a path name) of an user propery 
	 *file that resides not in the "user.home" directory.
	 */
/*	public static PropertiesManager getInstance(String path) throws java.io.FileNotFoundException
	{
		if (exists)
			return(propMan);
		else
			return(new PropertiesManager(path));		
	}
*/
	/**
	 *This returns the default and only instance of this class. Maybe this is a to
	 *complicated solution of such a little problem.
	 */
	public static PropertiesManager getInstance()
	{
		if (propMan == null )
			System.out.println( "PropertiesManager is null" ) ;
		return(propMan);
	}	
	
	/**
	 *This adds a Configurable-object to the list of objects interested in properties 
	 *and their change. The idea is based upon the AWT/swing event-system.
	 */
	public static void addPropertiesListener(Configurable conf)
	{
		for (int i = 0 ;i < listeners.size() ;i++ )
		{
			Configurable listenerObject = (Configurable)listeners.get(i) ;
			if (listenerObject.equals(conf))
				return ;
		}
		listeners.add(conf) ;
	}
	
	/**
	 *Sets a property and notifies all listeners about the change of the
	 *set property. The origin of the change will be notified too. (?!)
	 */ 
	public static void set(String key , String value)
	{
//		if (key.equals("smail.filter.count"))
//		System.out.println( "filter count set" ) ;
		//store the new or changed key/value pair in the properties-object
		props.setProperty(key,value) ;
		//now all listeners have to be notified about the change
		//so traverse or iterate through all listeners
		java.util.Iterator it = listeners.iterator() ;
		while (it.hasNext())
		{
			Configurable nextListener = (Configurable)it.next() ;
			nextListener.propertyChanged(new PropertyChangedEvent(key,value)) ;
		}
	}
	/**
	 *Returns the value of the spezified property-key. If the key is not found then
	 *it will return null just like the "real"getProperty(..) method in the class
	 *java.util.Properties .
	 */
	public static String get(String key)
	{
		return(props.getProperty(key));
	}
	
	/**
	 *this garantees that our properties are stored before the VM quits.
	 *The properties are stored in the default file in the "user.home"/smail
	 *directory.
	 */
	public void exiting()
	{
		removeEmptyEntries() ;
		try
		{
			FileOutputStream fileOut = new FileOutputStream(propertyFile) ;
			props.store((java.io.OutputStream)fileOut,"smail property file") ;
			fileOut.flush() ;
			fileOut.close() ;
			System.out.print( "properties stored to : " ) ;
			System.out.println( propertyFile.getPath() ) ;
		}
		catch (java.io.FileNotFoundException fnfexc)
		{
			System.out.println( "serious error : can not store the proerties !!!" ) ;
		}
		catch (java.io.IOException ioexc)
		{
			System.out.println( "can not flush/close the stream" ) ;
		}		
	}	
	
	private void removeEmptyEntries()
	{
		java.util.Enumeration propEnum = props.propertyNames() ;
		Properties newProps = new Properties() ;
		String key ;
		int counter = 0;
		while (propEnum.hasMoreElements())
		{
			key = (String)propEnum.nextElement() ;
			if ( ! props.getProperty(key).equals(""))
			{
				newProps.setProperty(key,props.getProperty(key)) ;
			}
			else
			{
				counter++ ;
			}
		}
		props = newProps ;
	}
	
	//TEST
	/**
	 *test the class and gives a small working exmple of this class.
	 *It will also print the specific user.home-directory path.
	 */
	public static void main(String[] arg)
	{	
		Vector t = new Vector() ;
		System.out.println( "user.home is : "+System.getProperty("user.home") ) ;
		System.out.println( "user.dir is :"+System.getProperty("user.dir")  ) ;
		System.out.println( "now testing some mechanism to get resources from differetn places" ) ;
		java.net.URL testurl = t.getClass().getResource("../../some.property") ;
		if (testurl == null)
		{
			System.out.println( "testurl ist null !") ;
		}
		else
		{
			try
			{
				System.out.println( "testurl toStirng is :"+testurl.toString() ) ;
				System.out.println( "testurl getFile is :"+testurl.getFile() ) ;
				System.out.println( "testurl getPath is :"+testurl.getPath() ) ;
				java.io.FileInputStream fin = new java.io.FileInputStream( new java.io.File( testurl.getPath()) );
				java.io.InputStreamReader reader = new java.io.InputStreamReader(fin) ;
				java.io.BufferedReader buffer = new java.io.BufferedReader(reader) ;
				String line = buffer.readLine() ;
				System.out.println( "the received line is :"+line ) ;
			}
			catch (java.io.IOException e)
			{
				System.out.println( "IOException in here" ) ;
			}
		}
//		smail.utils.PropertiesManager p = smail.utils.PropertiesManager.getInstance() ;
		String name = smail.utils.PropertiesManager.get("smail.server.smtp") ;
		System.out.println("result : "+ name ) ;
		String teststring = "" ;
		System.out.println( teststring.equals("")+":  :"+(0 == teststring.compareTo(""))) ;
		System.out.println( "kein key gleich null ? "+(props.getProperty("sadgvre") == null) ) ;
		int i ;
		for (i = 0 ;i<10 ;i++ )
		{
			System.out.println( i ) ;
		}
		System.out.println("outside :"+ i ) ;
//		p.set("smail.localstore.mboxpath",System.getProperty("user.home")+"\\smail\\mbox") ;
//		p.set("smail.account.number","1") ;
//		p.set("smail.account.1","swp06@swp.inf.fu-berlin.de") ;
//		p.set("smail.swp06@swp.inf.fu-berlin.de.smtp", "swp.inf.fu-berlin.de") ;
//		p.set("smail.swp06@swp.inf.fu-berlin.de.pop3", "swp.inf.fu-berlin.de") ;
//		p.exiting() ;
	}

}

